#include "config.h"

#include "common.hpp"

#include "config.hpp"
#include "themes.hpp"
#include "simplefile.hpp"
#include "updater.hpp"
#include "print.hpp"
#include "renderer.hpp"
#include "plugins.hpp"

#include <iostream>
#include <sstream>
#include <algorithm>
#include <cmath>

#include <unistd.h>

// file exists 
#include <sys/stat.h>

#include "gettext.hpp"

// conversion
#ifdef use_iconv
#include <iconv.h>
#include <langinfo.h>
#endif

#include <cassert>

using std::string;
using std::list;

// check if the current file matches one of the types defined 
// in the configuration file pairs
MyPair check_type(const string& s, const list<MyPair>& temp)
{
  for (list<MyPair>::const_iterator iter = temp.begin(); iter != temp.end(); ++iter)
    if (string_format::lowercase(s.substr(s.size() - (iter->first.size() + 1))) == "." + iter->first)
      return (*iter);
  return emptyMyPair;
}

int conv::atoi(const string& s)
{
  return ::atoi(s.c_str());
}

string conv::itos(int i)
{
  std::stringstream s;
  s << i;
  return s.str();
}

float conv::atof(const string& s)
{
  return ::atof(s.c_str());
}

string conv::ftos(float f)
{
  std::stringstream s;
  s << f;
  if (int(f) == f)
    s << ".0";
  return s.str();
}

bool conv::stob(const string& s)
{
  return !(s == "0" || s == "f" || s == "n" ||
	   s == "false" || s == "no" || s == "off" ||
	   s == gettext("false") || s == gettext("no") || s == gettext("off"));
}

string conv::btos(bool b)
{
  return (b) ? "true" : "false";
}

string string_format::unique_folder_name(const string& folder)
{
  string final = remove_doubles(folder);
  if (final[final.size()-1] != '/')
    final += '/';
  return final;
}

void string_format::format_to_size(string& s, const string& font, int size, bool dots, 
				   bool long_string)
{
  int x = string_format::calculate_string_width(s, font);

  if (x > size) {
    string test = s;
    int chars = int(size/(double(x)/s.size()));

    if (chars == 0) {
      s = "";
      return;
    }
    
    assert(chars > 0);

    if (dots)
      pretty_print_string(test, chars);
    else
      test = test.substr(0, chars);

    x = calculate_string_width(test, font);

    while (x < int(size*0.97)) {

      if (chars == s.size())
	break;

      test = s;
      ++chars;

      if (dots)
	pretty_print_string(test, chars);
      else
	test = test.substr(0, chars);

      x = calculate_string_width(test, font);
    }

    while (x > int(size*1.03)) {

      if (chars == 0) {
	test = "";
	break;
      }

      test = s;
      --chars;

      if (dots)
	pretty_print_string(test, chars);
      else
	test = test.substr(0, chars);

      x = calculate_string_width(test, font);
    }

    if (long_string)
      s = test.substr(0, test.size()-2); // utf-8 fix
    else
      s = test;
  } else {
    if (dots)
      pretty_print_string(s, s.size());
  }
}

void string_format::pretty_print_string(string& s, int maxlen)
{
  int dif=s.length()-maxlen;
  if (dif > 0) s.replace((s.length()>>1)-(dif>>1), dif+1, "..." );

  for (string::size_type i=0; (i=s.rfind("_")) != string::npos; s.replace(i,1," "));
}

string string_format::pretty_print_string_copy(const string& s, int maxlen)
{
  string result = s;
  pretty_print_string(result, maxlen);
  return result;
}

string string_format::escape_string(const string& input)
{
  string result = input;

  for (int i = 0, size = result.size(); i < size; ++i)
    if (result[i] == '\'') {
      result.replace(i, 1, "''");
      ++i;
    }

  return result;
}

string string_format::remove_doubles(const string& doubles)
{
  string temp = doubles;

  string::size_type pos = temp.rfind("//");

  while (pos != string::npos) {
    temp.replace(pos, 2, "/");
    pos = temp.rfind("//");
  }

  return temp;
}

string string_format::lowercase(string s) {
  transform(s.begin(), s.end(), s.begin(), tolower);
  return s;
}

string string_format::trim(string s)
{
  if (s.empty())
    return s;

  while (s[0] == ' ' || s[0] == '\t' || s[0] == '\n')
    if (s.size() == 1)
      return "";
    else
      s = s.substr(1);

  while (s[s.size()-1] == ' ' || s[s.size()-1] == '\t' || s[s.size()-1] == '\n')
    s = s.substr(0, s.size()-1);
  
  return s;
}

int string_format::calculate_string_width(const string& text, const string& font)
{
  int text_w;

  if (S_Render::get_instance()->set_font(font)) {
    // " " is there to compensate for imlib fix when drawing
    imlib_get_text_size((text + " ").c_str(), &text_w, 0);
    imlib_free_font();
  }

  // during rendering we add black borders
  text_w += 2;
  
  return text_w;
}

std::pair<int, int> string_format::calculate_string_size(const string& text, const string& font)
{
  int text_w, text_h;

  if (S_Render::get_instance()->set_font(font)) {
    // " " is there to compensate for imlib fix when drawing
    imlib_get_text_size((text + " ").c_str(), &text_w, &text_h);
    imlib_free_font();
  }
  
  // during rendering we add black borders
  text_w += 2;
  text_h += 2;

  return std::make_pair(text_w, text_h);
}

string string_format::get_line(const string& text, int &pos, int width, const string& font)
{
  assert(text.find('\n') == string::npos);

  bool padding = false;

  string desc = text.substr(pos);
  string last_desc = desc;

  string_format::format_to_size(desc, font, width, false, true);

  // check the end of the string
  if (desc.size() != last_desc.size() && desc.size() < last_desc.size() 
      && last_desc.size() >= 2) {
    char next_char = last_desc[desc.size()];
    char last_char = last_desc[desc.size()-1];
    if (next_char == ' ' || next_char == ',' || next_char == '.') {
      desc += next_char;
      if (last_desc[desc.size()] == ' ')
	desc += last_desc.substr(desc.size(), 1);
    } else if (last_char != ' ' && last_char != ',' && last_char != '.') {

      // if last word was small to not try to split it
      // or if you just get one char of a new word then don't try to
      // break the word onto multiple lines
      int first_space = desc.rfind(" ");

      if (first_space != string::npos) {

	string rest_of_string = last_desc.substr(desc.size());

	int last_space = rest_of_string.find(" ");
	int last_dot = rest_of_string.find(".");

	if (last_dot != string::npos) {
	  if (last_space == string::npos)
	    last_space = last_dot;
	  else
	    if (last_space > last_dot)
	      last_space = last_dot;
	} else if (last_space == string::npos)
	  last_space = 0;

	int last_word_size = desc.size() - (first_space+1) + last_space;

	if (last_word_size < 4 || first_space + 2 >= desc.size())
	  desc = last_desc.substr(0, first_space+1);
	else {
	  desc += "-";
	  padding = true;
	}

      }
    }
  }

  int size = desc.size();
  if (padding)
    --size;

  while (mblen(text.substr(pos+size).c_str(), 10) == -1) {
    //utf8 char cut off

    // increase string size
    ++size;
    
    desc = text.substr(pos, size);
    if (padding)
      desc += "-";
  }

  pos += size;

  return desc;
}

int string_format::difference(const string& lhs, const string& rhs)
{
  int min_size = std::min(lhs.size(), rhs.size());

  int difference = 0;

  for (int i = 0; i < min_size; ++i)
    if (lhs[i] != rhs[i])
      ++difference;

  return difference + std::abs(int(lhs.size()-rhs.size()));
}

string string_format::greatest_common_substring(const string& lhs, const string& rhs)
{
  const int shortest = std::min(lhs.size(), rhs.size());

  string result = "";
  
  for (int i = 0; i < shortest; ++i)
    if (lhs[i] == rhs[i])
      result += lhs[i];
    else
      break;
  
  return result;
}

string string_format::convert(const string& in, bool mp3_locale)
{
#ifdef use_iconv
  Config *conf = S_Config::get_instance();

  string from = conf->p_convert_locale();
  if (mp3_locale)
    from = conf->p_convert_mp3_locale();

  const char *fromencoding = from.c_str();
  const char *toencoding = "UTF-8";

  if (strcmp(fromencoding, toencoding) != -1 || in.empty())
    return in;

  size_t insize = in.length();

  char *inbuf = const_cast<char*>(in.data());

  size_t outsiz = insize * 6; // max length of utf-8 char is 6 per input char
  size_t outsiz_t = outsiz;

  char *outbuf = new char[outsiz];
  char *outbuf_t = outbuf;

  iconv_t conversion = iconv_open(toencoding, fromencoding);

  if (conversion == reinterpret_cast<iconv_t>(-1)) {
    DebugPrint perror(gettext("Failed to open iconv"), 
		      Print::DEBUGGING, DebugPrint::CRITICAL, "StringFormat");
    return "";
  }

  if (iconv(conversion, &inbuf, &insize, &outbuf_t, &outsiz_t) == -1)
    DebugPrint perror(gettext("String conversion failed"), 
		      Print::DEBUGGING, DebugPrint::CRITICAL, "StringFormat");

  iconv_close(conversion);

  return string(outbuf, outsiz - outsiz_t);
#else
  return in;
#endif
}

const int round_to_int(double number)
{
  return int(number + 0.5);
}

const int seconds_to_minutes(int time)
{
  return static_cast<int>(floor(time/60));
}
 
const int remaining_seconds(int time)
{
  return (time%60);
}

bool file_exists(const string& filename) {
  struct stat file_stat;
  return ((stat(filename.c_str(), &file_stat) == 0) ? true:false);
}

void create_if_not_exists(const string& dir)
{
  if (!file_exists(dir)) {

    string::size_type loc = dir.find("/", 0); // root
    
    while ((loc = dir.find("/", loc+1))) {

      if (!file_exists(dir.substr(0, loc+1)))
	mkdir(dir.substr(0, loc+1).c_str(), 0777);
      
      if (loc == string::npos)
	break;
    }
  }  
}

void deinit_mms(bool from_cleanup) 
{
  Config *conf = S_Config::get_instance();

  Render *render = S_Render::get_instance();

  render->prepare_new_image();
  render->clear();
  render->draw_and_release("exit");

  sleep(1);

#ifdef mms_debug
  cerr << "Lock input and one output plugin " << endl;
#endif

  Plugins *plugins = S_Plugins::get_instance();
  plugins->delete_plugins();

  if (!conf->p_shutdown_script().empty() && !from_cleanup)
    system(conf->p_shutdown_script().c_str());
}

void clean_up(int signal)
{
  deinit_mms(true);
  exit(1);
}
