#ifndef EXTRA_MENU_HPP
#define EXTRA_MENU_HPP

#include "functor.hpp"
#include "image.hpp"

#include "gettext.hpp"

#include <string>
#include <vector>

class ExtraMenuItem
{
public:
  ExtraMenuItem(const std::string c, const std::string& s, TimeFv *cb);

  std::string command;
  std::string shortcut;
  TimeFv *callback;
};

// ExtraMenu controls the ExtraMenuItems and thus it will automaticly delete
// them, when they are not needed anymore

class ExtraMenu
{
public:
  // items added using this function will be cleaned up upon destruction
  void add_item(const ExtraMenuItem& e);
  // items added using this function will NOT be cleaned up upon destruction
  void add_persistent_item(const ExtraMenuItem& e);

  void mainloop();

  ExtraMenu(const std::string& h = gettext("Extra Menu"), bool drop_down = false);
  ~ExtraMenu();

  void set_drop_down_pos(int x, int y);

private:
  std::string header;

  bool drop_down;
  int dd_x, dd_y;

  void print();
  void cleanup();

  // item, persistent
  std::vector<std::pair<ExtraMenuItem, bool> > items;
  int pos;
  Overlay o;
};

#endif
