#ifndef FUNCTOR_HPP
#define FUNCTOR_HPP

#include "plugins/input/input.hpp"

#include <iostream>
#include <vector>

class VoidFunctor
{
 private:
  void (*fpt)();
  
 public:
  VoidFunctor(void(*_fpt)())
  { fpt=_fpt; }

  void call() const
  { (*fpt)(); }
};

template <typename TClass, typename element>
class TSpecificFunctorNonconst
{
 private:
  void (TClass::*fpt)(std::vector<element>&);   // pointer to member function
  TClass* pt2Object;                  // pointer to object
  
 public:

  // constructor - takes pointer to an object and pointer to a member and stores
  // them in two private variables
  TSpecificFunctorNonconst(TClass* _pt2Object, void(TClass::*_fpt)(std::vector<element>&))
  { pt2Object = _pt2Object;  fpt=_fpt; }

  void call(std::vector<element>& list)
  { (*pt2Object.*fpt)(list);}             // execute member function
};

template <typename TClass, typename T, typename return_type>
class ConstFunctor
{
 private:
  return_type (TClass::*fpt)(const T&);
  TClass* pt2Object;
  
 public:
  ConstFunctor(TClass* _pt2Object, return_type(TClass::*_fpt)(const T&))
  { pt2Object = _pt2Object;  fpt=_fpt; }

  return_type call(const T& el)
  { return (*pt2Object.*fpt)(el);}
};

template <typename TClass, typename T, typename return_type>
class NonConstFunctor
{
 private:
  return_type (TClass::*fpt)(T&);
  TClass* pt2Object;
  
 public:
  NonConstFunctor(TClass* _pt2Object, return_type(TClass::*_fpt)(T&))
  { pt2Object = _pt2Object;  fpt=_fpt; }

  return_type call(T& el)
  { return (*pt2Object.*fpt)(el);}
};


template <typename TClass, typename T>
class ConstFunctor<TClass, T, void>
{
 private:
  void (TClass::*fpt)(const T&);
  TClass* pt2Object;
  
 public:
  ConstFunctor(TClass* _pt2Object, void(TClass::*_fpt)(const T&))
  { pt2Object = _pt2Object;  fpt=_fpt; }

  void call(const T& el)
  { (*pt2Object.*fpt)(el);}
};

template <typename TClass, typename F, typename T>
class CompareFunctor
{
 private:
  TClass* pt2Object;                  // pointer to object
  F *compare;
  std::vector<T> the_list;
  
 public:

  CompareFunctor(const std::vector<T>& lst, F* cmp)
  { compare = cmp; the_list = lst; }

  void set_func(TClass* _pt2Object)
  { pt2Object = _pt2Object; }

  const std::vector<T> call()
  { (pt2Object->gen_search)(the_list, compare);}             // execute member function
};

template <typename TClass, typename type>
class PrintFunctor
{
 private:
  void (TClass::*fpt)(const type& cur_element, const type& position, int y);
  TClass* pt2Object;
  
 public:
  
  PrintFunctor(TClass* _pt2Object, void(TClass::*_fpt)(const type&, const type&, int))
  { pt2Object = _pt2Object; fpt=_fpt; }

  void call(const type& cur_element, const type& position, int y)
  { (*pt2Object.*fpt)(cur_element, position, y);}
};

template <typename TClass, typename type>
class BoolPrintFunctorNonConst
{
 private:
  void (TClass::*fpt)(type& cur_element, const type& position, int y, bool z);
  TClass* pt2Object;
  bool z_val;
  
 public:
  
  BoolPrintFunctorNonConst(TClass* _pt2Object, void(TClass::*_fpt)(type&, const type&, int, bool), bool z)
  { pt2Object = _pt2Object; fpt=_fpt; z_val = z; }

  void call(type& cur_element, const type& position, int y)
  { (*pt2Object.*fpt)(cur_element, position, y, z_val);}
};

template <typename TClass, typename type>
class BoolPrintFunctor
{
 private:
  void (TClass::*fpt)(const type& cur_element, const type& position, int y, bool z);
  TClass* pt2Object;
  bool z_val;
  
 public:
  
  BoolPrintFunctor(TClass* _pt2Object, void(TClass::*_fpt)(const type&, const type&, int, bool), bool z)
  { pt2Object = _pt2Object; fpt=_fpt; z_val = z; }

  void call(const type& cur_element, const type& position, int y)
  { (*pt2Object.*fpt)(cur_element, position, y, z_val);}
};

// FIXME: rename this and TimeFi
class TimeFv
{
public:
  virtual void call() = 0;
  virtual ~TimeFv() {};
};

class TimeFi
{
public:
  virtual int call() = 0;
  virtual ~TimeFi() {};
};

class InputCheckBase
{
public:
  virtual bool call(const Input& i, bool disable_mode_change) = 0;
  virtual ~InputCheckBase() {};
};

template <typename TClass>
class TimeFunctorv : public TimeFv
{
 private:
  void (TClass::*fpt)();
  TClass* pt2Object;

 public:
  
  TimeFunctorv(TClass* _pt2Object, void(TClass::*_fpt)())
  { pt2Object = _pt2Object; fpt=_fpt; }

  void call()
  { (*pt2Object.*fpt)(); }
};

template <typename TClass>
class TimeFunctorvstring : public TimeFv
{
 private:
  void (TClass::*fpt)(const std::string&);
  TClass* pt2Object;
  std::string module;

 public:
  
  TimeFunctorvstring(TClass* _pt2Object, void(TClass::*_fpt)(const std::string&), const std::string& mod)
  { pt2Object = _pt2Object; fpt=_fpt; module = mod; }

  void call()
  { (*pt2Object.*fpt)(module); }
};

template <typename TClass>
class TimeFunctori : public TimeFi
{
 private:
  int (TClass::*fpt)();
  TClass* pt2Object;

 public:
  
  TimeFunctori(TClass* _pt2Object, int(TClass::*_fpt)())
  { pt2Object = _pt2Object; fpt=_fpt; }

  int call()
  { return (*pt2Object.*fpt)(); }
};

template <typename TClass>
class InputCheck : public InputCheckBase
{
 private:
  bool (TClass::*fpt)(const Input& i, bool disable_mode_change);
  TClass* pt2Object;

 public:
  
  InputCheck(TClass* _pt2Object, bool(TClass::*_fpt)(const Input& i, bool disable_mode_change))
  { pt2Object = _pt2Object; fpt=_fpt; }

  bool call(const Input& i, bool disable_mode_change)
  { return (*pt2Object.*fpt)(i, disable_mode_change); }
};

class TimeFunctorvNC : public TimeFv
{
 private:
  void (*fpt)();

 public:
  
  TimeFunctorvNC(void(*_fpt)())
  { fpt=_fpt; }

  void call()
  { (*fpt)(); }
};

class TimeFunctoriNC : public TimeFi
{
 private:
  int (*fpt)();

 public:
  
  TimeFunctoriNC(int(*_fpt)())
  { fpt=_fpt; }

  int call()
  { return (*fpt)(); }
};

#endif
