#ifndef GRAPHICAL_AUDIO_HPP
#define GRAPHICAL_AUDIO_HPP

#include "audio.hpp"

#include "cpp_sqlitepp.hpp"

#include "dbaudiofile.hpp"

#include "clientstub.h"

class GraphicalAudio : public AudioTemplate<Dbaudiofile>
{
  friend class Dbaudiofile;
private:
  
  SQLDatabase db;

  void create_db();
  void check_db();

  void reenter(int id);
  std::vector<std::pair<std::string, int> > gen_search_list(const std::string& search_word);

  void secondary_menu_in_add(ExtraMenu &em);
  void secondary_menu();

  // commands
  void print_information();
  // audio add specific
  void enter_dir();

  void search_func();
  
  ost::Mutex imms_mut;

  void add_file_to_list(std::vector<Dbaudiofile>& list, const Simplefile& el)
  {
    list.push_back(Dbaudiofile(el));
  }

  void print_element(Dbaudiofile& r, const Dbaudiofile& position, 
		     int y, bool picture_dir);

  // helper function for add_dir to add files in the correct order based on
  // track nr
  void add_tracks_from_dir(std::vector<Dbaudiofile> &files_in_dir, 
			   std::vector<Dbaudiofile> &cur_files);

  bool rdir_internal(const std::string& filename, const std::string& argv,
		     std::vector<Dbaudiofile> &cur_files);
  void insert_file_into_db(const std::string& filename, const std::string& parent);
  std::vector<std::string> check_db_for_folders(const std::string& parent);
  
  // add an audio dir recursively (all the files in it) to the playlist
  std::vector<Dbaudiofile> add_dir(const std::string& dir, bool from_media = false);
  // read a dir and spit out a list of files and dirs
  std::vector<Dbaudiofile> rdir(const std::string& argv);

  void intelligent_random_prev();
  void intelligent_random_next();

  void print_audiotrack_helper(int &x_pos, std::string& buffer, 
			       std::string& album, std::string& artist, std::string& title);
  void print_audio_screensaver_helper(int &x_pos, int &y,  int max_x_pic, int max_y_pic);

  struct FilterOps
  {
    static void set_next(int next)
    {
      GraphicalAudio *audio = get_class<GraphicalAudio>("Audio");
      audio->next_track_pos = next;
    }
    static void reset_selection()
    {
      GraphicalAudio *audio = get_class<GraphicalAudio>("Audio");
      audio->next_track_pos = -1;
    }
    static string get_item(int index)
    {
      GraphicalAudio *audio = get_class<GraphicalAudio>("Audio");
      return (index > audio->playlist_size())? "" : audio->get_playlist_element(index);
    }
    static int get_length()
    {
      GraphicalAudio *audio = get_class<GraphicalAudio>("Audio");
      return audio->playlist_size();
    }
  }; 

  typedef IMMSClient<FilterOps> IMMSC;
  IMMSC *imms;
  std::string imms_cur_path;

  // metadata extraction
  int check_metadata();
  void extract_metadata();

  bool file_exists_in_db(const std::string& file);
  int db_id(const string& file, bool is_dir);

  bool extracted_all_files;
  std::stack<std::string> all_metadata_files;

  bool no_folders_in_dir(const std::vector<Dbaudiofile>& cur_files);
  void print_graphical(std::vector<Dbaudiofile>& files);
  void print_tracklist(std::vector<Dbaudiofile>& cur_files);
  void print_dir(int pos, int j, int x, int y);

  void options_reloaded();

  // map to speed up finding covers
  std::map<std::string, std::string> cover_map;

  std::string get_cover_from_dir(const std::string& dir);
  std::string get_first_file_in_dir(const std::string& dir);

  int y_start;

  int image_width;
  int image_height;
  int image_height_all_eks_text;

  // including text and space around image
  int image_width_all;
  int image_height_all;
  int image_height_all_search;

  int images_per_row;
  int rows;
  int rows_search;

  std::string get_first_cover_in_dir(const std::string& dir);

  bool search_cur_dir_only;

  void print_choose_cover(const std::vector<std::string>& covers, int cover_pos, Overlay& o);
  void choose_cover();

  void page_up();
  void page_down();

  int check_search();
  
public:
  GraphicalAudio();
  ~GraphicalAudio();

  int next_track_pos;

  void read_dirs();

  std::string get_name_from_file(const Dbaudiofile& d);
  std::string get_short_name_from_file(const Dbaudiofile& d);

  std::string mainloop(AudioMode mode);

  void print(std::vector<Dbaudiofile>& cur_files);

  void print(const std::vector<Dbaudiofile>& files)
  {
    std::cout << "WARNING: not used" << std::endl;
  }

  void startup_updater();
};

#endif
