#include "config.h"

#include "common-feature.hpp"
#include "input.hpp"

#include "plugins.hpp"

// ugly c fork code
#include <sys/wait.h>

#include <errno.h>             /* per  errno */

void aquire_exclusive_plugin_access()
{
  Plugins *plugins = S_Plugins::get_instance();
  for(std::vector<FeaturePlugin*>::const_iterator iter=plugins->fp_data.begin(); 
      iter!=plugins->fp_data.end(); ++iter)
    (*iter)->module->begin_external();
}

void aquire_exclusive_access()
{
  InputMaster *im = S_InputMaster::get_instance();
  im->device->suspend();
  aquire_exclusive_plugin_access();
}

void run::release_exclusive_plugin_access()
{
  Plugins *plugins = S_Plugins::get_instance();
  for(std::vector<FeaturePlugin*>::const_iterator iter=plugins->fp_data.begin(); 
      iter!=plugins->fp_data.end(); ++iter)
    (*iter)->module->end_external();
}

void release_exclusive_access()
{
  run::release_exclusive_plugin_access();
  
  InputMaster *im = S_InputMaster::get_instance();
  im->device->wake_up();
}

// fork another program into the forground
bool run::exclusive_external_program(const std::string& command)
{
  aquire_exclusive_access();
  
  bool status = external_program(command);

  release_exclusive_access();

  return status;
}

std::pair<bool, Pipe> run::exclusive_external_program_pipe(const std::string& command)
{
  aquire_exclusive_plugin_access();

  std::pair<bool, Pipe> status = external_program_pipe(command);

  return status;
}

// not so extreme forker :)
bool run::external_program(const std::string& command, bool wait_for_finish)
{
  int retstatus = 0;
  pid_t waitpid = 0;
  
  pid_t pid = fork();

  if (pid == 0) {
    /* this is the child process. Execute the shell command. */
    // child inherits file descriptors
    // we don't want it to mess with them so we close them all
    // except  stdin, stdout and stderr
    for(int x = 3; x <sysconf(_SC_OPEN_MAX) - 1; x++)
      close(x);
 
    execl ("/bin/sh", "sh", "-c", command.c_str(), NULL);

    // if we're here something went wrong
    fprintf(stderr, "Got error '%s' launching external program\n", strerror(errno));
    exit(1);
  } else if (pid < 0) {
    /* the fork failed. Report failure. */
    fprintf(stderr, "Got error %s: Couldn't fork\n", strerror(errno));
    return false;
  } else {
    /* This is the parent process. Wait for child to end. */
    if (wait_for_finish) {
      waitpid=wait(&retstatus);

      if (waitpid <0)  //wait failed
	{
	  fprintf(stderr, "Got error %s: Couldn't wait for child to exit\n", 
		  strerror(errno));
	  return false;
	}
  
      //let's process return status
      if (WIFEXITED(retstatus)) {
	//child ended normally
	//      printf("Child ended with status %d\n",WEXITSTATUS(retstatus));
	return true;
      }
      else if (WIFSIGNALED(retstatus)) { //let's see what went wrong
	fprintf(stderr, "Child exited due to %d signal\n", WTERMSIG(retstatus));
	return false;
      }
      else if(WIFSTOPPED(retstatus)) {
	fprintf(stderr, "Child process was stopped\n");
	return false;
      }

      //this should never be executed
      fprintf(stderr, "Undefined or unhandled child terminatio\n");
      return false;
    }
  }

  return true;
}

std::pair<bool, Pipe> run::external_program_pipe(const std::string& command)
{
  Pipe p;

  int inpipe[2], outpipe[2], pipefl;

  if (pipe(inpipe) == -1) {
    std::cerr << "ERROR: pipe failed for inpipe" << std::endl;
    return std::make_pair(false, p);
  }
  pipefl|=1+2;
  if(pipe(outpipe)==-1) {
    std::cerr << "ERROR: pipe failed for outpipe" << std::endl;
    return std::make_pair(false, p);
  }
  pipefl|=4+8;

  pid_t pid = fork();

  if (pid == 0)
    {
      close(inpipe[1]);
      close(outpipe[0]);

      if (dup2(inpipe[0], STDIN_FILENO) < 0 ||
         dup2(outpipe[1],STDOUT_FILENO) < 0 ||
         dup2(outpipe[1],STDERR_FILENO) < 0)
	std::cerr << "ERROR: dup2() failed" << std::endl;

      close(inpipe[0]);
      close(outpipe[1]);

      /* this is the child process. Execute the shell command. */
      execle("/bin/sh", "sh", "-c", command.c_str(), 0, environ);
      exit(0);
    }
  else if (pid < 0) 
    /* the fork failed. Report failure. */
    return std::make_pair(false, p);

  close(inpipe[0]); pipefl&=~1;
  close(outpipe[1]); pipefl&=~8;
  fcntl(outpipe[0],F_SETFL,O_NONBLOCK);

  p.inpipe0 = inpipe[0];
  p.inpipe1 = inpipe[1];
  p.outpipe0 = outpipe[0];
  p.outpipe1 = outpipe[1];
  p.pipefl = pipefl;
  p.pid = pid;

  return std::make_pair(true, p);
}

void run::close_pipe(const Pipe& p)
{
  kill(p.pid,SIGTERM);
  waitpid(p.pid,0,0); // get status otherwise the child stays as a zombie forever
  if(p.pipefl&1) close(p.inpipe0);
  if(p.pipefl&2) close(p.inpipe1);
  if(p.pipefl&4) close(p.outpipe0);
  if(p.pipefl&8) close(p.outpipe1);
}
