#ifndef COMMON_FEATURE_HPP
#define COMMON_FEATURE_HPP

#include "plugins.hpp"

#include <string>

struct Pipe
{
  int inpipe0;
  int inpipe1;
  int outpipe0;
  int outpipe1;
  int pipefl;
  pid_t pid;
};

namespace run
{
  // just start an external program
  bool external_program(const std::string& command, bool wait = true);

  // start an external program that will have exclusive access to shared
  // ressources such as sound card
  bool exclusive_external_program(const std::string& command);

  // start an exclusive program and create a pipe to talk to the external program
  std::pair<bool, Pipe> exclusive_external_program_pipe(const std::string& command);

  void release_exclusive_plugin_access();
  void close_pipe(const Pipe& p);

  // helper function
  std::pair<bool, Pipe> external_program_pipe(const std::string& command);
}

template<typename T>
T* get_class(const std::string& plugin_name)
{
  Plugins *plugins = S_Plugins::get_instance();
  for (std::vector<FeaturePlugin*>::const_iterator iter=plugins->fp_data.begin(),
	 iend=plugins->fp_data.end(); iter!=iend; ++iter)
    if ((*iter)->plugin_name() == plugin_name)
      return static_cast<T*>((*iter)->module);

  return 0;
}

#endif
