#ifndef GAME_HPP
#define GAME_HPP

#include "config.h"

#include "cpp_sqlitepp.hpp"

#include "game_config.hpp"

#include "module.hpp"
#include "options.hpp"
#include "config.hpp"
#include "multifile.hpp"
#include "gettext.hpp"

#include <stack>
#include <vector>
#include <list>

class GameEntry : public Multifile
{
public:
  int db_id;
};

class Game : public Module 
{
private:
  SQLDatabase db;

  void create_db();
  void check_db();
  std::vector<std::pair<std::string, int> > gen_search_list(const std::string& search_word);

  bool rdir_internal(const std::string& filename, const std::string& argv, 
		     std::vector<GameEntry> &cur_files);
  void insert_file_into_db(const std::string& filename, const std::string& parent);
  int db_id(const std::string& file, bool is_dir);
  std::vector<std::string> check_db_for_folders(const std::string& parent);

  void reenter(int db_id);
  int find_position_in_folder_list(const std::list<std::string>& files, 
				   const std::string& filename);

  void reparse_current_dir();


  // commands
  void enter_dir();
  void go_back();
  void action_play();
  void options();
  void search_func();
  void exit();
  
  bool exit_loop;

  bool search_compare(const GameEntry& s);
  std::string get_name_from_file(const GameEntry& e);

  void print(const std::vector<GameEntry>& files);
  void print_game_element(const GameEntry& r, const GameEntry& position, int y);

  std::string get_cover(const std::string& s);
  void graphical_print(const std::vector<GameEntry>& files);

  // playback
  void playgame(const GameEntry file);
#ifdef use_media
  void showcd(int count);
  void playpsx();
#endif

#ifdef use_media
  // dir
  std::string testdir(int *count);
#endif

  GameEntry addfile(const std::string& name, const MyPair& filetype);
  GameEntry add_dir(const std::string& filename);

  std::vector<GameEntry> rdir(const std::string& argv);
  std::vector<GameEntry> parse_dir(const std::list<std::string>& dirs);

  // for saving information about how you traverse a filesystem a stack is a
  // much better solution. The first element is the dirs and the next is the
  // position
  std::stack<std::pair<std::list<std::string>, int > > folders;

  std::vector<GameEntry> files;

  int image_width;
  int image_height;
  int image_height_all_eks_text;

  // including text and space around image
  int image_width_all;
  int image_height_all;
  int image_height_all_search;

  int images_per_row;
  int rows;
  int rows_search;

  int y_start;

  GameConfig *game_conf;

  class GameOpts : public Options 
  {
  public:
    GameOpts();
    ~GameOpts();

    std::vector<Option*> values() 
    {
      return val;
    }

  private:
    void read_dirs();

    Option *reload_p;
  };

  GameOpts opts;

  struct file_sort
  {
    bool operator()(const GameEntry& lhs, const GameEntry& rhs);
  };

public:

  int files_size() { return files.size(); }

  void reset();

  void load_current_dirs();
  void read_dirs();
  
#ifdef use_media
  void determine_media();
#endif

  Options* get_opts()
  {
    return &opts;
  }
  
  void startup_updater();

  std::string mainloop();

  Game();
};

#endif
