#include "config.h"

#include "libfspp.hpp"

#include "module.hpp"

#include "input.hpp"
#include "config.hpp"
#include "common.hpp"
#include "renderer.hpp"
#include "text_object.hpp"

#include "updater.hpp"

#include <iostream>

using std::vector;
using std::list;
using std::string;

Module::Module() 
  : search_mode(false), search_str(""), offset(0), 
    search_help_offset(-1), search_need_cleanup(false), 
    search_top_size(35), overlay_search("search marker"), 
    id(1), loaded_correctly(false), reload_needed(false)
{
  input_master = S_InputMaster::get_instance();
  themes = S_Themes::get_instance();
  render = S_Render::get_instance();
  conf = S_Config::get_instance();
  global = S_Global::get_instance();
  if (conf->p_media())
    cd = S_Cd::get_instance();

  int size = 55;

  string all_chars = global->get_valid_keys();

  letters.clear();

  string spaced_keys = "";
  vector<int> group_start;

  int start_pos = 51;

  for (int i = 0; i < all_chars.size(); ++i) {
    if (all_chars[i] != '|') {
      spaced_keys += (all_chars.substr(i,1) + " ");
    }
      
    if (all_chars[i] == '|' || i == all_chars.size()-1) {

      spaced_keys = spaced_keys.substr(0, spaced_keys.size()-1);

      int letters_width = string_format::calculate_string_width(spaced_keys, "Vera/22");

      spaced_keys = "";
      start_pos += 5 + letters_width;

      group_start.push_back(start_pos);
    }
  }

  int group_pos = 0;

  for (int i = 0; i < all_chars.size(); ++i) {
    string cur_char = all_chars.substr(i, 1);
    if (cur_char != "|") {
      letters.push_back(size);
      int letter_space = string_format::calculate_string_width(cur_char, "Vera/22");
      size += (letter_space-1);
    } else {
      size = vector_lookup(group_start, group_pos);
      ++group_pos;
    }
  }
}

void Module::print_marked_search_letter()
{
  if (conf->p_input() != "lirc" && conf->p_input() != "evdev")
    return;

  if (search_mode) {

    search_need_cleanup = true;

    render->wait_and_aquire();

    if (overlay_search.elements.size() > 0)
      overlay_search.partial_cleanup();

    if (time(0)-conf->p_last_key() <= 2 && search_help_offset != -1)
      overlay_search.add(new RObj(vector_lookup(letters, search_help_offset), 
				  60+60, 25, 25, 160, 150, 90, 100, 4));

    string keys = global->get_valid_keys();
    string spaced_keys = "";

    int start_key_box = 0;
    int letters_pos = 0;

    int start_pos = 55;

    for (int i = 0; i < keys.size(); ++i) {
      if (keys[i] != '|') {
	spaced_keys += (keys.substr(i,1) + " ");
	//	++letters_pos;
      }
      
      if (keys[i] == '|' || i == keys.size()-1) {

	spaced_keys = spaced_keys.substr(0, spaced_keys.size()-1);

	int letters_width = string_format::calculate_string_width(spaced_keys, "Vera/22");
	
	overlay_search.add(new RObj(start_pos-4, 60+60, letters_width, 25, 
				    20, 30, 20, 255, 3));

	overlay_search.add(new TObj(spaced_keys, "Vera/22", 
				    start_pos, 60+55, themes->audio_font1, 
				    themes->audio_font2, themes->audio_font3, 5));

	spaced_keys = "";
	start_pos = start_pos + 5 + letters_width;
      }
    }

    render->draw_and_release("search marker", true);
  }
}

void Module::clean_up_search_print()
{
  if (search_need_cleanup) {
    render->wait_and_aquire();
    overlay_search.cleanup();
    render->image_mut.leaveMutex();
    search_need_cleanup = false;
  }
}

void Module::print_range_top(int int_position, int list_size, int& y, int& start, int& range)
{
  y = 105;

  if (conf->p_h_res() == 405)
    y = 85;

  if (search_mode)
    y += search_top_size;

  int buttom = 90;
    
  if (conf->p_h_res() == 405)
    buttom = 65;

  range = (conf->p_h_res()-y-buttom)/28;

  int width = conf->p_v_res()-2*55;
  int height = range*28+25;

  if (search_mode)
    render->current.add(new RObj(55, y-15, width, height, 
				 themes->general_rectangle_color1, themes->general_rectangle_color2,
				 themes->general_rectangle_color3, themes->general_rectangle_color4, 1));
  else
    render->current.add(new RObj(55, y-15, width, height, 
				 themes->general_rectangle_color1, themes->general_rectangle_color2,
				 themes->general_rectangle_color3, themes->general_rectangle_color4, 1));

  start = int_position - (range/2);

  // special cases
  if (range < list_size) {
    if (int_position >= list_size - (range/2)) // end
      start = list_size - range;
    else if ((range/2) >= int_position) // beginning
      start = 0;
  } else { // playlist smaller than range
    start = 0;
    range = list_size;
  }
}

// ====================================================================
//
// dir
//
// ====================================================================

// Add a file to the list
Simplefile Module::addsimplefile(const string& name, const MyPair& filetype, bool from_media)
{
  Simplefile r;
  r.id = ++id;
  r.path = name;

  string::size_type pos = name.rfind("/");
  if (pos != string::npos)
    r.name = name.substr(pos+1, name.size()-((pos+1)+(filetype.first.size()+1)));
  else
    r.name = name.substr(0, name.size()-(filetype.first.size()+1));

  if (conf->p_convert())
    r.name = string_format::convert(r.name);

  r.lowercase_name = string_format::lowercase(r.name);

  if (from_media)
    r.type = "media-file";
  else
    r.type = filetype.second;

  return r;
}
