#ifndef MODULE_HPP
#define MODULE_HPP

#include "config.h"

#include "common.hpp"
#include "themes.hpp"
#include "renderer.hpp"
#include "config.hpp"
#include "simplefile.hpp"
#include "functor.hpp"
#include "options.hpp"
#include "global.hpp"
#include "image.hpp"

#include "cd.hpp"

#include "rectangle_object.hpp"

#include "input.hpp"

#include <cmath>
#include <string>
#include <list>
#include <vector>

#define X_DISPLAY_MISSING

#include <Imlib2.h>

class Module
{
protected:

  // implemented in terms of:
  InputMaster *input_master;
  Themes *themes;
  Render *render;
  Config *conf;
  Global *global;

  Cd *cd;

  // search variables
  bool search_mode;
  std::string search_str;
  std::string lowercase_search_str;
  int offset;
  int search_help_offset;
  bool search_need_cleanup;
  int search_top_size;

  std::vector<int> letters;

  Overlay overlay_search;

  void print_marked_search_letter();
  void clean_up_search_print();

  void print_range_top(int int_position, int list_size, int& y, int& start, int& range);
  
  template<typename T, typename F>
  void print_range(const std::vector<T>& files, const T& position,
		   const int int_position, F* print_func)
  {
    int y, start, range;
    print_range_top(int_position, files.size(), y, start, range);
  
    // goto start of print
    typename std::vector<T>::const_iterator iter = files.begin() + start;

    // print 
    for (int j = 0; j < range; ++j)
      {
	print_func->call(*iter, position, y);
	
	y += 28;
	++iter;
      }
  }

  template<typename T, typename F>
  void print_range_modifying(std::vector<T>& files, const T& position,
			     const int int_position, F* print_func)
  {
    int y, start, range;
    print_range_top(int_position, files.size(), y, start, range);
  
    // goto start of print
    typename std::vector<T>::iterator iter = files.begin() + start;

    // print 
    for (int j = 0; j < range; ++j)
      {
	print_func->call(*iter, position, y);

	y += 28;
	++iter;
      }
  }

  // dir
  int id;

  Simplefile addsimplefile(const std::string& name, const MyPair& filetype, bool from_media = false);

public:

  // FIXME: maybe convert to this, nicer code
  virtual Options *get_opts() = 0;
  virtual bool init() { std::cerr << "Module virtual init" << std::endl; return true; };

  bool loaded_correctly;
  bool reload_needed;
  
  // must be implemented by modules
  virtual void startup_updater() = 0;
  // must be implemented by modules if external programs might use their
  // resources such as the soundcard
  virtual void begin_external() {};
  virtual void end_external() {};

  virtual ~Module() {};

  Module();

  virtual std::string mainloop() = 0;
};

#endif
