#include "collection.hpp"

#include "search.hpp"
#include "busy_indicator.hpp"
#include "shutdown.hpp"

using std::string;
using std::vector;
using std::list;

MovieCollection::MovieCollection()
  : MovieDB("collection.db", false)
{
  opts.clear();

  // WTF?
  loaded_correctly = false;
}

void MovieCollection::startup_updater()
{
  EnterFunctor<MovieCollection> *e = new EnterFunctor<MovieCollection>(this, &MovieCollection::reenter);
  SearchGenList<MovieCollection> *s = new SearchGenList<MovieCollection>(this, &MovieCollection::gen_search_list);
  S_Search::get_instance()->register_module(SearchModule("Movie Coll.", s, e));
}

void MovieCollection::reenter(int db_id)
{
  if (!loaded_correctly)
    read_dirs();
  
  int pos = 0;
  for (vector<CIMDBMovie>::iterator i = files.begin(), iend = files.end(); i != iend; ++i, ++pos)
    if (db_id == i->id) {
      folders.top().second = pos;
      break;
    }

  mainloop();
}

vector<std::pair<string, int> > MovieCollection::gen_search_list(const string& search_word)
{
  vector<std::pair<string, int> > result;

  SQLQuery *q = db.query("Actors", ("SELECT DISTINCT Movie.id, Movie.title FROM %t INNER JOIN AMovie ON Actors.id = AMovie.aid INNER JOIN Movie ON Movie.id = AMovie.mid WHERE fuzzycmp('" + 
			 search_word + "', name, 1) OR fuzzycmp('" + search_word + "', title, 1) OR fuzzycmp('" + 
			 search_word + "', tagline, 1) OR fuzzycmp('" + search_word + "', plot, 1)").c_str());

  if (q) {
    for (int i = 0; i < q->numberOfTuples(); ++i) {
      SQLRow &row = (*q)[i];
      result.push_back(std::make_pair(row["title"], conv::atoi(row["id"])));
    }
  }
  delete q;

  return result;
}
  
vector<CIMDBMovie> MovieCollection::parse_dir(const list<string>& dirs)
{
  // Dummy function, this is not needed for this plugin

  std::vector<CIMDBMovie> empty;
  return empty;
}

void MovieCollection::read_dirs()
{
  // try to find the movie
  SQLQuery *q = db.query("Movie", "SELECT title FROM %t order by title asc");

  if (q) {
    for (int i = 0; i < q->numberOfTuples(); ++i) {
      SQLRow &movie = (*q)[i];
      CIMDBMovie tmp;
      tmp.m_strTitle = movie["title"];
      tmp.Load(physical, db);
      tmp.name = tmp.m_strTitle;
      files.push_back(tmp);
    }
  }

  delete q;

  // FIXME: folders?

  loaded_correctly = true;
}

// commands
void MovieCollection::secondary_menu()
{
  ExtraMenu em;

  em.add_item(ExtraMenuItem(gettext("Print information"), input_master->find_shortcut("action"), 
			    new TimeFunctorv<MovieCollection>(this, &MovieCollection::print_info)));

  if (files.size() > 0)
    em.add_item(ExtraMenuItem(gettext("Update movie information"), input_master->find_shortcut("reget"), 
			      new TimeFunctorv<MovieCollection>(this, 
								&MovieCollection::reget_movie_information)));

  em.add_item(ExtraMenuItem(gettext("Search IMDb"), input_master->find_shortcut("search_imdb"), 
			    new TimeFunctorv<MovieCollection>(this, &MovieCollection::search_imdb)));

  em.add_item(ExtraMenuItem(gettext("Return to startmenu"), 
			    input_master->find_shortcut("back"), 
			    new TimeFunctorv<MovieCollection>(this, &MovieCollection::exit)));

  add_standard(em);

  for (vector<ExtraMenuItem>::const_iterator i = global->menu_items.begin(), 
	 end = global->menu_items.end(); i != end; ++i)
    em.add_persistent_item(*i);

  conf->s_sam(true);

  em.mainloop();
}

string MovieCollection::mainloop()
{
  BusyIndicator &busy_indicator = BusyIndicator::get_instance();
  Shutdown *sd = S_Shutdown::get_instance();

  input_master->set_map("movie");

  Input input;

  bool update_needed = true;

  while (!(input.command == "back" && input.mode == "general"))
    {
      if (update_needed)
	print(files);

      busy_indicator.idle();

      input = input_master->device->get_input();

      busy_indicator.busy();

      if (sd->is_enabled()) {
	sd->cancel();
	continue;
      }

      update_needed = true;

      if (input.command == "prev")
	{
	  prev();
 	}
      else if (input.command == "next")
 	{
	  next();
 	}
      else if (input.command == "left")
	{
	  left();
	}
      else if (input.command == "right")
 	{
	  right();
 	}
      else if (input.command == "back")
	{
	  exit();
	}
      else if (input.command == "reget")
	{
	  if (files.size() > 0)
	    reget_movie_information();
	}
      else if (input.command == "search_imdb")
	{
	  search_imdb();
	}
      else if (input.command == "action") 
	{
	  print_info();
	}
      else if (input.command == "search")
	{
	  search_func();
        }
      else if (input.command == "second_action")
	{
	  secondary_menu();
	}
      else
	MovieTemplate<CIMDBMovie>::movie_mainloop_common(input);

      update_needed = !global->check_commands(input);
    }

  exit_loop = false;

  return "";
}
