#include "graphical_movie.hpp"

#include "libfspp.hpp"

#include "search.hpp"
#include "busy_indicator.hpp"
#include "shutdown.hpp"

// ostringstream
#include <sstream>
#include <fstream>

#include <assert.h>

using std::string;
using std::vector;
using std::list;

GraphicalMovie::GraphicalMovie()
  : MovieDB("moviehd.db", true)
{
  image_width_all = image_width+25;
  image_height_all = image_height+55;
  image_height_all_eks_text = image_height + 6;

  images_per_row = (conf->p_v_res()-2*25)/image_width_all;

  // lets make some more room for text 
  image_width_all += ((conf->p_v_res()-2*25)%image_width_all)/images_per_row;

  // + 20 is for the last row, since we don't need to add extra space
  rows = (conf->p_h_res()-2*y_start+20)/image_height_all;
  rows_search = (conf->p_h_res()-2*y_start+20-50)/image_height_all;

  image_height_all = (conf->p_h_res()-2*y_start+20)/rows;
  image_height_all_search = (conf->p_h_res()-2*y_start+20-50)/rows_search;

  imdb_dir = "/var/lib/mms/movies/";
  if (conf->p_homedir() != "/etc/mms/")
    imdb_dir = conf->p_homedir() + "movies/";

  if (!file_exists(imdb_dir)) {
    if (mkdir(imdb_dir.c_str(), 0777) == -1)
      DebugPrint perror(gettext("Could not create directory ") + imdb_dir,
			Print::DEBUGGING, DebugPrint::CRITICAL, "MOVIE");
  }

  imdb_message_not_displayed = false;
}

void GraphicalMovie::startup_updater()
{
  EnterFunctor<GraphicalMovie> *e = new EnterFunctor<GraphicalMovie>(this, &GraphicalMovie::reenter);
  SearchGenList<GraphicalMovie> *s = new SearchGenList<GraphicalMovie>(this, &GraphicalMovie::gen_search_list);
  S_Search::get_instance()->register_module(SearchModule("Movies", s, e));
  Movie::startup_updater();
}

int GraphicalMovie::find_position_in_folder_list(const list<string>& files, 
						 const string& filename)
{
  int pos = 0;
  bool pos_found = false;
  for (list<string>::const_iterator i = files.begin(), iend = files.end(); i != iend; ++i){
    vector<CIMDBMovie> templist = rdir(*i);
    pos = 0;
    for (vector<CIMDBMovie>::iterator fi = templist.begin(), fiend = templist.end(); fi != fiend; ++fi, ++pos) {
      if ((filename[filename.size()-1] == '/' && filename == fi->path) ||
	  filename == fi->filenames.front()) {
	pos_found = true;
	break;
      }
    }
    if (pos_found)
      break;
  }
  return pos;
}

void GraphicalMovie::reenter(int db_id)
{
  reset();

  SQLQuery *q = db.query("Folders", ("SELECT * FROM %t WHERE id='" + conv::itos(db_id) + "'").c_str());

  if (q && q->numberOfTuples() > 0) {

    string filename = (*q)[0]["filename"];
    string cur_dir_path;

    if (filename[filename.size() - 1] == '/')
      filename = filename.substr(0, filename.size() - 1);

    list<string> dirs = movie_conf->p_movie_dirs();

    int pos;
    for (list<string>::iterator i = dirs.begin(), iend = dirs.end(); i != iend; ++i)
      if ((pos = filename.rfind(*i)) != string::npos) {
	cur_dir_path = filename.substr(0, i->size());
	filename = filename.substr(i->size());
      }

    if (!filename.empty()) {
      while ((pos = filename.find("/")) != string::npos) {
	folders.top().second = find_position_in_folder_list(folders.top().first, cur_dir_path + filename.substr(0, pos + 1));

	list<string> templs; templs.push_back(cur_dir_path + filename.substr(0, pos + 1));
	folders.push(std::make_pair(templs, 0));
	cur_dir_path += filename.substr(0, pos+1);
	filename = filename.substr(pos+1);
      }
    }

    // we should now be in the correct folder, find the correct file
    folders.top().second = find_position_in_folder_list(folders.top().first, (*q)[0]["filename"]);

    load_current_dirs();

#ifdef use_inotify
    S_Notify::get_instance()->register_plugin("movie", folders.top().first,
					      new TimeFunctorv<Movie>(this, &Movie::reparse_current_dir));
#endif

    mainloop();
  }

  delete q;
}

std::vector<std::pair<std::string, int> > GraphicalMovie::gen_search_list(const std::string& search_word)
{
  std::vector<std::pair<std::string, int> > result;

  list<string> current_folders = folders.top().first;

  vector<int> ids_already_found;

  // folders
  SQLQuery *q = db.query("Folders", ("SELECT DISTINCT id, filename, name, is_folder FROM %t WHERE fuzzycmp('" + search_word + "', name, 1)").c_str());

  if (q) {

    for (int i = 0; i < q->numberOfTuples(); ++i) {

      SQLRow &row = (*q)[i];

      string out;

      bool result_ok = false;
      list<string> current_folders = folders.top().first;
      for (list<string>::iterator i = current_folders.begin(), iend = current_folders.end();
	   i != iend; ++i) {
	if (row["filename"].find(*i) != string::npos) {
	  result_ok = true;
	  break;
	}
      }

      if (result_ok) {
	out = row["name"];
	result.push_back(std::make_pair(string_format::convert(out), conv::atoi(row["id"])));
      }
    }
  }
  delete q;

  q = db.query("HDMovie", ("SELECT DISTINCT * FROM %t WHERE fuzzycmp('" + search_word + "', title, 1)").c_str());
  if (q) {
    for (int i = 0; i < q->numberOfTuples(); ++i) {

      SQLRow &row = (*q)[i];

      int id;
	  
      SQLQuery *q4 = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(row["path"]) + "'").c_str());
      if (q4 && q4->numberOfTuples() > 0) {
	assert(q4->numberOfTuples() == 1);
	SQLRow &c_row = (*q4)[0];
	id = conv::atoi(c_row["id"]);
      } else
	assert(false);

      delete q4;

      bool found_match = false;
      for (vector<int>::const_iterator iter = ids_already_found.begin(), 
	     iend = ids_already_found.end();
	   iter != iend; ++iter)
	if (*iter == id) {
	  found_match = true;
	  break;
	}

      if (found_match)
	continue;

      result.push_back(std::make_pair(row["title"], id));
    }
  }
  delete q;

  return result;
}

// commands
void GraphicalMovie::secondary_menu()
{
  ExtraMenu em;

  if (vector_lookup(files, folders.top().second).type != "file") {
    em.add_item(ExtraMenuItem(gettext("Enter directory"), input_master->find_shortcut("action"), 
			      new TimeFunctorv<GraphicalMovie>(this, &GraphicalMovie::enter_dir)));

    em.add_item(ExtraMenuItem(gettext("Play all files in dir"), input_master->find_shortcut("play_movie"), 
			      new TimeFunctorv<GraphicalMovie>(this, &GraphicalMovie::action_play)));
  } else 
    em.add_item(ExtraMenuItem(gettext("Play movie"), input_master->find_shortcut("action"), 
			      new TimeFunctorv<GraphicalMovie>(this, &GraphicalMovie::action_play)));

  em.add_item(ExtraMenuItem(gettext("Update movie information"), input_master->find_shortcut("reget"), 
			    new TimeFunctorv<GraphicalMovie>(this, &GraphicalMovie::reget_movie_information)));

  em.add_item(ExtraMenuItem(gettext("Print information"), input_master->find_shortcut("info"), 
			    new TimeFunctorv<GraphicalMovie>(this, &GraphicalMovie::print_info)));

  if (folders.size() > 1)
    em.add_item(ExtraMenuItem(gettext("Go up one directory"), input_master->find_shortcut("back"), 
			      new TimeFunctorv<GraphicalMovie>(this, &GraphicalMovie::go_back)));
  else
    em.add_item(ExtraMenuItem(gettext("Return to startmenu"), input_master->find_shortcut("back"), 
			      new TimeFunctorv<GraphicalMovie>(this, &GraphicalMovie::exit)));

  add_standard(em);

  for (vector<ExtraMenuItem>::const_iterator i = global->menu_items.begin(), 
	 end = global->menu_items.end(); i != end; ++i)
    em.add_persistent_item(*i);

  conf->s_sam(true);

  em.mainloop();
}

void GraphicalMovie::go_back()
{
  if (!(folders.size() == 1)) {
    folders.pop();
    load_current_dirs();

#ifdef use_inotify
    S_Notify::get_instance()->register_plugin("movie", folders.top().first,
					      new TimeFunctorv<Movie>(this, &Movie::reparse_current_dir));
#endif
  }
}

void GraphicalMovie::enter_dir()
{
  if (vector_lookup(files, folders.top().second).type != "file") {
    string temp_cur_dir = vector_lookup(files ,folders.top().second).path;
    vector<CIMDBMovie> templist = rdir(temp_cur_dir);

    if (templist.size() != 0) {
      list<string> templs; templs.push_back(temp_cur_dir);
      folders.push(std::make_pair(templs, 0));
      files = templist;

#ifdef use_inotify
      S_Notify::get_instance()->register_plugin("movie", folders.top().first,
						new TimeFunctorv<Movie>(this, &Movie::reparse_current_dir));
#endif
    } else {
      DialogWaitPrint pdialog(gettext("Folder is empty"), 1000);
    }
  }
}

string GraphicalMovie::mainloop()
{
  BusyIndicator &busy_indicator = BusyIndicator::get_instance();
  Shutdown *sd = S_Shutdown::get_instance();

  input_master->set_map("movie");

  Input input;

  bool update_needed = true;

  while (!exit_loop)
    {
      if (update_needed) {
	print(files);
	print_lcd_menu();
      }

      busy_indicator.idle();
      
      input = input_master->device->get_input();

      busy_indicator.busy();

      if (sd->is_enabled()) {
	sd->cancel();
	continue;
      }

      update_needed = true;

      fullscreen_check(input);

      if (input.command == "prev")
	{
	  prev();
 	}
      else if (input.command == "next")
 	{
	  next();
 	}
      else if (input.command == "left")
	{
	  left();
	}
      else if (input.command == "right")
 	{
	  right();
 	}
      else if (input.command == "back" && folders.size() > 1)
	{
	  go_back();
	}
      else if (input.command == "back" && folders.size() == 1)
	{
	  exit();
	}
      else if (input.command == "play_movie")
	{
	  action_play();
	}
      else if (input.command == "reget")
	{
	  reget_movie_information();
	}
      else if (input.command == "search_imdb")
	{
	  search_imdb();
	}
      else if (input.command == "info" && input.mode == "graphical" && 
	       vector_lookup(files, folders.top().second).type != "multi-dir") 
	{
	  print_info();
	}
      else if (input.command == "action")
	{
	  if (vector_lookup(files, folders.top().second).type != "file")
	    enter_dir();
	  else
	    action_play();
	}
      else if (input.command == "search")
	{
	  search_func();
        }
      else if (input.command == "second_action")
	{
	  secondary_menu();
	}
      else
	MovieTemplate<CIMDBMovie>::movie_mainloop_common(input);

      update_needed != global->check_commands(input);
    }

  exit_loop = false;

  return "";
}

vector<CIMDBMovie> GraphicalMovie::parse_dir(const list<string>& dirs)
{
  vector<CIMDBMovie> vec_files;

  for (list<string>::const_iterator iter = dirs.begin(); iter != dirs.end(); ++iter)
    {
      vector<CIMDBMovie> tempfiles = rdir(*iter);

      for (vector<CIMDBMovie>::const_iterator iter2 = tempfiles.begin(); iter2 != tempfiles.end(); ++iter2)
	vec_files.push_back(*iter2);
    }

  return vec_files;
}

vector<string> GraphicalMovie::check_db_for_folders(const string& parent)
{
  vector<string> names;

  string good_parent = string_format::unique_folder_name(parent);

  SQLQuery *q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(good_parent) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    assert(q->numberOfTuples() == 1);
    SQLRow &row = (*q)[0];
    string parent_id = row["id"];
    SQLQuery *q2 = db.query("Folders", ("SELECT filename FROM %t WHERE parent='" + parent_id + "'").c_str());
    if (q2) {
      for (int i = 0; i < q2->numberOfTuples(); ++i) {
	SQLRow &row = (*q2)[i];
	names.push_back(row["filename"]);
      }
    }
    delete q2;
  }
  delete q;

  return names;
}

bool GraphicalMovie::rdir_internal(const string& original_name, const string& argv, 
				   vector<CIMDBMovie> &cur_files, bool& db_load_succes)
{
  if (global->check_stop_bit()) {
    cur_files.clear();
    return false;
  }

  string filename = original_name;

  bool inside_movie_directory = false;
  list<string> filenames;

  // if the directory where the files reside is known to be a movie, use it
  // instead of trying to fetch information. But use the names of the files
  // to differentiate files like CD1.avi CD2.avi etc.

  string small_name = "";

  if (filename[filename.size()-1] == '/')
    filename = filename.substr(0, filename.size()-1);
      
  string::size_type pos_parent = filename.rfind("/");
  if (pos_parent != string::npos) {
    small_name = filename.substr(pos_parent+1);
    filename = filename.substr(0, pos_parent);
  }

#if 0
  string small_parent_name = "";

  if (filename[filename.size()-1] == '/')
    filename = filename.substr(0, filename.size()-1);
      
  pos_parent = filename.rfind("/");
  if (pos_parent != string::npos)
    small_parent_name = filename.substr(pos_parent+1);

  bool root_dir = false;

  list<string> root_dirs = movie_conf->p_movie_dirs();
  for (list<string>::const_iterator i_root = root_dirs.begin(), 
	 end = root_dirs.end(); i_root != end; ++i_root)
    if ((*i_root) == filename) {
      root_dir = true;
      break;
    }

  bool inside_movie_directory = false;

  list<string> filenames;

  // lets see if the dir is recognized as a movie
  if (!root_dir && file_exists(imdb_dir + small_parent_name)) {

    inside_movie_directory = true;
	
    // skip not recognizable files
    if (!isDirectory (original_name)) {
      if (check_type(i->getName(), movie_conf->p_filetypes_m()) == emptyMyPair)
	continue;
      else
	filenames.push_back(original_name);
    } else {
      bool good_dir = false;
      for (file_iterator<file_t, default_order> i2 (original_name); i2 != i2.end (); ++i2) {
	if (check_type(i2->getName(), movie_conf->p_filetypes_m()) != emptyMyPair) {
	  filenames.push_back(i2->getName());
	  good_dir = true;
	  break;
	}
      }
      if (!good_dir)
	continue;
    }
  } else
#endif
    filename = original_name;
      
  if (filename[filename.size()-1] == '/')
    filename = filename.substr(0, filename.size()-1);
      
  if (isDirectory (filename)) { // dir
    Multifile dir = add_dir(filename);
    dir.path += "/";
    if (dir.filenames.size() != 0 || dir.filetype == "dir") {
      CIMDBMovie m = CIMDBMovie(dir);
      m.db_id = db_id(m.path, true);
      if (m.Load(physical, db))
	db_load_succes = true;
      if (inside_movie_directory) {
	m.real_name = small_name;
	if (conf->p_convert())
	  m.real_name = string_format::convert(m.real_name);
	m.type = "file"; // we don't want to enter anything inside a
	// movie directory
	m.filenames = filenames;
      }
      cur_files.push_back(m);
    }
  } else { // file
    MyPair filetype = check_type(filename, movie_conf->p_filetypes_m());
    if (filetype != emptyMyPair) {
      Multifile e = addfile(filename, filetype);
      CIMDBMovie m = CIMDBMovie(e);
      m.db_id = db_id(filename, false);
      if (m.Load(physical, db))
	db_load_succes = true;
      cur_files.push_back(m);
    }
  }

  return true;
}

void GraphicalMovie::insert_file_into_db(const string& filename, const string& parent)
{
  string parent_id = "";
  
  string good_parent = string_format::unique_folder_name(parent);
  
  SQLQuery *q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(good_parent) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    assert(q->numberOfTuples() == 1);
    SQLRow &row = (*q)[0];
    parent_id = row["id"];
  }
  delete q;

  if (parent_id.empty()) {
    char *tmp = sqlite3_mprintf("INSERT INTO Folders VALUES(NULL, '0', '%q', '', '1')", good_parent.c_str());
    db.execute(tmp);
    sqlite3_free(tmp);

    q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(good_parent) + "'").c_str());
    if (q && q->numberOfTuples() > 0) {
      assert(q->numberOfTuples() == 1);
      SQLRow &row = (*q)[0];
      parent_id = row["id"];
    }
    delete q;
  }

  bool is_dir = isDirectory(filename);

  string good_filename;
  string cut_filename;
  
  if (is_dir) {
    good_filename = string_format::unique_folder_name(filename);
    cut_filename = good_filename.substr(0, good_filename.size()-1);
  } else {
    good_filename = filename;
    cut_filename = filename;
  }

  string::size_type i;
  if ((i = cut_filename.rfind('/')) == string::npos)
    i = 0;
  string name = cut_filename.substr((i != 0) ? i+1 : i);
  
  if (!is_dir && (i = name.rfind('.')) != string::npos)
    name = name.substr(0, i);

  char *tmp_insert = sqlite3_mprintf("INSERT INTO Folders VALUES(NULL, '%q', '%q', '%q', '%q')", 
				     parent_id.c_str(), good_filename.c_str(), name.c_str(),
				     conv::itos(is_dir).c_str());
  db.execute(tmp_insert);
  sqlite3_free(tmp_insert);
}

int GraphicalMovie::db_id(const string& file, bool is_dir)
{
  int i = -1;

  string good_file;

  if (is_dir)
    good_file = string_format::unique_folder_name(file);
  else
    good_file = file;

  SQLQuery *q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(good_file) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    SQLRow &row = (*q)[0];
    i = conv::atoi(row["id"]);
  }
  delete q;

  return i;
}

// read a dir and spit out a list of files and dirs. 
vector<CIMDBMovie> GraphicalMovie::rdir(const string& argv)
{
  vector<CIMDBMovie> cur_files;

  vector<string> db_files = check_db_for_folders(argv);
  
  bool db_load_succes = false;

  if (db_files.size() > 0) {
    for (vector<string>::const_iterator i = db_files.begin(), iend = db_files.end();
	 i != iend; ++i)
      if (!rdir_internal(*i, argv, cur_files, db_load_succes))
	break;
  } else {
    for (file_iterator<file_t, default_order> i (argv); i != i.end (); i.advance(false)) {
      string filename = filesystem::FExpand(i->getName());
      insert_file_into_db(filename, argv);
      if (!rdir_internal(filename, argv, cur_files, db_load_succes))
	break;
    }
  }

  // check if it's a root dir, we don't want to display the message for all
  // folders
  bool root_dir = false;

  list<string> root_dirs = movie_conf->p_movie_dirs();
  for (list<string>::const_iterator i_root = root_dirs.begin(), 
	 end = root_dirs.end(); i_root != end; ++i_root)
    if ((*i_root) == argv) {
      root_dir = true;
      break;
    }

  if (!db_load_succes && !imdb_message_not_displayed && root_dir) {
    DialogWaitPrint pdialog(5000);
    pdialog.add_line(gettext("No IMDb information found about movies"));
    pdialog.add_line(gettext("Use the reget movie information action"));
    pdialog.add_line(gettext("to download information about a movie"));
    pdialog.print();
    imdb_message_not_displayed = true;
  }

#if 0
  // this means that we tried to enter a dir and we just can't go any further,
  // happens if you are inside a movie dir and you try to enter another
  // directory.
  if (cur_files == files) {
    vector<CIMDBMovie> cur_files_empty;
    cur_files = cur_files_empty;
  }
#endif

  return cur_files;
}

void GraphicalMovie::save_runtime_settings()
{
  std::ofstream file;

  string path = conf->p_var_data_dir() + "options/Movie Graphical Runtime";

  file.open(path.c_str());
    
  if (!file) {
    DebugPrint perror(gettext("Could not write options to file ") + path,
		      Print::DEBUGGING, DebugPrint::CRITICAL, "MOVIE");
  } else {
    file << "imdb_warning_displayed," << imdb_message_not_displayed << std::endl;
  }

  file.close();
}

void GraphicalMovie::load_runtime_settings()
{
  // sane defaults
  bool imdb_warning = false;

  std::string path = conf->p_var_data_dir() + "options/Movie Graphical Runtime";

  std::ifstream in;
  in.open(path.c_str(), std::ios::in);
  
  if (!in.is_open()) {
    DebugPrint perror(gettext("Could not open options file ") + path,
                      Print::DEBUGGING, DebugPrint::CRITICAL, "MOVIE");
  } else {
   
    string name, value, str;
    
    while (getline(in, str)) {

      string::size_type comma = str.find(",");

      if (comma == string::npos)
        continue;
      
      name = str.substr(0,comma);
      value = str.substr(comma+1);

      if (name == "imdb_warning_displayed")
	imdb_warning = conv::stob(value);
    }
    in.close();
  }

  imdb_message_not_displayed = imdb_warning;
}

