// used for #ifdef
#include "config.h"

#include "movie.hpp"

//Input
#include "input.hpp"

#include "common.hpp"
#include "common-feature.hpp"

#include "gettext.hpp"

#include "extra_menu.hpp"
#include "updater.hpp"
#include "busy_indicator.hpp"
#include "shutdown.hpp"

#include "plugins.hpp"

#include "graphics.hpp"

#include <iostream>

#include <sys/shm.h>

using std::list;
using std::vector;
using std::string;

Movie::MovieOpts::MovieOpts() 
  : players_set_correctly(false)
{
  // header
  header = "Movie Options";
  translated_header = gettext("Movie Options");

  // values

  // reload
  vector<string> none;
  reload_p = new Option(false, gettext("reload"), "reload", 0, none, none);
  val.push_back(reload_p);

  // global options
  val.push_back(go->dir_order);

  values_added = 2;

  if (conf->p_cdpath().size() > 1) {
    val.push_back(go->cd_device);
    ++values_added;
  }

  if (go->audio_device->values.size() > 1) {
    val.push_back(go->audio_device);
    ++values_added;
  }

  lang_values.push_back("en");
  lang_values.push_back("de");
  lang_values.push_back("es");
  lang_values.push_back("nl");
  lang_values.push_back("fr");
  lang_values.push_back("pt");
  lang_values.push_back("hu");
  lang_values.push_back("da");
  lang_values.push_back("sv");
  lang_values.push_back("no");
  lang_values.push_back("fi");
  lang_values.push_back("it");
  lang_values.push_back("pl");

  vector<string> sub_lang_values;

  sub_lang_values = lang_values;
  sub_lang_values.push_back(gettext("none"));

  sub_lang_p = new Option(false, gettext("subtitle language"), "subtitle language", 0, 
			  sub_lang_values, sub_lang_values);
  val.push_back(sub_lang_p);
}

void Movie::MovieOpts::clear() 
{
  val.clear();
}

void Movie::MovieOpts::set_players_correctly()
{
  vector<string> movie_player_values;
  vector<string> english_movie_player_values;
  
  vector<string> vcd_player_values;
  vector<string> english_vcd_player_values;

  vector<string> dvd_player_values;
  vector<string> english_dvd_player_values;

  Plugins *plugins = S_Plugins::get_instance();
  for (vector<MoviePlayerPlugin*>::const_iterator iter=plugins->movie_players.begin(), 
	 iend=plugins->movie_players.end(); iter!=iend; ++iter) {
    if ((*iter)->player->movie_playback())
      movie_player_values.push_back((*iter)->plugin_name());
    if ((*iter)->player->vcd_playback())
      vcd_player_values.push_back((*iter)->plugin_name());
    if ((*iter)->player->dvd_playback())
      dvd_player_values.push_back((*iter)->plugin_name());
  }

  english_movie_player_values = movie_player_values;
  english_vcd_player_values = vcd_player_values;
  english_dvd_player_values = dvd_player_values;

  // FIXME: check that at least one of each is available

  movie_player_p = new Option(false, gettext("movie player"), "movie player", 0, 
			      movie_player_values, english_movie_player_values);
  val.push_back(movie_player_p);

  vcd_player_p = new Option(false, gettext("vcd player"), "vcd player", 0, 
			    vcd_player_values, english_vcd_player_values);
  val.push_back(vcd_player_p);

  dvd_player_p = new Option(false, gettext("dvd player"), "dvd player", 0, 
			    dvd_player_values, english_dvd_player_values);
  val.push_back(dvd_player_p);

  if (movie_player() == "MPlayer")
    vcd_mplayer_construct();
  
  if (dvd_player() == "MPlayer")
    dvd_mplayer_construct();

  players_set_correctly = true;
}

void Movie::MovieOpts::dvd_mplayer_construct()
{
  dvd_audio_lang_p = new Option(false, gettext("dvd audio language"), "dvd audio language", 
				0, lang_values, lang_values);
  val.push_back(dvd_audio_lang_p);

  vector<string> dvdtitle_values;
  
  for (int i = 1; i < 100; ++i)
    dvdtitle_values.push_back(conv::itos(i));
  
  dvdtitle_p = new Option(false, gettext("dvd title"), "dvd title", 0, 
			  dvdtitle_values, dvdtitle_values);
  val.push_back(dvdtitle_p);
}

void Movie::MovieOpts::vcd_mplayer_construct()
{
  vector<string> vcdtitle_values;
    
  for (int i = 1; i < 100; ++i)
    vcdtitle_values.push_back(conv::itos(i));
    
  vcdtitle_p = new Option(false, gettext("vcd title"), "vcd title", 0, 
			  vcdtitle_values, vcdtitle_values);
  val.push_back(vcdtitle_p);
}

Movie::MovieOpts::~MovieOpts() 
{
  save();

  delete reload_p;
  delete sub_lang_p;

  if (players_set_correctly) {
    if (movie_player() == "MPlayer")
      delete vcdtitle_p;
    
    if (dvd_player() == "MPlayer") {
      delete dvdtitle_p;
      delete dvd_audio_lang_p;
    }
  }
}

string Movie::MovieOpts::sub_lang()
{
  return val[values_added]->english_values[val[values_added]->pos];
}

string Movie::MovieOpts::dvd_audio_lang() 
{
  if (movie_player() == "MPlayer")
    return val[values_added+4]->values[val[values_added+4]->pos];
  else
    return "";
}

string Movie::MovieOpts::vcdtitle()
{
  if (movie_player() == "MPlayer")
    return val[values_added+5]->values[val[values_added+5]->pos];
  else
    return "";
}

string Movie::MovieOpts::dvdtitle()
{
  if (movie_player() == "MPlayer")
    return val[values_added+6]->values[val[values_added+6]->pos];
  else
    return "";
}

string Movie::MovieOpts::movie_player()
{
  if (val[values_added]->values.size() > 0)
    return val[values_added+1]->values[val[values_added+1]->pos];
  else
    return "";
}

string Movie::MovieOpts::vcd_player()
{
  if (val[values_added+1]->values.size() > 0)
    return val[values_added+2]->values[val[values_added+3]->pos];
  else
    return "";
}

string Movie::MovieOpts::dvd_player()
{
  if (val[values_added+2]->values.size() > 0)
    return val[values_added+3]->values[val[values_added+3]->pos];
  else
    return "";
}

void Movie::MovieOpts::read_dirs() 
{ 
  Movie *mov = get_class<Movie>("Movie");

  mov->read_dirs();
  mov->reset();
}

bool Movie::MovieOpts::mainloop()
{
  bool mov_mplayer_before = false;
  if (movie_player() == "MPlayer")
    mov_mplayer_before = true;
  
  bool dvd_mplayer_before = false;
  if (dvd_player() == "MPlayer")
    dvd_mplayer_before = true;

  bool status = Options::mainloop();

  if (movie_player() != "MPlayer" && mov_mplayer_before) {
    delete vcdtitle_p;
    for (vector<Option*>::iterator i = val.begin(), iend = val.end();
	 i != iend; ++i)
      if ((*i)->english_name == "vcd title") {
	val.erase(i);
	break;
      }
  } else if (dvd_player() != "MPlayer" && dvd_mplayer_before) {
    delete dvdtitle_p;
    delete dvd_audio_lang_p;

    for (vector<Option*>::iterator i = val.begin(), iend = val.end();
	 i != iend; ++i) {
      if ((*i)->english_name == "dvd audio language")
	i = val.erase(i);
      else if ((*i)->english_name == "dvd title")
	i = val.erase(i);
    }
  } else if (movie_player() == "MPlayer" && !mov_mplayer_before)
    vcd_mplayer_construct();
  else if (dvd_player() == "MPlayer" && !dvd_mplayer_before)
    dvd_mplayer_construct();

  return status;
}

Movie::Movie()
  : active_player(0), movie_playing(false)
{
  movie_conf = S_MovieConfig::get_instance();

  input_master->parse_keys("movie", "2");

  reset();
}

void Movie::save_runtime_settings()
{}

void Movie::load_runtime_settings()
{}

void Movie::reset()
{
  for (int i = 0, size = folders.size(); i < size; ++i)
    folders.pop();

  folders.push(std::make_pair(movie_conf->p_movie_dirs(), 0));
}

void Movie::startup_updater()
{
  load_runtime_settings();

  ConstFunctor<Movie, string, void> *playback_func = 
    new ConstFunctor<Movie, string, void>(this, &Movie::external_movie_playback);
  global->register_movie_playback_func(playback_func);

  opts.set_players_correctly();
  add_playback_keys();
}

void Movie::add_playback_keys()
{
  Playback p = Playback("Movie");
  p.add_command(GlobalCommand(new TimeFunctorv<Movie>(this, &Movie::player_play), "play", gettext("Play")));
  p.add_command(GlobalCommand(new TimeFunctorv<Movie>(this, &Movie::player_pause), "pause", gettext("Pause")));
  p.add_command(GlobalCommand(new TimeFunctorv<Movie>(this, &Movie::player_stop), "stop", gettext("Stop")));
  p.add_command(GlobalCommand(new TimeFunctorv<Movie>(this, &Movie::player_ff), "ff", gettext("Fast forward")));
  p.add_command(GlobalCommand(new TimeFunctorv<Movie>(this, &Movie::player_fb), "fb", gettext("Fast backward")));
  global->add_playback(p);
}

bool Movie::active_control_player()
{
  return (active_player != 0 && active_player->control_playback());
}

void Movie::player_play()
{
  if (active_control_player())
    active_player->play();
}

void Movie::player_pause()
{
  if (active_control_player())
    active_player->pause();
}

void Movie::player_stop()
{
  if (active_control_player())
    active_player->stop();
}

void Movie::player_ff()
{
  if (active_control_player())
    active_player->ff();
}

void Movie::player_fb()
{
  if (active_control_player())
    active_player->fb();
}

void Movie::player_fullscreen()
{
  if (active_control_player())
    active_player->switch_screen_size();
}

bool Movie::playback_command(const Input& i)
{
  return (i.command == "play" || i.command == "pause" || i.command == "stop" || 
	  i.command == "ff" || i.command == "fb");
}

void Movie::fullscreen_check(const Input& i)
{
  if (!playback_command(i))
    if (active_control_player() && active_player->fullscreen() && active_player->playing())
      player_fullscreen();
}

bool Movie::file_sort::operator()(const Multifile& lhs, const Multifile& rhs)
{
  return sort_order(lhs.lowercase_name, rhs.lowercase_name, !lhs.path.empty(), !rhs.path.empty());
}

void Movie::options()
{
  BusyIndicator::get_instance().idle();

  if (opts.mainloop()) {
    // reload
    load_current_dirs();

    // FIXME: can this be right?

    folders.top().second = 0;
  }
}

void Movie::exit()
{
  exit_loop = true;
}

void Movie::add_standard(ExtraMenu& em)
{
  if (render->device->supports_wid())
    em.add_item(ExtraMenuItem(gettext("Switch movie playback to fullscreen"), "", 
			      new TimeFunctorv<Movie>(this, &Movie::player_fullscreen)));

  em.add_item(ExtraMenuItem(gettext("Search"), input_master->find_shortcut("search"), 
			    new TimeFunctorv<Movie>(this, &Movie::search_func)));
  em.add_item(ExtraMenuItem(gettext("Options"), input_master->find_shortcut("options"), 
			    new TimeFunctorv<Movie>(this, &Movie::options)));
}

string Movie::get_name_from_file(const Multifile& e)
{
  return e.name;
}

bool Movie::search_compare(const Multifile& s)
{
  return (s.lowercase_name.substr(0, search_str.size()) == lowercase_search_str);
}

std::pair<Movie::SUBTYPE, vector<string> > Movie::find_subtitles(const string& dir)
{
  SUBTYPE type = NOSUB;

  vector<string> subtitles;

  for (file_iterator<file_t, default_order> i (dir); i != i.end (); ++i) {
    string name = i->getName();
    if (name.substr(name.size()-3) == "srt") {
      if (type == NOSUB)
	type = SRT;
      subtitles.push_back(name);
    } else if (name.substr(name.size()-3) == "idx") {
      if (type == NOSUB)
	type = IDX;
      subtitles.push_back(name.substr(0, name.size()-4));
    }
  }

  return std::make_pair(type, subtitles);
}

void Movie::external_movie_playback(const string& path)
{
  Multifile m;
  m.filetype = "file";
  m.filenames.push_back(path);

  playmovie(m, true);
}

void Movie::playmovie(const Multifile file, bool window)
{
  DialogWaitPrint pdialog(gettext("Starting Playback..."), 1000);

  Plugins *plugins = S_Plugins::get_instance();
  MoviePlayerPlugin *plugin = plugins->find(plugins->movie_players, opts.movie_player());
  active_player = plugin->player;

  if (active_player->wants_exclusivity())
    render->device->unlock();

  BusyIndicator::get_instance().disable();

  if (file.filetype == "dvd") {
    play_dvd_helper(file.filenames.front());
    if (active_player->wants_exclusivity())
      render->device->lock();
    BusyIndicator::get_instance().enable();
    return;
  } else {

    string subcommand = "";
    vector<string> empty_path;
    std::pair<SUBTYPE, vector<string> > subtitles = std::make_pair(NOSUB, empty_path);

    string cur_files = "";

    // support playlist files
    if (file.filetype == "pls")
      cur_files += " -playlist ";

    if (opts.movie_player() == "Xine") {
      if (subtitles.second.size() != file.filenames.size())
	for (list<string>::const_iterator iter = (file.filenames).begin(); iter != (file.filenames).end(); ++iter)
	  cur_files += " \"" + (*iter) + "\"";
      else {
	int i = 0;
	for (list<string>::const_iterator iter = (file.filenames).begin(); iter != (file.filenames).end(); ++iter, ++i)
	  cur_files += " \"" + (*iter) + "#subtitle:" + vector_lookup(subtitles.second, i) + "\"";
      }
    } else if (opts.movie_player() == "MPlayer")
      for (list<string>::const_iterator iter = (file.filenames).begin(); iter != (file.filenames).end(); ++iter)
	cur_files += " \"" + (*iter) + "\"";

    active_player->play_movie(cur_files, window);
  }
  
  if (active_player->wants_exclusivity())
    render->device->lock();

  BusyIndicator::get_instance().enable();
}

string Movie::testdir(int *count)
{
  *count = 0;

  for (file_iterator<file_t, default_order> i (cd->get_mount_point()); i != i.end (); ++i) 
    {
      if (global->check_stop_bit()) {
	*count = 0;
	break;
      }

      string filename = filesystem::FExpand(i->getName());
	
      if (isDirectory (filename)) {

	string cur_dir = string_format::lowercase(filename.substr(filename.rfind("/")+1));
	if (cur_dir == "svcd")
	  return "svcd";

	if (cur_dir == "vcd")
	  return "vcd";

	if (cur_dir == "video_ts")
	  return "dvd";
      } else {
	if (check_type(filename, movie_conf->p_filetypes_m()) != emptyMyPair)
	  ++(*count);
      }
    }

  if (*count == 0)
    return "empty";
  else
    // none of the above found, it must be a divx
    return "divx";
}

// play a vcd
void Movie::playvcd()
{
  render->device->unlock();

  //play disc
  Plugins *plugins = S_Plugins::get_instance();
  MoviePlayerPlugin *plugin = plugins->find(plugins->movie_players, opts.vcd_player());
  plugin->player->play_vcd(cd->get_device());

  render->device->lock();
}

// play a dvd
void Movie::playdvd()
{
  render->device->unlock();

  play_dvd_helper(cd->get_device());

  render->device->lock();
}

void Movie::play_dvd_helper(const string& device)
{
  //play disc
  Plugins *plugins = S_Plugins::get_instance();
  MoviePlayerPlugin *plugin = plugins->find(plugins->movie_players, opts.dvd_player());
  plugin->player->play_dvd(device);
}

// ====================================================================
//
// dir
//
// ====================================================================

// Add a file to the list
Multifile Movie::addfile(const string& name, const MyPair& filetype)
{
  Multifile r;
  r.id = ++id;
  r.filenames.push_back(name);
  string tempname = name.substr(0, name.size()-((filetype.first).size()+1));
  string::size_type pos = tempname.rfind("/");
  if (pos != string::npos)
    r.name = tempname.substr(pos+1);
  else
    r.name = tempname;
  if (conf->p_convert())
    r.name = string_format::convert(r.name);
  r.lowercase_name = string_format::lowercase(r.name);
  r.type = "file";
  r.filetype = filetype.second;

  return r;
}

// Add a moviedir as "one file" to the movielist
Multifile Movie::add_dir(const string& filename)
{
  Multifile output;

  string type = "";

  vector<string> known_filetypes;

  int nr_of_elements = 0;

  for (file_iterator<file_t, default_order> i (filename); i != i.end (); i.advance(false))
    {
      string name = i->getName();

      if (string_format::lowercase(name).rfind("video_ts") != string::npos) {
	type = "dvd";
	break;
      }

      string short_name = name.substr(name.size()-3, 2);

      if (string_format::lowercase(short_name) == "cd") {
	type = "cd";
	break;
      } else if (check_type(name, movie_conf->p_filetypes_m()) != emptyMyPair) {
	known_filetypes.push_back(name);
	type = "movie";
      }

      ++nr_of_elements;
    }

  if (type == "dvd") {

    output.id = ++id;
    string::size_type pos = filename.rfind("/");
    output.name = filename.substr(pos+1);
    output.type = "dir-single"; 
    output.path = filename;
    output.filetype = "dvd";

    for (file_iterator<file_t, default_order> i (filename); 
	 i != i.end (); i.advance(false)) {
      if (string_format::lowercase(i->getName()).rfind("/video_ts") != string::npos) {
	output.filenames.push_back(i->getName());
	break;
      }
    }

  } else if (type == "cd") {

    string::size_type pos = filename.rfind("/");

    output.id = ++id;
    output.name = filename.substr(pos+1);
    output.type = "dir-single"; 
    output.path = filename;

    for (file_iterator<file_t, default_order> i (filename); i != i.end (); i.advance(false)) {
      file_t cur_file = *i;
      
      string name = cur_file.getName();
      string short_name = name.substr(name.size()-3, 2);

      if (string_format::lowercase(short_name) == "cd") {
	for (file_iterator<file_t, default_order> i2 (name); i2 != i2.end(); i2.advance(false)) {
	  cur_file = *i2;

	  const MyPair filetype = check_type(cur_file.getName(), movie_conf->p_filetypes_m());
	  if (filetype != emptyMyPair) {
	    output.filetype = filetype.second;
	    output.filenames.push_back((*i2).getName());
	  }
	}
      }
    }

  } else if (!known_filetypes.empty()) {
    output.id = ++id; 
    string::size_type pos = filename.rfind("/");
    output.name = filename.substr(pos+1);
    output.path = filename;

    output.type = "dir-single";
    if (known_filetypes.size() > 1) {
      string first = known_filetypes.front();
      vector<string>::iterator i = known_filetypes.begin();
      ++i;
      for (vector<string>::iterator end = known_filetypes.end();
	   i != end; ++i) 
	if (string_format::difference(first, *i) > 1) {
	  output.type = "dir-multi";
	  break;
	}
    }

    if (conf->p_convert())
      output.name = string_format::convert(output.name);

    const MyPair filetype = check_type(known_filetypes.front(), movie_conf->p_filetypes_m());
    output.filetype = filetype.second; 
    for (vector<string>::iterator i = known_filetypes.begin(), end = known_filetypes.end();
	 i != end; ++i)
      output.filenames.push_back(*i);

  } else {

    output.id = ++id;
    string::size_type pos = filename.rfind("/");
    output.name = filename.substr(pos+1);
    if (nr_of_elements > 1)
      output.type = "dir-multi"; 
    else
      output.type = "dir"; 
    output.path = filename;
    output.filetype = "dir";
  }

  output.lowercase_name = string_format::lowercase(output.name);

  return output;
}
