// used for #ifdef
#include "config.h"

#include "libfspp.hpp"

#include "pictures.hpp"

//Input
#include "input.hpp"

#include "config.hpp"
#include "common.hpp"
#include "functor.hpp"
#include "graphics.hpp"
#include "updater.hpp"
#include "common-feature.hpp"
#include "gettext.hpp"

#include "search.hpp"

#include "startmenu.hpp"
#include "busy_indicator.hpp"
#include "shutdown.hpp"
#include "extra_menu.hpp"

#ifdef use_inotify
#include "notify.hpp"
#endif

// ostringstream
#include <sstream>

#include <iostream>

#include <unistd.h>

using std::list;
using std::vector;
using std::string;

Pictures::Pictures() 
  : db((conf->p_var_data_dir() + "pictures.db").c_str()), 
    exit_loop(false), orientation(0), 
    zoom_level(1), zoom_position(1), 
    thumbnailed_all_files(false), pause(false)
{
  input_master->parse_keys("pictures", "2");

  pic_conf = S_PictureConfig::get_instance();

  reset();

  y_start = 100;

  if (conf->p_v_res() == 640)
    y_start = 30;
  else if (conf->p_v_res() == 720)
    y_start = 70;
  else if (conf->p_v_res() == 800)
    y_start = 90;
  else if (conf->p_v_res() == 1024)
    y_start = 110;
  else if (conf->p_v_res() == 1280)
    y_start = 130;

  image_width = 125;
  image_height = 96;

  if (conf->p_h_res() == 405) {
    image_width = 80;
    image_height = 67;
  }

  // including text and space around image
  image_width_all = image_width+25;
  image_height_all = image_height+55;
  image_height_all_eks_text = image_height + 6;

  images_per_row = (conf->p_v_res()-2*25)/image_width_all;

  rows = (conf->p_h_res()-2*y_start+20)/image_height_all;
  rows_search = (conf->p_h_res()-2*y_start+20-40)/image_height_all;

  image_height_all = (conf->p_h_res()-2*y_start+20)/rows;
  image_height_all_search = (conf->p_h_res()-2*y_start+20-40)/rows_search;

  // lets make some more room for text 
  image_width_all += ((conf->p_v_res()-2*25)%image_width_all)/images_per_row;

  check_db();
}

void Pictures::reset()
{
  for (int i = 0, size = folders.size(); i < size; ++i)
    folders.pop();

  list<string> dirs = pic_conf->p_picture_dirs();
  for (list<string>::iterator iter = dirs.begin(); iter != dirs.end(); ++iter)
    if ((*iter)[iter->size()-1] == '/')
      *iter = iter->substr(0, iter->size()-1);

  folders.push(std::make_pair(dirs, 0)); 
}

void Pictures::startup_updater()
{
  BackgroundUpdater *background_updater = S_BackgroundUpdater::get_instance();
  TimeFunctori<Pictures> *time_func = new TimeFunctori<Pictures>(this, &Pictures::check_thumbnail);
  TimeFunctorv<Pictures> *call_func = new TimeFunctorv<Pictures>(this, &Pictures::background_thumbnails);
  background_updater->timer.add(TimeElement("extra_thumbnails", time_func, call_func));

  EnterFunctor<Pictures> *e = new EnterFunctor<Pictures>(this, &Pictures::reenter);
  SearchGenList<Pictures> *s = new SearchGenList<Pictures>(this, &Pictures::gen_search_list);
  S_Search::get_instance()->register_module(SearchModule("Pictures", s, e));
}
  
Pictures::PicOpts::PicOpts() 
{
  // header
  header = "Picture Options";
  translated_header = gettext("Picture Options");

  // values

  // reload
  vector<string> none;
  reload_p = new Option(false, gettext("reload"), "reload", 0, none, none);
  val.push_back(reload_p);

  // sorting order
  val.push_back(go->dir_order);

  if (conf->p_cdpath().size() > 1) 
    val.push_back(go->cd_device);

  vector<string> slideshow_values;

  for (int i = 0; i < 120; ++i)
    slideshow_values.push_back(conv::itos(i));

  slideshow_p = new Option(false, gettext("slideshow delay"), "slideshow delay", 10, 
			   slideshow_values, slideshow_values);
  val.push_back(slideshow_p);

  vector<string> zoom_values;

  zoom_values.push_back(gettext("no"));
  zoom_values.push_back(gettext("yes"));

  vector<string> english_zoom_values;

  english_zoom_values.push_back("no");
  english_zoom_values.push_back("yes");

  zoom_p = new Option(false, gettext("zoom to fullscreen"), "zoom to fullscreen", 0, 
		      zoom_values, english_zoom_values);
  val.push_back(zoom_p);

  vector<string> preview_values;

  preview_values.push_back(gettext("no"));
  preview_values.push_back(gettext("yes"));

  vector<string> english_preview_values;

  english_preview_values.push_back("no");
  english_preview_values.push_back("yes");

  preview_p = new Option(false, gettext("show previews"), "show previews", 1, 
			 preview_values, english_preview_values);
  val.push_back(preview_p);

  vector<string> recurse_values;

  recurse_values.push_back(gettext("no"));
  recurse_values.push_back(gettext("yes"));

  vector<string> english_recurse_values;

  english_recurse_values.push_back("no");
  english_recurse_values.push_back("yes");

  recurse_p = new Option(false, gettext("recurse dirs in fullscreen"), "recurse dirs in fullscreen", 0, 
			 recurse_values, english_recurse_values);
  val.push_back(recurse_p);

  if (conf->p_idle_time() != 0) {
    vector<string> thumbnail_extract_values;

    thumbnail_extract_values.push_back(gettext("no")); 
    thumbnail_extract_values.push_back(gettext("yes"));

    vector<string> english_thumbnail_extract_values;

    english_thumbnail_extract_values.push_back("no"); 
    english_thumbnail_extract_values.push_back("yes");

    thumbnail_extract_p = new Option(false, gettext("bg thumbnail generation"), "bg thumbnail generation", 0, 
				     thumbnail_extract_values, english_thumbnail_extract_values);
    val.push_back(thumbnail_extract_p);
  }
}

Pictures::PicOpts::~PicOpts() 
{
  save();

  delete reload_p;
  delete slideshow_p;
  delete zoom_p;
  delete preview_p;
  delete recurse_p;
  if (conf->p_idle_time() != 0) {
    delete thumbnail_extract_p;
  }
}

void Pictures::PicOpts::read_dirs() 
{ 
  Pictures *pictures = get_class<Pictures>("Picture");
  pictures->read_dirs(); 
  pictures->reset();
}

void Pictures::create_db()
{
  vector<string> fields;
  fields.push_back("version INTEGER NOT NULL");
  db.createTable("Schema", fields);

  db.execute("INSERT INTO Schema VALUES('1')");

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("parent INTEGER");
  fields.push_back("filename TEXT");
  fields.push_back("name TEXT");
  fields.push_back("is_folder INTEGER"); // 1 = true
  db.createTable("Folders", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("name TEXT");
  db.createTable("Group", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("pid INTEGER NOT NULL");
  fields.push_back("gid INTEGER NOT NULL");
  db.createTable("GPicture", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("filename TEXT");
  fields.push_back("title TEXT");
  fields.push_back("comment TEXT");
  fields.push_back("date INTEGER");
  fields.push_back("gid INTEGERL");
  fields.push_back("rating INTEGER");
  fields.push_back("rotation INTEGER");
  db.createTable("Pictures", fields);
}

void Pictures::check_db()
{
  if (db.hasTable("Schema")) {
    // check for correct version
    SQLQuery *q = db.query("Schema", "SELECT version FROM %t");
    if (q && q->numberOfTuples() > 0) {
      SQLRow &row = (*q)[0];
      if (row["version"] != "1")
	; // for upgrading db
    }
    delete q;
  } else {
    // create the whole db
    create_db();
  }
}

int Pictures::get_db_orientation(int db_id)
{
  int db_orientation = 0;

  SQLQuery *q = db.query("Pictures", ("SELECT * FROM %t WHERE id='" + conv::itos(db_id) + "'").c_str());
  if (q && q->numberOfTuples() > 0)
    db_orientation = conv::atoi((*q)[0]["rotation"]);

  return db_orientation;
}

void Pictures::set_db_orientation(int db_id)
{
  db.execute(("UPDATE Pictures SET rotation = '" + conv::itos(orientation) + "' WHERE id = '" + conv::itos(db_id) + "'").c_str());
}

int Pictures::find_position_in_folder_list(const list<string>& files, const string& filename)
{
  int pos = 0;
  bool pos_found = false;
  for (list<string>::const_iterator i = files.begin(), iend = files.end(); i != iend; ++i){
    vector<Picture> templist = rdir(*i);
    pos = 0;
    for (vector<Picture>::iterator fi = templist.begin(), fiend = templist.end(); fi != fiend; ++fi, ++pos)
      if (filename == fi->path) {
	pos_found = true;
	break;
      }
    if (pos_found)
      break;
  }
  return pos;
}

void Pictures::reenter(int db_id)
{
  reset();

  SQLQuery *q = db.query("Pictures", ("SELECT * FROM %t WHERE id='" + conv::itos(db_id) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {

    string filename = (*q)[0]["filename"];
    string cur_dir_path;

    list<string> dirs = pic_conf->p_picture_dirs();

    int pos;
    for (list<string>::iterator i = dirs.begin(), iend = dirs.end(); i != iend; ++i)
      if ((pos = filename.rfind(*i)) != string::npos) {
	cur_dir_path = filename.substr(0, i->size());
	filename = filename.substr(i->size());
      }

    if (!filename.empty()) {
      while ((pos = filename.find("/")) != string::npos) {
	folders.top().second = find_position_in_folder_list(folders.top().first, cur_dir_path + filename.substr(0, pos));

	list<string> templs; templs.push_back(cur_dir_path + filename.substr(0, pos));
	folders.push(std::make_pair(templs, 0));
	cur_dir_path += filename.substr(0, pos+1);
	filename = filename.substr(pos+1);
      }
    }

    // we should now be in the correct folder, find the correct file
    folders.top().second = find_position_in_folder_list(folders.top().first, (*q)[0]["filename"]);

    load_current_dirs();
    
#ifdef use_inotify
    S_Notify::get_instance()->register_plugin("pictures", folders.top().first,
					      new TimeFunctorv<Pictures>(this, &Pictures::reparse_current_dir));
#endif

    mainloop();
  }

  delete q;
}

vector<std::pair<string, int> > Pictures::gen_search_list(const string& search_word)
{
  std::vector<std::pair<string, int> > result;

  SQLQuery *q = db.query("Folders", ("SELECT DISTINCT id, filename, name, is_folder FROM %t WHERE fuzzycmp('" + search_word + "', name, 1)").c_str());

  if (q) {

    for (int i = 0; i < q->numberOfTuples(); ++i) {

      SQLRow &row = (*q)[i];

      string out;

      bool result_ok = false;
      list<string> current_folders = folders.top().first;
      for (list<string>::iterator i = current_folders.begin(), iend = current_folders.end();
	   i != iend; ++i) {
	if (row["filename"].find(*i) != string::npos) {
	  result_ok = true;
	  break;
	}
      }

      if (result_ok) {
	out = row["name"];
	result.push_back(std::make_pair(string_format::convert(out), conv::atoi(row["id"])));
      }
    }
  }
  delete q;

  return result;
}

bool Pictures::file_sort::operator()(const Simplefile& lhs, const Simplefile& rhs)
{
  return sort_order(lhs.lowercase_name, rhs.lowercase_name, lhs.type == "dir", rhs.type == "dir");
}

void Pictures::action_no_find_recursion()
{
  if (vector_lookup(pic_list, folders.top().second).type != "dir") {
    pictures_fullscreen();
  } else {
    enter_dir();
  }
}

// commands
void Pictures::action()
{
  if (vector_lookup(pic_list, folders.top().second).type != "dir" && 
      conv::stob(opts.recurse_in_fullscreen())) {
    recurse_files = parse_dirs_recursion();
    if (find_recursion_pos())
      action_no_find_recursion();
  } else 
    action_no_find_recursion();
}

bool Pictures::enter_dir()
{
  if(vector_lookup(pic_list, folders.top().second).type != "dir") {
    input_master->set_map("pictures");
    pictures_fullscreen();
    input_master->set_map("default");
  } else {
    // go into the directory.
    string temp_cur_dir = vector_lookup(pic_list, folders.top().second).path;

    // get the total number of files:
    int nr_files = 0;
    for (file_iterator<file_t, default_order> i (temp_cur_dir); i != i.end (); i.advance(false))
      if (!isDirectory (i->getName()))
	++nr_files;

    ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

    // disable screensaver if it's running
    bool audio_fullscreen_running = true;

    if (!screen_updater->timer.status("audio_fullscreen"))
      audio_fullscreen_running = false;
    else
      screen_updater->timer.deactivate("audio_fullscreen");

    screen_updater->status_progressbar = 0;
    screen_updater->total_progressbar = nr_files;
    screen_updater->header = gettext("Generating thumbnails");

    TimeFunctoriNC cp(&check_progressbar);
    TimeFunctorvNC pg(&print_progressbar);
  
    screen_updater->timer.add(TimeElement("progressbar", &cp, &pg));

    if (conf->p_priority_change()) 
      nice(19); //Lower priority to keep audio from skipping

    // process picture dir 
    vector<Picture> templist = rdir(temp_cur_dir);

    if (conf->p_priority_change()) 
      nice(-19);

    screen_updater->timer.del("progressbar");
    cleanup_progressbar();  

    // reenable screensaver
    if (audio_fullscreen_running)
      screen_updater->timer.activate("audio_fullscreen");

    if (templist.size() != 0) {
      list<string> templs; templs.push_back(temp_cur_dir);
      folders.push(std::make_pair(templs, 0));
      pic_list = templist;

#ifdef use_inotify
      S_Notify::get_instance()->register_plugin("pictures", folders.top().first,
						new TimeFunctorv<Pictures>(this, &Pictures::reparse_current_dir));
#endif

      return true;
    } else {
      DialogWaitPrint pdialog(gettext("Folder is empty"), 1000);
      return false;
    }
  }
  return true;
}

void Pictures::go_back()
{
  if (folders.size() != 1) {
    folders.pop();

    load_current_dirs();

#ifdef use_inotify
    S_Notify::get_instance()->register_plugin("pictures", folders.top().first,
					      new TimeFunctorv<Pictures>(this, &Pictures::reparse_current_dir));
#endif

  } else
    exit();
}

void Pictures::load_current_dirs()
{
  pic_list = parse_dir(folders.top().first);

  if (folders.top().first.size() > 1 && folders.size() == 1)
    std::sort(pic_list.begin(), pic_list.end(), file_sort());
}

bool Pictures::find_recursion_pos()
{
  bool found_match = false;
  list<string> dirs = folders.top().first;
  pos_recursion = 0;

  // find the first image compared to the current position
  string cur_file = vector_lookup(pic_list, folders.top().second).path;
  for (vector<Picture>::iterator p = recurse_files.begin(), pend = recurse_files.end(); 
       p != pend; ++p, ++pos_recursion) {
    if (p->path.substr(0, cur_file.size()) == cur_file) {
      found_match = true;
      break;
    }
  }

  return found_match;
}

void Pictures::recurse_dirs()
{
  // create list, find the next image compared to where we stand currently
  recurse_files = parse_dirs_recursion();

  bool found_match = find_recursion_pos();

  if (found_match) {

    if (recurse_files.size() > 1) {
      if (pos_recursion == 0)
	pos_recursion = recurse_files.size() -1;
      else
	pos_recursion -= 1;
    }
    
    next_no_skip_folders();

    action_no_find_recursion();

  }
}

void Pictures::options()
{
  BusyIndicator::get_instance().idle();
	  
  if (opts.mainloop()) {
    load_current_dirs();

    // reload
    folders.top().second = 0;
    thumbnailed_all_files = false;
  }
}

void Pictures::search_func()
{
  BusyIndicator::get_instance().idle();

  ConstFunctor<Pictures, std::string, vector<std::pair<string, int> > > gen_list_func(this, &Pictures::gen_search_list);
  ConstFunctor<Pictures, vector<Picture>, void> print_func(this, &Pictures::print);

  int pos = input_master->device->graphical_search(pic_list, &print_func, &gen_list_func,
						   search_mode, search_str, lowercase_search_str, 
						   offset, search_help_offset);

  if (pos != -1)
    folders.top().second = pos;
}

void Pictures::exit()
{
  exit_loop = true;
}

void Pictures::find_recursion_file()
{
  string next_file = vector_lookup(recurse_files, pos_recursion).path;
  string next_path = next_file;
  string::size_type pos;
  if ((pos = next_path.rfind('/')) != string::npos)
    next_path = next_path.substr(0, pos);

  while (true) {

    // quick check

    // next
    if (folders.top().second + 1 <= pic_list.size() - 1) {
      if (vector_lookup(pic_list, folders.top().second + 1).path == next_file) {
	folders.top().second += 1;
	return;
      }
    } 

    // prev
    if (folders.top().second - 1 >= 0) {
      if (vector_lookup(pic_list, folders.top().second - 1).path == next_file) {
	folders.top().second -= 1;
	return;
      }
    }

    // check if on current level
    list<string> dirs = folders.top().first;

    int file_pos = 0;
    for (vector<Picture>::const_iterator iter = pic_list.begin(); 
	 iter != pic_list.end(); ++iter, ++file_pos) {
      if (!isDirectory(iter->path)) {
	if (iter->path == next_file) {
	  folders.top().second = file_pos;
	  return;
	}
      }
    }
    
    // try to find the correct subdir in current dir
    bool found_dir = false;
    file_pos = 0;
    for (vector<Picture>::const_iterator iter = pic_list.begin(); 
	 iter != pic_list.end(); ++iter, ++file_pos) {
      string cur_file = iter->path;
      if (cur_file.size() <= next_path.size())
	if (next_path.substr(0, cur_file.size()) == cur_file)
	  if (isDirectory(cur_file)) {
	    folders.top().second = file_pos;
	    enter_dir();
	    found_dir = true;
	    break;
	  }
    }
    
    if (!found_dir) {
      // it must be on a level below
      go_back();
    }
  }
}

// helper functions, to minimize code copying, functions used in mainloops.
void Pictures::prev()
{
  if (folders.top().second == 0)
    folders.top().second = pic_list.size()-1;
  else
    --(folders.top().second);
}

void Pictures::next()
{
  folders.top().second = (folders.top().second+1)%pic_list.size();
}

void Pictures::prev_no_skip_folders()
{
  if (pos_recursion != 0)
    --pos_recursion;
  else
    pos_recursion = recurse_files.size() - 1;
  
  find_recursion_file();
}

void Pictures::prev_skip_folders()
{
  do { 
    prev();
  } while (vector_lookup(pic_list, folders.top().second).type == "dir");
}

void Pictures::next_no_skip_folders()
{
  if (pos_recursion != recurse_files.size() - 1)
    ++pos_recursion;
  else
    pos_recursion = 0;
  
  find_recursion_file();
}

void Pictures::next_skip_folders()
{
  do { 
    next();
  } while (vector_lookup(pic_list, folders.top().second).type == "dir");
}

string Pictures::mainloop()
{
  Input input;

  bool update_needed = true;

  BusyIndicator &busy_indicator = BusyIndicator::get_instance();
  Shutdown *sd = S_Shutdown::get_instance();

  while (!exit_loop)
    {
      if (update_needed)
	print(pic_list);

      busy_indicator.idle();

      input = input_master->device->get_input();

      busy_indicator.busy();
      
      if (sd->is_enabled()) {
      	sd->cancel();
      	continue;
      }

      update_needed = true;
	  
      if (input.command == "prev")
 	{
	  if (pic_list.size() > images_per_row) {
	    for (int i = 0; i < images_per_row; ++i) {
	      if (folders.top().second == 0) {
		folders.top().second = pic_list.size()-1;
		i += images_per_row-pic_list.size()%images_per_row;
	      }
	      else 
		--(folders.top().second);
	    }
	  }
 	}
      else if (input.command == "next")
 	{
	  if (pic_list.size() > images_per_row) {
	    for (int i = 0; i < images_per_row; ++i) {
	      if (folders.top().second == pic_list.size()-1 && i == 0) {
		folders.top().second = 0;
		break;
	      } else if (folders.top().second == pic_list.size()-1) {
		break;
	      } else {
		++(folders.top().second);
	      }
	    }
	  }
 	}
      else if (input.command == "left")
 	{
	  prev();
 	}
      else if (input.command == "right")
 	{
	  next();
 	}
      else if (input.command == "page_up")
 	{
	  int jump = images_per_row*rows;

	  if (search_mode)
	    jump = images_per_row*rows_search;

	  if (pic_list.size() > jump) {
	    int diff = folders.top().second-jump;
	    if (folders.top().second == 0)
	      folders.top().second = (pic_list.size()-1)+diff;
	    else if (diff < 0)
	      folders.top().second = 0;
	    else
	      folders.top().second = diff;
	  }
 	}
      else if (input.command == "page_down")
 	{
	  int jump = images_per_row*rows;

	  if (search_mode)
	    jump = images_per_row*rows_search;

	  if (pic_list.size() > jump) {
	    if (folders.top().second > (pic_list.size() - jump) && folders.top().second != (pic_list.size()-1))
	      folders.top().second = pic_list.size()-1;
	    else
	      folders.top().second = (folders.top().second+jump)%pic_list.size();
	  }
 	}
      else if (input.command == "recurse_dirs" && conv::stob(opts.recurse_in_fullscreen()))
	{
	  recurse_dirs();
	}
      else if (input.command == "action")
	{
	  action();
	}
      else if (input.command == "back")
	{
	  go_back();
	}
      else if (input.command == "second_action")
	{
	  ExtraMenu em;

	  string t = gettext("Show picture in fullscreen");
	  
	  if (vector_lookup(pic_list, folders.top().second).type == "dir")
	    t = gettext("Enter directory");
	  
	  em.add_item(ExtraMenuItem(t, input_master->find_shortcut("action"), 
				    new TimeFunctorv<Pictures>(this, &Pictures::action)));

	  if (conv::stob(opts.recurse_in_fullscreen())) {
	    if (vector_lookup(pic_list, folders.top().second).type == "dir")
	      em.add_item(ExtraMenuItem(gettext("Recurse all folders on current level"), 
					input_master->find_shortcut("recurse_dirs"), 
					new TimeFunctorv<Pictures>(this, &Pictures::recurse_dirs)));
	  }
	  
	  em.add_item(ExtraMenuItem(gettext("Search"), input_master->find_shortcut("search"), 
				    new TimeFunctorv<Pictures>(this, &Pictures::search_func)));
	  em.add_item(ExtraMenuItem(gettext("Options"), input_master->find_shortcut("options"), 
				    new TimeFunctorv<Pictures>(this, &Pictures::options)));

	  if (folders.size() == 1)
	    em.add_item(ExtraMenuItem(gettext("Return to startmenu"), input_master->find_shortcut("back"), 
				      new TimeFunctorv<Pictures>(this, &Pictures::exit)));
	  else
	    em.add_item(ExtraMenuItem(gettext("Go up one directory"), input_master->find_shortcut("back"), 
				      new TimeFunctorv<Pictures>(this, &Pictures::go_back)));

	  for (vector<ExtraMenuItem>::const_iterator i = global->menu_items.begin(), 
		 end = global->menu_items.end(); i != end; ++i)
	    em.add_persistent_item(*i);

	  em.mainloop();
	}
      else if (input.command == "options")
	{
	  options();
	}
      else if (input.command == "search")
	{
	  search_func();
	}
      else
	update_needed = !global->check_commands(input);
    }

  exit_loop = false;

  return "";
}

vector<string> Pictures::images_in_dir(const list<string>& dirs)
{
  std::stack<string> queued_dirs;
  vector<string> image_files;

  for (list<string>::const_iterator iter = dirs.begin(); iter != dirs.end(); ++iter) {
    queued_dirs.push(*iter);
    while (!queued_dirs.empty()) {
      string cur_dir = queued_dirs.top();
      queued_dirs.pop();
      for (file_iterator<file_t, default_order> i (cur_dir); i != i.end (); i.advance(false)) {

	string cur_file = i->getName();

	// exclude folders ?
	if (pic_conf->p_excluded_picture_dirs().size() > 0) {
	  list<string> excluded = pic_conf->p_excluded_picture_dirs();
	  bool stop = false;
	      
	  for (list<string>::const_iterator epd = excluded.begin(), epd_end = excluded.end(); 
	       epd != epd_end; ++epd)
	    if (cur_file.substr(cur_file.rfind("/")+1) == *epd) {
	      stop = true;
	      break;
	    }
	      
	  if (stop)
	    continue;
	}

	if (isDirectory(cur_file))
	  queued_dirs.push(cur_file);
	else
	  image_files.push_back(cur_file);
      }
    }
  }

  return image_files;
}


void Pictures::background_thumbnails()
{
  if (!thumbnailed_all_files && (conf->p_idle_time() > 0) && 
      (opts.thumbnail_extract() == gettext("yes"))) {

    int nr_extracts = 3;

    // check if we are idle
    int idle_time = time(0)-conf->p_last_key();
    if (idle_time < 3)
      return;
    else if (idle_time > 60 * conf->p_idle_time())
      nr_extracts = 4;

    if (all_image_files.empty()) {
      all_image_files = images_in_dir(pic_conf->p_picture_dirs());
      // optimization, we don't want to both make the list and start extracting
      return;
    }

    if (conf->p_priority_change()) 
      nice(19); //Lower priority to keep audio from skipping

    while (!all_image_files.empty()) {

      bool extracted = false;

      if (!file_exists(render->image_get_fullpath(all_image_files.back(), image_width, image_height))) {
	extracted = true;
	render->image_mut.enterMutex();
	render->device->wait_for_not_drawing();
	render->create_scaled_image_wrapper(all_image_files.back(), image_width, image_height);
	render->image_mut.leaveMutex();
      } 

      if (!file_exists(render->image_get_fullpath(all_image_files.back(), conf->p_v_res(), conf->p_h_res()))) {
	extracted = true;
	render->image_mut.enterMutex();
	render->device->wait_for_not_drawing();
	render->create_scaled_image_wrapper(all_image_files.back(), conf->p_v_res(), conf->p_h_res());
	render->image_mut.leaveMutex();
      }
      
      if (nr_extracts == 0) {
	if (conf->p_priority_change()) 
	  nice(-19);
	return;
      } else if (extracted)
	--nr_extracts;

      all_image_files.erase(all_image_files.end());
    }

    if (conf->p_priority_change()) 
      nice(-19);

    thumbnailed_all_files = true;
  } 
}

int Pictures::check_thumbnail()
{
  return 3000; // ms
}

string Pictures::get_name_from_file(const Simplefile& s)
{
  return s.name;
}

bool Pictures::search_compare(const Simplefile& s)
{
  return (s.lowercase_name.substr(0, search_str.size()) == lowercase_search_str);
}

void Pictures::prepare_and_show_fullscreen()
{
  string left_path, right_path;

  if (conv::stob(opts.recurse_in_fullscreen())) {

    int left_pos_recursion;
    if (pos_recursion != 0)
      left_pos_recursion = pos_recursion - 1;
    else
      left_pos_recursion = recurse_files.size() - 1;

    left_path = vector_lookup(recurse_files, left_pos_recursion).path;
    right_path = vector_lookup(recurse_files, (pos_recursion + 1) % recurse_files.size()).path;

  } else {

    int left = folders.top().second;
    do {
      left -= 1;
      if (left == -1)
	left=pic_list.size()-1;
    } while (vector_lookup(pic_list, left).type == "dir");

    int right = folders.top().second;
    do {
      right += 1;
      if (right == pic_list.size())
	right=0;
    } while (vector_lookup(pic_list, right).type == "dir");

    left_path = pic_list[left].path;
    right_path = pic_list[right].path;
  }
  
  if (conf->p_priority_change()) 
    nice(19); //Lower priority to keep audio from skipping
  
  print_fullscreen(pic_list[folders.top().second], left_path, right_path);

  if (conf->p_priority_change()) 
    nice(-19);
}

void Pictures::zoom() 
{
  if (zoom_level == 1) {

    // calculate zoom position
    render->image_mut.enterMutex();
    PFSObj p = PFSObj(vector_lookup(pic_list, folders.top().second).path, 
		      false, orientation, zoom_level, zoom_position, 1, 1, 1);
    render->image_mut.leaveMutex();

    // don't zoom small images
    if (p.real_w < conf->p_v_res() && p.real_h < conf->p_h_res())
      return;

    zoom_level = 2;

    int pic_res_w = p.real_w*zoom_level;
    max_pos_v = 3; // defeault
    if (pic_res_w <= conf->p_v_res())
      max_pos_v = 1;
    else if (pic_res_w <= int(1.5*conf->p_v_res()))
      max_pos_v = 2;

    int pic_res_h = p.real_h*zoom_level;
    max_pos_h = 3; // defeault
    if (pic_res_h <= conf->p_h_res())
      max_pos_h = 1;
    else if (pic_res_h <= int(1.5*conf->p_h_res()))
      max_pos_h = 2;

    zoom_pos_h = int(max_pos_h/2)+1;
    zoom_pos_v = int(max_pos_v/2)+1;

    zoom_position = (zoom_pos_h-1)*max_pos_v+zoom_pos_v-1;

  } else {
    zoom_level = 1;
    zoom_position = 1;
  }
}

void Pictures::suspend()
{
  pause = !pause;
}

void Pictures::rotate_left()
{
  orientation = (orientation+1)%4;
}

void Pictures::rotate_right()
{
  orientation = (orientation == 0) ? 3 : orientation - 1;
}

void Pictures::options_fullscreen()
{
  // FIXME:
  //#ifdef c_music
  // Audio *audio = Audio::get_instance();
  //audio->set_audio_playback_pos(0);
  //#endif

  bool recursion_before = conv::stob(opts.recurse_in_fullscreen());

  options();

  if (!recursion_before && conv::stob(opts.recurse_in_fullscreen())) {
    recurse_files = parse_dirs_recursion();
    find_recursion_pos();
  }

  //#ifdef c_music
  //audio->set_audio_playback_pos(image_width + 10);
  //#endif
}

void Pictures::deactivate_updaters()
{
  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();
  screen_updater->timer.deactivate("pictures");
  screen_updater->timer.deactivate("audio_fullscreen");
}

void Pictures::activate_updaters()
{
  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();
  if (opts.slideshow() != "0")
    screen_updater->timer.activate("pictures");
  else
    screen_updater->timer.activate("audio_fullscreen");
}

void Pictures::pictures_fullscreen()
{
  Input input;

  orientation = get_db_orientation(vector_lookup(pic_list, folders.top().second).db_id);

  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

  TimeFunctori<Pictures> time_func(this, &Pictures::check_picture_time);
  TimeFunctorv<Pictures> call_func(this, &Pictures::pictures_check);
  screen_updater->timer.add(TimeElement("pictures", &time_func, &call_func));

  if (opts.slideshow() != "0")
    screen_updater->timer.deactivate("audio_fullscreen");
  else
    screen_updater->timer.deactivate("pictures");

  // FIXME
  //#ifdef c_music
  //audio->set_audio_playback_pos(image_width + 10);
  //#endif

  bool update_needed = true;

  BusyIndicator &busy_indicator = BusyIndicator::get_instance();

  while (!exit_loop)
    {
      global->set_playback_offset(image_width + 10);

      prepare_and_show_fullscreen();

      busy_indicator.idle();

      input = input_master->device->get_input();
      
      last_press = time(0);

      busy_indicator.busy();

      update_needed = true;

      global->set_playback_offset(0);

      if (zoom_level != 1) {

	if (input.command == "prev") {
	  if (zoom_pos_h != 1)
	    --zoom_pos_h;
	} else if (input.command == "next") {
	  if (zoom_pos_h < max_pos_h)
	    ++zoom_pos_h;
	} else if (input.command == "left") {
	  if (zoom_pos_v != 1)
	    --zoom_pos_v;
	} else if (input.command == "right") {
	  if (zoom_pos_v < max_pos_v)
	    ++zoom_pos_v;
	} else if (input.command == "action" || (input.command == "back" && input.mode == "general")) {
	  zoom_level = 1;
	  zoom_position = 1;
	  activate_updaters();
	  continue;
	}

	// calculate zoom_position
	zoom_position = (zoom_pos_h-1)*max_pos_v + zoom_pos_v - 1;

	continue;
      }

      if (input.command == "back" && input.mode == "general")
	{
	  exit();
	}
      else if (input.command == "left")
	{
	  if (conv::stob(opts.recurse_in_fullscreen()))
	    prev_no_skip_folders();
	  else
	    prev_skip_folders();

	  orientation = get_db_orientation(vector_lookup(pic_list, folders.top().second).db_id);
	}
      else if (input.command == "right")
	{
	  if (conv::stob(opts.recurse_in_fullscreen()))
	    next_no_skip_folders();
	  else
	    next_skip_folders();

	  orientation = get_db_orientation(vector_lookup(pic_list, folders.top().second).db_id);
	}
      else if (input.command == "page_up")
 	{
	  int jump = images_per_row*rows;

	  if (search_mode)
	    jump = images_per_row*rows_search;

	  if (pic_list.size() > jump) {
	    int diff = folders.top().second-jump;
	    if (folders.top().second == 0)
	      folders.top().second = (pic_list.size()-1)+diff;
	    else if (diff < 0)
	      folders.top().second = 0;
	    else
	      folders.top().second = diff;

	    while (vector_lookup(pic_list, folders.top().second).type == "dir")
	      prev();

	    orientation = get_db_orientation(vector_lookup(pic_list, folders.top().second).db_id);
	  }
 	}
      else if (input.command == "page_down")
 	{
	  int jump = images_per_row*rows;

	  if (search_mode)
	    jump = images_per_row*rows_search;

	  if (pic_list.size() > jump) {
	    if (folders.top().second > (pic_list.size() - jump) && folders.top().second != (pic_list.size()-1))
	      folders.top().second = pic_list.size()-1;
	    else
	      folders.top().second = (folders.top().second+jump)%pic_list.size();

	    while (vector_lookup(pic_list, folders.top().second).type == "dir")
	      next();

	    orientation = get_db_orientation(vector_lookup(pic_list, folders.top().second).db_id);
	  }
 	}
      else if (input.command == "next") 
	{
	  rotate_left();
	  set_db_orientation(vector_lookup(pic_list, folders.top().second).db_id);
	}
      else if (input.command == "prev") 
	{
	  rotate_right();
	  set_db_orientation(vector_lookup(pic_list, folders.top().second).db_id);
	}
      else if (input.command == "action")
	{
	  zoom();

	  if (zoom_level != 1)
	    deactivate_updaters();
	  else
	    activate_updaters();
	}
      else if (input.command == "suspend")
	{
	  deactivate_updaters();
	  suspend();
	  activate_updaters();
	}
      else if (input.command == "second_action")
	{
	  ExtraMenu em;

	  em.add_item(ExtraMenuItem(gettext("Zoom"), input_master->find_shortcut("action"), 
				    new TimeFunctorv<Pictures>(this, &Pictures::zoom)));

	  string slideshow_status = gettext("Suspend slideshow");

	  if (pause)
	    slideshow_status = gettext("Resume slideshow");

	  em.add_item(ExtraMenuItem(slideshow_status, input_master->find_shortcut("suspend"), 
				    new TimeFunctorv<Pictures>(this, &Pictures::suspend)));

	  em.add_item(ExtraMenuItem(gettext("Rotate left"), input_master->find_shortcut("left"), 
				    new TimeFunctorv<Pictures>(this, &Pictures::rotate_left)));

	  em.add_item(ExtraMenuItem(gettext("Rotate right"), input_master->find_shortcut("right"), 
				    new TimeFunctorv<Pictures>(this, &Pictures::rotate_right)));

	  em.add_item(ExtraMenuItem(gettext("Options"), input_master->find_shortcut("options"), 
				    new TimeFunctorv<Pictures>(this, &Pictures::options_fullscreen)));

	  em.add_item(ExtraMenuItem(gettext("Leave fullscreen"), 
				    input_master->find_shortcut("back"), 
				    new TimeFunctorv<Pictures>(this, &Pictures::exit)));

	  for (vector<ExtraMenuItem>::const_iterator i = global->menu_items.begin(), 
		 end = global->menu_items.end(); i != end; ++i)
	    em.add_persistent_item(*i);

	  deactivate_updaters();
	    
	  em.mainloop();

	  if (zoom_level == 1)
	    activate_updaters();
	}
      else if (input.command == "options")
	{
	  deactivate_updaters();
	  options_fullscreen();
	  activate_updaters();
	}
      else
	update_needed = !global->check_commands(input);
    }

  if (zoom_level != 1) {
    zoom_level = 1;
    zoom_position = 1;
  }

  exit_loop = false;

  global->set_playback_offset(0);

  screen_updater->timer.del("pictures");
  screen_updater->timer.activate("audio_fullscreen");
}

void Pictures::print(const vector<Picture>& cur_list)
{
  render->prepare_new_image();

  render->current.add(new PObj(themes->pictures_background, 0, 0, 0, SCALE_FULL));

  render->current.add(new PObj(themes->startmenu_pictures_dir, 25, 10, 2, NOSCALING));

  if (themes->show_header) {
    render->current.add(new TObj(gettext("Pictures"), "Vera/28", 100, 10, 
				 themes->pictures_header_font1, themes->pictures_header_font2, 
				 themes->pictures_header_font3, 2));
  }
  
  if (search_mode) {

    int y = 60;
    if (conf->p_v_res() == 720) {
      y -= 3;
    }

    render->current.add(new RObj(0, y, conf->p_v_res(), 45, 
				 themes->general_search_rectangle_color1, 
				 themes->general_search_rectangle_color2, 
				 themes->general_search_rectangle_color3, 
				 themes->general_search_rectangle_color4, 2));

    render->current.add(new PObj(themes->general_search, 47, y, 3, NOSCALING));

    render->current.add(new TObj(gettext("Search: ") + search_str, "Vera/22", 100, y+5, 
				 themes->pictures_font1, themes->pictures_font2, 
				 themes->pictures_font3, 3));
  }

  // print the range
  int pos, start, pos_in_list, cur_rows, cur_image_height_all;

  if (search_mode) {
    pos_in_list = offset;
    cur_rows = rows_search;
    cur_image_height_all = image_height_all_search;
  } else {
    pos_in_list = folders.top().second;
    cur_rows = rows;
    cur_image_height_all = image_height_all;
  }
  
  int nr_in_last_row = cur_list.size()%images_per_row;
  if (nr_in_last_row == 0)
    nr_in_last_row = images_per_row;

  if (cur_list.size() > cur_rows*images_per_row) {
    if (pos_in_list < std::ceil(double(cur_rows)/2.0)*images_per_row) { // first two rows
      pos = pos_in_list;
      start = 0;
    } else if (pos_in_list >= cur_list.size() - (images_per_row+nr_in_last_row)) { // last two rows
      start = cur_list.size()-((cur_rows-1)*images_per_row+nr_in_last_row);
      pos = pos_in_list-start;
    } else {
      pos = pos_in_list%images_per_row+images_per_row;
      if (pos == images_per_row-1)
	pos = 2*images_per_row-1;
      start = pos_in_list-pos;
    }
  } else {
    pos = pos_in_list;
    start = 0;
  }

  int x_start, x = (conf->p_v_res()-images_per_row*image_width_all)/2, y = y_start;
  x_start = x;

  if (search_mode) {
    y = y_start;
    if (conf->p_input() != "keyboard")
      y += 60;
  }

  for (int j = 0; j < cur_rows*images_per_row && j < cur_list.size(); ++j) {

    Simplefile s = vector_lookup(cur_list, start);

    if (s.type == "dir") {

      PObj *p;
      
      if (pos == j)
	p = new PObj(themes->pictures_dir_marked, x, y, 3, NOSCALING);
      else
	p = new PObj(themes->pictures_dir, x, y, 3, NOSCALING);

      p->x = p->real_x = x + (image_width_all - p->w)/2;
      p->y = p->real_y = y + (image_height_all_eks_text - p->h)/2;
      
      render->current.add(p);

    } else {
      bool border = false;
	
      if (pos == j)
	border = true;
	
      PFObj *p = new PFObj(s.path, x, y, image_width, image_height, border, 3);
	
      p->x = p->real_x = x + (image_width_all - (p->w + 10))/2;
      p->y = p->real_y = y + (image_height_all_eks_text - (p->h + 10))/2;

      if (p->border) {
	p->real_x -= 5;
	p->real_y -= 5;
      } else {
	p->real_x -= 1;
	p->real_y -= 1;
      }

      render->current.add(p);
    }

    string name = s.name;
    string old_name = name;
    
    string_format::format_to_size(name, "Vera/13", image_width_all-10, false);

    string name_second_part = old_name.substr(name.size());

    if (name_second_part.size() > 0 && name_second_part[0] != ' ') {
      // splitting words
      string::size_type last_whole_word = name.rfind(' ');
      if (last_whole_word != string::npos) {
	name = name.substr(0, last_whole_word);
	name_second_part = old_name.substr(name.size());
      }
    }

    int x_size = string_format::calculate_string_width(name, "Vera/13");

    string_format::format_to_size(name_second_part, "Vera/13", image_width_all-10, false);

    int x_size2 = string_format::calculate_string_width(name_second_part, "Vera/13");

    if (pos == j) {
      render->current.add(new TObj(name, "Vera/13", x + (image_width_all-x_size)/2, y+image_height_all_eks_text, 
				   themes->pictures_marked_font1, themes->pictures_marked_font2, 
				   themes->pictures_marked_font3, 3));
      render->current.add(new TObj(name_second_part, "Vera/13", x + (image_width_all-x_size2)/2, y+image_height_all_eks_text+20, 
				   themes->pictures_marked_font1, themes->pictures_marked_font2, 
				   themes->pictures_marked_font3, 3));
    } else {
      render->current.add(new TObj(name, "Vera/13", x + (image_width_all-x_size)/2, y+image_height_all_eks_text, 
				   themes->pictures_font1, themes->pictures_font2, 
				   themes->pictures_font3, 3));
      render->current.add(new TObj(name_second_part, "Vera/13", x + (image_width_all-x_size2)/2, y+image_height_all_eks_text+20, 
				   themes->pictures_font1, themes->pictures_font2, 
				   themes->pictures_font3, 3));
    }

    x += image_width_all;

    if (start == cur_list.size()-1) {
      if ((j+1)%images_per_row != 0)
	j += (images_per_row-(j+1)%images_per_row);
      
      x = x_start;
      y += cur_image_height_all;

      start = 0;
      continue;
      
    } else
      ++start;
    
    if ((j+1)%images_per_row == 0) {
      y += cur_image_height_all;
      x = x_start;
    }
  }

  std::ostringstream position;

  if (search_mode)
    if (cur_list.size() > 0)
      position << offset + 1 << "/" << cur_list.size();
    else
      position << "";
  else
    position << folders.top().second+1 << "/" << cur_list.size();

  int x_size = string_format::calculate_string_width(position.str(), "Vera/18");

  render->current.add(new TObj(position.str(), "Vera/18", conf->p_v_res()-(60+x_size), 20, 
			       themes->pictures_font1, themes->pictures_font2, 
			       themes->pictures_font3, 3));

  render->draw_and_release("pictures");
}

void Pictures::print_fullscreen(const Simplefile& e, const string& left_path, const string& right_path)
{
  // sanity
  if (e.type == "dir")
    return;

  if (conf->p_priority_change()) 
    nice(19);

  render->prepare_new_image();

  render->current.add(new RObj(0, 0, conf->p_v_res(), conf->p_h_res(), 0, 0, 0, 255, 0));

  // FIXME: create thumbnail for at least the db stored rotation

  if (zoom_level != 1 || orientation != 0) {
    render->current.add(new PFSObj(e.path, conv::stob(opts.zoom()), orientation, 
				   zoom_level, zoom_position, max_pos_v, max_pos_h, 1));
  } else {
    // use thumbnailed for performance
    render->create_scaled_image_wrapper(e.path, conf->p_v_res(), conf->p_h_res());
    render->current.add(new PFSObj(render->image_get_fullpath(e.path, conf->p_v_res(), conf->p_h_res()), 
				   conv::stob(opts.zoom()), orientation, zoom_level, zoom_position, 
				   max_pos_v, max_pos_h, 1));
  }

  if (conv::stob(opts.show_preview()) && zoom_level == 1) {
    bool is_transparent;
    // left
    std::pair<int, int> dimensions = render->image_dimensions(render->image_get_fullpath(left_path, image_width, image_height), &is_transparent);
    render->current.add(new PFObj(left_path, 5, conf->p_h_res()-dimensions.first-5, image_width, image_height, true, 2));
    // right
    dimensions = render->image_dimensions(render->image_get_fullpath(right_path, image_width, image_height), &is_transparent);
    render->current.add(new PFObj(right_path, conf->p_v_res()-dimensions.second-5, conf->p_h_res()-dimensions.first-5, image_width, image_height, true, 2));
  }
  
  std::ostringstream position;
  if (conv::stob(opts.recurse_in_fullscreen()))
    position << pos_recursion+1 << "/" << recurse_files.size();
  else
    position << folders.top().second+1 << "/" << pic_list.size();

  int x_size = string_format::calculate_string_width(position.str(), "Vera/18");
  
  int x_pos = 20, y_pos = 20;

  if (conf->p_busy_indicator()) {
    x_pos = 35;
    y_pos = 10;
  }

  render->current.add(new TObj(position.str(), "Vera/18", conf->p_v_res()-(x_pos+x_size), y_pos, 
			       themes->pictures_font1, themes->pictures_font2, 
			       themes->pictures_font3, 2));

  string name = e.name;

  if (pause && conv::stob(opts.slideshow()))
    name += gettext(" ( paused )");

  string_format::format_to_size(name, "Vera/16", conf->p_v_res()-(20+x_size+30), false);
  
  render->current.add(new TObj(name, "Vera/16", 20, 20, 
			       themes->pictures_font1, themes->pictures_font2, 
			       themes->pictures_font3, 2));

  render->draw_and_release("pictures full");

  if (conf->p_priority_change()) 
    nice(-19);
}

void Pictures::play_pictures_cd()
{
  if (cd->data_disc()) {
    // backup ;)
    vector<Picture> old_list = pic_list;
    std::stack<std::pair<list<string>, int > > temp_folders = folders;
    while (!folders.empty())
      folders.pop();
    list<string> templs; templs.push_back(cd->get_mount_point());
    folders.push(std::make_pair(templs, 0));
  
    //mount the disc
    run::external_program("mount " + cd->get_mount_point());

    bool valid_files = testdir(cd->get_mount_point());
  
    // get the total number of files:
    int nr_files = 0;
    for (file_iterator<file_t, default_order> i (cd->get_mount_point()); i != i.end (); i.advance(false))
      if (!isDirectory (i->getName()))
	++nr_files;

    ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

    // disable screensaver if it's running
    bool audio_fullscreen_running = true;

    if (!screen_updater->timer.status("audio_fullscreen"))
      audio_fullscreen_running = false;
    else
      screen_updater->timer.deactivate("audio_fullscreen");

    screen_updater->status_progressbar = 0;
    screen_updater->total_progressbar = nr_files;
    screen_updater->header = gettext("Generating thumbnails");

    TimeFunctoriNC cp(&check_progressbar);
    TimeFunctorvNC pg(&print_progressbar);
  
    screen_updater->timer.add(TimeElement("progressbar", &cp, &pg));

    pic_list = rdir(cd->get_mount_point());

    screen_updater->timer.del("progressbar");
    cleanup_progressbar();  

    if (audio_fullscreen_running)
      // reenable screensaver
      screen_updater->timer.activate("audio_fullscreen");

    if (pic_list.size() == 0 || !valid_files)
      Print pdialog(gettext("The disc does not contain any recognizable files"), 
		    Print::SCREEN);
    else
      mainloop();
    
    //umount the disc
    run::external_program("umount " + cd->get_mount_point());

    pic_list = old_list;
    folders = temp_folders;
  } else
    Print pdialog(gettext("The disc does not contain any recognizable files"), 
		  Print::SCREEN);
}

// ====================================================================
//
// dir
//
// ====================================================================

bool Pictures::testdir(const string& dir)
{
  for (file_iterator<file_t, default_order> i (dir); i != i.end (); ++i) {
    if (global->check_stop_bit())
      break;

    if (check_type((*i).getName(), conf->p_filetypes_p()) != emptyMyPair)
      return true;
  }

  return false;
}

void Pictures::insert_picture_into_db(const string& filename, const string& name)
{
  SQLQuery *q = db.query("Pictures", ("SELECT * FROM %t WHERE filename='" + string_format::escape_string(filename) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    delete q;
    return; // file already exists, don't insert
  }
  delete q;
  
  struct stat s;
  stat(filename.c_str(), &s);

  // FIXME: group id (value 6)
  char *tmp = sqlite3_mprintf("INSERT INTO Pictures VALUES(NULL, '%q', '%q', '%q', '%i', 0, 0, 0)",
			      filename.c_str(), name.c_str(), "", s.st_mtime);
  db.execute(tmp);
  sqlite3_free(tmp);
}

// Add an picture to the list 
Picture Pictures::addfile(const string& name, const MyPair& filetype)
{
  render->image_mut.enterMutex();
  render->device->wait_for_not_drawing();
  render->create_scaled_image_wrapper(name, image_width, image_height);
  render->image_mut.leaveMutex();

  Picture p = addsimplefile(name, filetype);

  insert_picture_into_db(name, p.name);

  return p;
}

void Pictures::insert_file_into_db(const string& filename, const string& parent)
{
  string parent_id = "";
  
  string good_parent = string_format::unique_folder_name(parent);
  
  SQLQuery *q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(good_parent) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    assert(q->numberOfTuples() == 1);
    SQLRow &row = (*q)[0];
    parent_id = row["id"];
  }
  delete q;

  if (parent_id.empty()) {
    char *tmp = sqlite3_mprintf("INSERT INTO Folders VALUES(NULL, '0', '%q', '', '1')", good_parent.c_str());
    db.execute(tmp);
    sqlite3_free(tmp);

    q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(good_parent) + "'").c_str());
    if (q && q->numberOfTuples() > 0) {
      assert(q->numberOfTuples() == 1);
      SQLRow &row = (*q)[0];
      parent_id = row["id"];
    }
    delete q;
  }

  bool is_dir = isDirectory(filename);

  string good_filename;
  string cut_filename;
  
  if (is_dir) {
    good_filename = string_format::unique_folder_name(filename);
    cut_filename = good_filename.substr(0, good_filename.size()-1);
  } else {
    good_filename = filename;
    cut_filename = filename;
  }

  string::size_type i;
  if ((i = cut_filename.rfind('/')) == string::npos)
    i = 0;
  string name = cut_filename.substr((i != 0) ? i+1 : i);
  
  if (!is_dir && (i = name.rfind('.')) != string::npos)
    name = name.substr(0, i);

  char *tmp_insert = sqlite3_mprintf("INSERT INTO Folders VALUES(NULL, '%q', '%q', '%q', '%q')", 
				     parent_id.c_str(), good_filename.c_str(), name.c_str(),
				     conv::itos(is_dir).c_str());
  db.execute(tmp_insert);
  sqlite3_free(tmp_insert);
}

int Pictures::db_id(const string& file, bool is_dir)
{
  int i = -1;

  string good_file;

  if (is_dir)
    good_file = string_format::unique_folder_name(file);
  else
    good_file = file;

  SQLQuery *q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(good_file) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    SQLRow &row = (*q)[0];
    i = conv::atoi(row["id"]);
  }
  delete q;

  return i;
}

bool Pictures::rdir_internal(const string& original_name, const string& argv, 
			     vector<Picture> &cur_files)
{
  if (global->check_stop_bit()) {
    cur_files.clear();
    return false;
  }
      
  string cur_file = original_name;
  if (cur_file[cur_file.size()-1] == '/')
    cur_file = cur_file.substr(0, cur_file.size()-1);

  string name = cur_file.substr(argv.size()+1);
  if (isDirectory (cur_file)) {
    // exclude folders ?
    if (pic_conf->p_excluded_picture_dirs().size() > 0) {
      list<string> excluded = pic_conf->p_excluded_picture_dirs();
      bool stop = false;
         
      for (list<string>::const_iterator epd = excluded.begin(), epd_end = excluded.end(); 
	   epd != epd_end; ++epd)
	if (cur_file.substr(cur_file.rfind("/")+1) == *epd) {
	  stop = true;
	  break;
	}

      if (stop)
	return true;
    }

    // dir
    Picture r;
    r.id = ++id;
    r.name = name;
    if (conf->p_convert())
      r.name = string_format::convert(r.name);
    r.lowercase_name = r.name;
    r.path = cur_file;
    r.type="dir"; 
    r.db_id = db_id(r.path, true);

    cur_files.push_back(r);

  } else {
    // file
    const MyPair filetype = check_type(cur_file, conf->p_filetypes_p());
    if (filetype != emptyMyPair) {
      Picture p = addfile(cur_file, filetype);
      p.db_id = db_id(p.path, false);
      cur_files.push_back(p);
    }
    
    ++S_ScreenUpdater::get_instance()->status_progressbar;
  }

  return true;
}

vector<string> Pictures::check_db_for_folders(const string& parent)
{
  vector<string> names;

  string good_parent = string_format::unique_folder_name(parent);

  SQLQuery *q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_string(good_parent) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    assert(q->numberOfTuples() == 1);
    SQLRow &row = (*q)[0];
    string parent_id = row["id"];
    SQLQuery *q2 = db.query("Folders", ("SELECT filename FROM %t WHERE parent='" + parent_id + "'").c_str());
    if (q2) {
      for (int i = 0; i < q2->numberOfTuples(); ++i) {
	SQLRow &row = (*q2)[i];
	names.push_back(row["filename"]);
      }
    }
    delete q2;
  }
  delete q;

  return names;
}

// read a dir and spit out a list of files and dirs
vector<Picture> Pictures::rdir(const string& argv)
{
  vector<Picture> files;

  vector<string> db_files = check_db_for_folders(argv);
  
  if (db_files.size() > 0) {
    for (vector<string>::const_iterator i = db_files.begin(), iend = db_files.end();
	 i != iend; ++i)
      if (!rdir_internal(*i, argv, files))
	break;
  } else {
    for (file_iterator<file_t, default_order> i (argv); i != i.end (); i.advance(false)) {
      string filename = filesystem::FExpand(i->getName());
      insert_file_into_db(filename, argv);
      if (!rdir_internal(filename, argv, files))
	break;
    }
  }

  return files;
}

// The ber function for parsing dirs.
vector<Picture> Pictures::parse_dir(const list<string>& dirs)
{
  vector<Picture> files;

  for (list<string>::const_iterator iter = dirs.begin(); iter != dirs.end(); ++iter)
    {
      // Add files
      vector<Picture> tempfiles = rdir(*iter);
      files.insert(files.end(), tempfiles.begin(), tempfiles.end());
    }

  return files;
}

// functions for creating lists for recursion

vector<Picture> Pictures::parse_dirs_recursion()
{
  vector<string> image_files = images_in_dir(folders.top().first);

  // we need them in the correct order for recursion
  sort(image_files.begin(), image_files.end());

  vector<Picture> files;

  for (vector<string>::const_iterator i = image_files.begin(), iend = image_files.end();
       i != iend; ++i)
    {
      const MyPair filetype = check_type(*i, conf->p_filetypes_p());
      if (filetype != emptyMyPair)
	files.push_back(addfile(*i, filetype));
    }

  return files;
}

void Pictures::read_dirs() 
{
  // get the total number of files:
  int nr_files = 0;
  list<string> dirs = pic_conf->p_picture_dirs();
  for (list<string>::const_iterator iter = dirs.begin(); iter != dirs.end(); ++iter)
    for (file_iterator<file_t, default_order> i (*iter); i != i.end (); i.advance(false))
      if (!isDirectory (i->getName()))
	++nr_files;

  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

  // disable screensaver if it's running
  bool audio_fullscreen_running = true;

  if (!screen_updater->timer.status("audio_fullscreen"))
    audio_fullscreen_running = false;
  else
    screen_updater->timer.deactivate("audio_fullscreen");

  screen_updater->status_progressbar = 0;
  screen_updater->total_progressbar = nr_files;
  screen_updater->header = gettext("Generating thumbnails");

  TimeFunctoriNC cp(&check_progressbar);
  TimeFunctorvNC pg(&print_progressbar);
  
  screen_updater->timer.add(TimeElement("progressbar", &cp, &pg));

  // process picture dir 
  for (list<string>::iterator iter = dirs.begin(); iter != dirs.end(); ++iter)
    if ((*iter)[iter->size()-1] == '/')
      *iter = iter->substr(0, iter->size()-1);

  pic_list = parse_dir(dirs);

  screen_updater->timer.del("progressbar");
  cleanup_progressbar();  

  if (audio_fullscreen_running)
    // reenable screensaver
    screen_updater->timer.activate("audio_fullscreen");

  if (pic_conf->p_picture_dirs().size() > 1)
    std::sort(pic_list.begin(), pic_list.end(), file_sort());
  
  if (pic_list.size() != 0)
    folders.top().second = 0;

#ifdef use_inotify
  S_Notify::get_instance()->register_plugin("pictures", pic_conf->p_picture_dirs(),
					    new TimeFunctorv<Pictures>(this, &Pictures::reparse_current_dir));
#endif
}

// notify
void Pictures::reparse_current_dir()
{
  // process audio dir 
  load_current_dirs();
  
  // sanity check
  if (folders.top().second > pic_list.size()-1)
    folders.top().second = 0;
}

void Pictures::pictures_check()
{
  if (opts.slideshow() != "0" && !pause && last_press <= time(0) - conv::atoi(opts.slideshow())) {

    if (conv::stob(opts.recurse_in_fullscreen()))
      next_no_skip_folders();
    else
      next_skip_folders();
    
    prepare_and_show_fullscreen();
  }
}

int Pictures::check_picture_time()
{
  // here we can assume that opts.slideshow() > 0 from the calling code
  return std::min(static_cast<long int>(conv::atoi(opts.slideshow())), time(0)-last_press)*1000;
}
