#ifndef PICTURES_HPP
#define PICTURES_HPP

#include <stack>

#include "cpp_sqlitepp.hpp"

#include "module.hpp"
#include "simplefile.hpp"
#include "options.hpp"
#include "singleton.hpp"
#include "picture_config.hpp"

class Picture : public Simplefile
{
public:
  Picture(Simplefile s)
    : Simplefile(s)
  {}

  Picture()
  {}

  int db_id;
};

class Pictures : public Module 
{
private:

  SQLDatabase db;

  void create_db();
  void check_db();

  int get_db_orientation(int id);
  void set_db_orientation(int id);

  void reenter(int id);
  std::vector<std::pair<std::string, int> > gen_search_list(const std::string& search_word);

  void action_no_find_recursion();

  // commands
  void action();
  void go_back();
  void recurse_dirs();
  void options();
  void search_func();
  void exit();

  // fullscreen commands
  void zoom();
  void suspend();
  void rotate_left();
  void rotate_right();
  void options_fullscreen();

  void deactivate_updaters();
  void activate_updaters();

  bool enter_dir();

  bool exit_loop;

  bool rdir_internal(const std::string& filename, const std::string& argv, 
		     std::vector<Picture> &cur_files);
  void insert_file_into_db(const std::string& filename, const std::string& parent);
  int db_id(const std::string& file, bool is_dir);
  std::vector<std::string> check_db_for_folders(const std::string& parent);

  void insert_picture_into_db(const std::string& filename, const std::string& name);
  int find_position_in_folder_list(const std::list<std::string>& files, const std::string& filename);

  bool search_compare(const Simplefile& s);
  std::string get_name_from_file(const Simplefile& e);

  void print(const std::vector<Picture>& pic_list);
  void prepare_and_show_fullscreen();
  void pictures_fullscreen();

  // helper functions, to minimize code copying, functions used in mainloops.
  void prev();
  void next();

  std::vector<std::string> images_in_dir(const std::list<std::string>& dirs);

  std::vector<Picture> parse_dirs_recursion();
  void find_recursion_file();
  bool find_recursion_pos();

  int pos_recursion;
  std::vector<Picture> recurse_files;

  void prev_no_skip_folders();
  void next_no_skip_folders();

  void prev_skip_folders();
  void next_skip_folders();

  bool testdir(const std::string& dir);

  Picture addfile(const std::string& name, const MyPair& filetype);
  std::vector<Picture> rdir(const std::string& argv);
  std::vector<Picture> parse_dir(const std::list<std::string>& dirs);

  int image_width;
  int image_height;
  int image_height_all_eks_text;

  // including text and space around image
  int image_width_all;
  int image_height_all;
  int image_height_all_search;

  int images_per_row;
  int rows;
  int rows_search;

  int y_start;

  int orientation;

  int zoom_level; // 1 or 2 currently
  int zoom_position; // 1 to X, where 1..max_pos_v is top, etc.
  int zoom_pos_h, zoom_pos_v;
  int max_pos_h, max_pos_v;

  class PicOpts : public Options 
  {
  public:
    PicOpts();
    ~PicOpts();

    std::string slideshow() 
    {
      return val[2]->values[val[2]->pos];
    }
    
    std::string zoom() 
    {
      return val[3]->values[val[3]->pos];
    }

    std::string show_preview() 
    {
      return val[4]->values[val[4]->pos];
    }

    std::string recurse_in_fullscreen()
    {
      return val[5]->values[val[5]->pos];
    }

    std::string thumbnail_extract() 
    {
      return val[6]->values[val[6]->pos];
    }

    std::vector<Option*> values() 
    {
      return val;
    }

  private:
    void read_dirs();

    Option *reload_p, *slideshow_p, *zoom_p, *preview_p, *recurse_p, *thumbnail_extract_p;
  };

  PicOpts opts;

  struct file_sort
  {
    bool operator()(const Simplefile& lhs, const Simplefile& rhs);
  };

  // background thumbnail extraction
  void background_thumbnails();
  int check_thumbnail();

  bool thumbnailed_all_files;
  std::vector<std::string> all_image_files;

  // status of fullscreen
  bool pause;
  long int last_press;

  PictureConfig *pic_conf;

  // notify
  void reparse_current_dir();

public:

  Pictures();

  PicOpts* get_opts()
  {
    return &opts;
  }

  void startup_updater();

  // gcc 2.95 fix

  // for saving information about how you traverse a filesystem a stack is a
  // much better solution. The first element is the dirs and the next is the
  // position
  std::stack<std::pair<std::list<std::string>, int > > folders;

  void print_fullscreen(const Simplefile& e, const std::string& l, const std::string& r);

  std::vector<Picture> pic_list;

  void reset();

  void load_current_dirs();
  void read_dirs();

  void play_pictures_cd();

  std::string mainloop();

  void pictures_check();
  int check_picture_time();
};

#endif
