#include "plugin.hpp"
#include "pictures.hpp"
#include "themes.hpp"
#include "print.hpp"
#include "cd.hpp"
#include "common-feature.hpp"

void pic_hd()
{
  Pictures *pictures = get_class<Pictures>("Picture");

  if (!pictures->loaded_correctly) {
    pictures->read_dirs();
    pictures->loaded_correctly = true;
  } 

  if (pictures->reload_needed) {
    DialogWaitPrint pdialog(gettext("Reloading directories"), 1000);

    pictures->read_dirs();
  }
	    
  if (pictures->pic_list.size() == 0) {
	      
    Print pdialog(Print::SCREEN);
    pdialog.add_line(gettext("Could not find any pictures"));
    pdialog.add_line("");
    pdialog.add_line(gettext("Please specify a correct path "));
    pdialog.add_line(gettext("in the configuration file"));
    pdialog.print();

    pictures->reload_needed = true;

  } else {
    pictures->reload_needed = false;
    pictures->mainloop();
  }
}

void pic_cd()
{
  Cd *cd = S_Cd::get_instance();

  if (cd->open()) {

    Pictures *pictures = get_class<Pictures>("Picture");

    Cd::cdstatus cdstatus = cd->check_cddrive();

    if (cdstatus == Cd::OK)
      pictures->play_pictures_cd();

    cd->close();
  }
}

class PicturesPlugin : public FeaturePlugin
{
public:
  PicturesPlugin() {
    Themes *themes = S_Themes::get_instance();
    Config *conf = S_Config::get_instance();
    PictureConfig *pic_conf = S_PictureConfig::get_instance();

    pic_conf->parse_configuration_file(conf->p_homedir());

    module = new Pictures();

    if (pic_conf->p_picture_hd()) {
      features.push_back(startmenu_item(gettext("View images from harddrive"),
					"picture_hd", themes->startmenu_pictures_dir, 
					0, VoidFunctor(&pic_hd)));
    }
    
    if (conf->p_media())
      features.push_back(startmenu_item(gettext("View images from cd"),
					"picture_disc", themes->startmenu_pictures_cd, 
					1, VoidFunctor(&pic_cd)));
  }
  std::string plugin_name() const { return gettext("Picture"); }
  int priority() const { return 3; }
  unsigned long capability() const { return 0; }
};

MMS_PLUGIN(PicturesPlugin)
