#include "config.h"
#include "plugins.hpp"
#include "simplefile.hpp"
#include "config.hpp"
#include "audio_config.hpp"

#include "player/audioplayer.hpp"

#include <vector>

pymms::player::APlayer::APlayer() : audioplayer_mutex("audioplayer")
{
  audio_player_set = false;
  AudioConfig *audio_conf = S_AudioConfig::get_instance();
  Plugins *plugins = S_Plugins::get_instance();

  for (std::vector<AudioPlayerPlugin*>::const_iterator iter=plugins->audio_players.begin(),
         iend=plugins->audio_players.end(); iter!=iend; ++iter)
    if ( (*iter)->plugin_name() == audio_conf->p_audio_player()) {
      audioplayer = (*iter)->player;
      audio_player_set = true;
      break;
    }

  if(!audio_player_set)
    DialogWaitPrint pdialog(gettext("No audio support detected"), 2000);

}

void pymms::player::APlayer::play(std::string path, std::string name)
{
  #ifdef c_music
  ost::MutexLock lock(audioplayer_mutex);
  if(audio_player_set)
  {
    Simplefile track;
    track.path = path;
    track.name = name;

    audioplayer->set_cur_nr(track);  
    audioplayer->addfile(track);
  }
  #endif
}

void pymms::player::APlayer::play()
{
  #ifdef c_music
  ost::MutexLock lock(audioplayer_mutex);
  if(audio_player_set)
    audioplayer->play();
  #endif
}

void pymms::player::APlayer::stop()
{
  #ifdef c_music
  ost::MutexLock lock(audioplayer_mutex);
  if(audio_player_set)
    audioplayer->stop();
  #endif
}

void pymms::player::APlayer::pause()
{
  #ifdef c_music
  ost::MutexLock lock(audioplayer_mutex);
  if(audio_player_set)
    audioplayer->pause();
  #endif
}

void pymms::player::APlayer::mute()
{
  #ifdef c_music
  ost::MutexLock lock(audioplayer_mutex);
  if(audio_player_set)
    audioplayer->mute();
  #endif
}

void pymms::player::APlayer::volup()
{
  #ifdef c_music
  ost::MutexLock lock(audioplayer_mutex);
  if(audio_player_set)
    audioplayer->volup();
  #endif
}

void pymms::player::APlayer::voldown()
{
  #ifdef c_music
  ost::MutexLock lock(audioplayer_mutex);
  if(audio_player_set)
    audioplayer->voldown();
  #endif
}

bool pymms::player::APlayer::is_playing()
{
  #ifdef c_music
  ost::MutexLock lock(audioplayer_mutex);
  if(audio_player_set)
    return audioplayer->is_playing();
  #endif
}
