#include "pymms/mmspython.hpp"

pymms::MMSPython::MMSPython() : m_critSection("mmspython")
{
  bInitialized = false;
  nextid = 0;
  mainThreadState = NULL;
}

void pymms::MMSPython::initialize()
{
  ost::MutexLock lock(m_critSection);
  if(!bInitialized)
  {
    Py_Initialize();
    PyEval_InitThreads();

    mainThreadState = PyThreadState_Get();
			
    // release the lock
    PyEval_ReleaseLock();

    bInitialized = true;
  }
}

int pymms::MMSPython::eval_file(std::string src)
{
  initialize();
  ost::MutexLock lock(m_critSection);
  if (!bInitialized) 
    return -1;

  nextid++;
  MMSPyThread *pyThread = new MMSPyThread(this, mainThreadState, nextid);
  pyThread->eval_file(src);
  PyElem inf;
  inf.id = nextid;
  inf.bDone = false;
  inf.strFile = src;
  inf.pyThread = pyThread;

  vecPyList.push_back(inf);

  return nextid;
}

void pymms::MMSPython::set_done(int id)
{
  ost::MutexLock lock(m_critSection);
  PyList::iterator it = vecPyList.begin();
  while (it != vecPyList.end())
  {
    if (it->id == id)
    {
      it = vecPyList.erase(it);
      break;
    }
    ++it;
  }
}

PyThreadState *pymms::MMSPython::get_mainthread_state()
{
  ost::MutexLock lock(m_critSection);
  return mainThreadState;
}

int pymms::MMSPython::scripts_size()
{
  ost::MutexLock lock(m_critSection);
  return vecPyList.size();
}

const char* pymms::MMSPython::get_filename(int scriptId)
{
  ost::MutexLock lock(m_critSection);
  const char* cFileName = NULL;

  PyList::iterator it = vecPyList.begin();
  while (it != vecPyList.end())
  {
    if (it->id == scriptId) 
      cFileName = it->strFile.c_str();
    ++it;
  } 

  return cFileName;
}

int pymms::MMSPython::get_scriptid(std::string strFile)
{
  ost::MutexLock lock(m_critSection);
  int iId = -1;

  PyList::iterator it = vecPyList.begin();
  while (it != vecPyList.end())
  {
    if (it->strFile == strFile) 
      iId = it->id;
    
    ++it;
  }

  return iId;
}

bool pymms::MMSPython::is_running(int scriptId)
{
  ost::MutexLock lock(m_critSection);
  bool bRunning = false;

  PyList::iterator it = vecPyList.begin();
  while (it != vecPyList.end())
  {
    if (it->id == scriptId)	
      bRunning = true;
    ++it;
  }

  return bRunning;
}

int pymms::MMSPython::get_pythonscript_id(int scriptPosition)
{
  ost::MutexLock lock(m_critSection);
  return (int)vecPyList[scriptPosition].id;
}
