#include <Python.h>
#include "structmember.h"
#include "pycontrol.hpp"

using namespace pymms::gui;

#ifdef __cplusplus
extern "C" {
#endif

namespace pymms {

namespace python {

PyObject* ButtonControl_New(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
  ButtonControl *self;
  int x = 0, y = 0, w = 0, h = 0, l = 4, tos = 0;
  char *text = "";
  char *focus = "";
  char *nofocus = "";
  char *font = "Vera";
  char *rgb = "0xffffff";
  char *rgbfocus = "0xffffff";
  char *alignment = "left";
  
  static char *kwlist[] = {"x", "y", "width", "height", 
                           "text", "font", "rgb", "rgbfocus", "focus", "nofocus", 
			   "textoffset", "alignment", NULL};
  
  self = (ButtonControl*)type->tp_alloc(type, 0);
  if (!self) 
    return NULL;
    
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "llll|ssssssls", kwlist, &x, &y, 
                                   &w, &h, &text, &font, &rgb, &rgbfocus, &focus, &nofocus, &tos, &alignment))
    return NULL;
    
  self->guicontrol = new GUIButtonControl(x, y, w, h, text, l, focus, nofocus, font, rgb, rgbfocus, tos, alignment);

  return (PyObject*)self;
}

PyObject* ButtonControl_set_label(ButtonControl *self, PyObject *args, PyObject *kwds)
{
  char *text = NULL;
  char *font = NULL;
  char *rgb = NULL;
  char *rgbfocus = NULL;

  static char *kwlist[] = {"text", "font", "rgb", "rgbfocus", NULL};
    
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "|ssss", kwlist, &text, &font, &rgb, &rgbfocus))	
    return NULL;

  if(text)
    ((GUIButtonControl*)self->guicontrol)->set_label(text);
  if(font)
    ((GUIButtonControl*)self->guicontrol)->set_labelfont(font);
  if(rgb)
    ((GUIButtonControl*)self->guicontrol)->set_labelrgb(rgb);
  if(rgbfocus)
    ((GUIButtonControl*)self->guicontrol)->set_labelrgbfocus(rgbfocus);

  Py_INCREF(Py_None);
  return Py_None;
}

void ButtonControl_Dealloc(ButtonControl* self)
{
  delete self->guicontrol;
  self->ob_type->tp_free((PyObject*)self);
}

PyMethodDef ButtonControl_methods[] = {
  {"set_label", (PyCFunction)ButtonControl_set_label, METH_VARARGS | METH_KEYWORDS, 0},
  {NULL, NULL, 0, NULL}
};

PyTypeObject ButtonControl_Type = {
	PyObject_HEAD_INIT(NULL)
	0,                         /*ob_size*/
	"mmsv2gui.ButtonControl",    /*tp_name*/
	sizeof(ButtonControl),      /*tp_basicsize*/
	0,                         /*tp_itemsize*/
	(destructor)ButtonControl_Dealloc,/*tp_dealloc*/
	0,                         /*tp_print*/
	0,                         /*tp_getattr*/
	0,                         /*tp_setattr*/
	0,                         /*tp_compare*/
	0,                         /*tp_repr*/
	0,                         /*tp_as_number*/
	0,                         /*tp_as_sequence*/
	0,                         /*tp_as_mapping*/
	0,                         /*tp_hash */
	0,                         /*tp_call*/
	0,                         /*tp_str*/
	0,                         /*tp_getattro*/
	0,                         /*tp_setattro*/
	0,                         /*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
	0,       /* tp_doc */
	0,		                     /* tp_traverse */
	0,		                     /* tp_clear */
	0,		                     /* tp_richcompare */
	0,		                     /* tp_weaklistoffset */
	0,		                     /* tp_iter */
	0,		                     /* tp_iternext */
	ButtonControl_methods,      /* tp_methods */
	0,                         /* tp_members */
	0,                         /* tp_getset */
	&Control_Type,             /* tp_base */
	0,                         /* tp_dict */
	0,                         /* tp_descr_get */
	0,                         /* tp_descr_set */
	0,                         /* tp_dictoffset */
	0,                         /* tp_init */
	0,                         /* tp_alloc */
	ButtonControl_New,          /* tp_new */
};

}

}

#ifdef __cplusplus
}
#endif
