#include "pycontrol.hpp"

#ifdef __cplusplus
extern "C" {
#endif

namespace pymms {

namespace python {

PyObject* Control_set_visible(Control* self, PyObject* args)
{
  bool visible;

  if (!PyArg_ParseTuple(args, "b", &visible)) 
    return NULL;

  if (self->guicontrol) 	
    self->guicontrol->set_visible(visible);

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject* Control_get_visible(Control* self, PyObject* args)
{
  return Py_BuildValue("b", self->guicontrol->get_visible());
}

PyObject* Control_set_height(Control* self, PyObject* args)
{
  int height;

  if (!PyArg_ParseTuple(args, "l", &height))
    return NULL;

  if (self->guicontrol)
    self->guicontrol->set_height(height);

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject* Control_set_width(Control* self, PyObject* args)
{
  int width;

  if (!PyArg_ParseTuple(args, "l", &width))
    return NULL;

  if (self->guicontrol)
    self->guicontrol->set_width(width);

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject* Control_set_layer(Control* self, PyObject* args)
{
  int layer;

  if (!PyArg_ParseTuple(args, "l", &layer))
    return NULL;

  if (self->guicontrol)
    self->guicontrol->set_layer(layer);

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject* Control_set_position(Control* self, PyObject* args)
{
  int x, y;

  if (!PyArg_ParseTuple(args, "ll", &x, &y))
    return NULL;

  if (self->guicontrol)
    self->guicontrol->set_position(x, y);

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject* Control_control_left(Control* self, PyObject* args)
{
  Control *control;

  if (!PyArg_ParseTuple(args, "O", &control))
    return NULL;

  if (!Control_Check(control))
  {
    PyErr_SetString(PyExc_TypeError, "Object should be of type Control");
    return NULL;
  }
  if(control->guicontrol->get_id() == 0)
  {
    PyErr_SetString(PyExc_ReferenceError, "Control has to be added to a window first");
    return NULL;
  }

  if (self->guicontrol)
    self->guicontrol->control_left(control->guicontrol);

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject* Control_control_right(Control* self, PyObject* args)
{
  Control *control;

  if (!PyArg_ParseTuple(args, "O", &control))
    return NULL;

  if (!Control_Check(control))
  {
    PyErr_SetString(PyExc_TypeError, "Object should be of type Control");
    return NULL;
  }
  if(control->guicontrol->get_id() == 0)
  {
    PyErr_SetString(PyExc_ReferenceError, "Control has to be added to a window first");
    return NULL;
  }

  if (self->guicontrol)
    self->guicontrol->control_right(control->guicontrol);

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject* Control_control_prev(Control* self, PyObject* args)
{
  Control *control;

  if (!PyArg_ParseTuple(args, "O", &control))
    return NULL;

  if (!Control_Check(control))
  {
    PyErr_SetString(PyExc_TypeError, "Object should be of type Control");
    return NULL;
  }
  if(control->guicontrol->get_id() == 0)
  {
    PyErr_SetString(PyExc_ReferenceError, "Control has to be added to a window first");
    return NULL;
  }

  if (self->guicontrol)
    self->guicontrol->control_prev(control->guicontrol);

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject* Control_control_next(Control* self, PyObject* args)
{
  Control *control;

  if (!PyArg_ParseTuple(args, "O", &control))
    return NULL;

  if (!Control_Check(control))
  {
    PyErr_SetString(PyExc_TypeError, "Object should be of type Control");
    return NULL;
  }
  if(control->guicontrol->get_id() == 0)
  {
    PyErr_SetString(PyExc_ReferenceError, "Control has to be added to a window first");
    return NULL;
  }

  if (self->guicontrol)
    self->guicontrol->control_next(control->guicontrol);

  Py_INCREF(Py_None);
  return Py_None;
}


PyMethodDef Control_methods[] = {
  {"get_visible", (PyCFunction)Control_get_visible, METH_VARARGS, 0},
  {"set_visible", (PyCFunction)Control_set_visible, METH_VARARGS, 0},
  {"control_left", (PyCFunction)Control_control_left, METH_VARARGS, 0},
  {"control_right", (PyCFunction)Control_control_right, METH_VARARGS, 0},
  {"control_next", (PyCFunction)Control_control_next, METH_VARARGS, 0},
  {"control_prev", (PyCFunction)Control_control_prev, METH_VARARGS, 0},
  {"set_height", (PyCFunction)Control_set_height, METH_VARARGS, 0},
  {"set_width", (PyCFunction)Control_set_width, METH_VARARGS, 0},
  {"set_layer", (PyCFunction)Control_set_layer, METH_VARARGS, 0},
  {"set_position", (PyCFunction)Control_set_position, METH_VARARGS, 0},
  {NULL, NULL, 0, NULL}
};

PyTypeObject Control_Type = {
	PyObject_HEAD_INIT(NULL)
	0,                         /*ob_size*/
	"mmsv2gui.Control",         /*tp_name*/
	sizeof(Control),           /*tp_basicsize*/
	0,                         /*tp_itemsize*/
	0,//(destructor)Control_Dealloc,/*tp_dealloc*/
	0,                         /*tp_print*/
	0,                         /*tp_getattr*/
	0,                         /*tp_setattr*/
	0,           /*tp_compare*/
	0,                         /*tp_repr*/
	0,                         /*tp_as_number*/
	0,                         /*tp_as_sequence*/
	0,                         /*tp_as_mapping*/
	0,                         /*tp_hash */
	0,                         /*tp_call*/
	0,                         /*tp_str*/
	0,                         /*tp_getattro*/
	0,                         /*tp_setattro*/
	0,                         /*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
	0,            /* tp_doc */
	0,		                     /* tp_traverse */
	0,		                     /* tp_clear */
	0,		                     /* tp_richcompare */
	0,		                     /* tp_weaklistoffset */
	0,		                     /* tp_iter */
	0,		                     /* tp_iternext */
	Control_methods,           /* tp_methods */
	0,                         /* tp_members */
	0,                         /* tp_getset */
	0,                         /* tp_base */
	0,                         /* tp_dict */
	0,                         /* tp_descr_get */
	0,                         /* tp_descr_set */
	0,                         /* tp_dictoffset */
	0,                         /* tp_init */
	0,                         /* tp_alloc */
	0,//Control_New,               /* tp_new */
};

}

}

#ifdef __cplusplus
}
#endif

