#include <Python.h>
#include "structmember.h"
#include "pycontrol.hpp"

using namespace pymms::gui;

#ifdef __cplusplus
extern "C" {
#endif

namespace pymms {

namespace python {

static PyObject* ImageControl_New(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
  ImageControl *self;
  int x = 0, y = 0, w = 0, h = 0, l = 1;
  char *path = "";
  
  static char *kwlist[] = {"x", "y", "width", "height", "path", NULL};

  self = (ImageControl*)type->tp_alloc(type, 0);
  if (!self) 
    return NULL; 
  
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "llll|s", kwlist, &x, &y, 
                                   &w, &h, &path))
    return NULL;

  if( (w*h) < 640*480)
    l = 3;
  
  self->guicontrol = new GUIImageControl(x, y, w, h, path, l);
  
  return (PyObject*)self;
}

static void ImageControl_Dealloc(ImageControl* self)
{
  delete self->guicontrol;
  self->ob_type->tp_free((PyObject*)self);
}

static PyMethodDef ImageControl_methods[] = {
  {NULL, NULL, 0, NULL}
};

PyTypeObject ImageControl_Type = {
	PyObject_HEAD_INIT(NULL)
	0,                         /*ob_size*/
	"mmsv2gui.ImageControl",    /*tp_name*/
	sizeof(ImageControl),      /*tp_basicsize*/
	0,                         /*tp_itemsize*/
	(destructor)ImageControl_Dealloc,/*tp_dealloc*/
	0,                         /*tp_print*/
	0,                         /*tp_getattr*/
	0,                         /*tp_setattr*/
	0,                         /*tp_compare*/
	0,                         /*tp_repr*/
	0,                         /*tp_as_number*/
	0,                         /*tp_as_sequence*/
	0,                         /*tp_as_mapping*/
	0,                         /*tp_hash */
	0,                         /*tp_call*/
	0,                         /*tp_str*/
	0,                         /*tp_getattro*/
	0,                         /*tp_setattro*/
	0,                         /*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
	0,       /* tp_doc */
	0,		                     /* tp_traverse */
	0,		                     /* tp_clear */
	0,		                     /* tp_richcompare */
	0,		                     /* tp_weaklistoffset */
	0,		                     /* tp_iter */
	0,		                     /* tp_iternext */
	ImageControl_methods,      /* tp_methods */
	0,                         /* tp_members */
	0,                         /* tp_getset */
	&Control_Type,             /* tp_base */
	0,                         /* tp_dict */
	0,                         /* tp_descr_get */
	0,                         /* tp_descr_set */
	0,                         /* tp_dictoffset */
	0,                         /* tp_init */
	0,                         /* tp_alloc */
	ImageControl_New,          /* tp_new */
};

}

}

#ifdef __cplusplus
}
#endif
