#include <Python.h>
#include "structmember.h"
#include "pycontrol.hpp"

using namespace pymms::gui;

#ifdef __cplusplus
extern "C" {
#endif

namespace pymms {

namespace python {

static PyObject* TextBoxControl_New(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
  TextBoxControl *self;
  int x = 0, y = 0, w = 0, h = 0, l = 4, textheight = 30, textspace = 0;
  char *font = "Vera";
  char *rgb = "0xffffff";
  
  static char *kwlist[] = {"x", "y", "width", "height", 
                           "font", "rgb", "textheight", "textspace", NULL};
  
  self = (TextBoxControl*)type->tp_alloc(type, 0);
  if (!self) 
    return NULL;
    
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "llll|ssll", kwlist, &x, &y, 
                                   &w, &h, &font, &rgb, &textheight, &textspace))
    return NULL;
    
  self->guicontrol = new GUITextBoxControl(x, y, w, h, l, font, rgb, textheight, textspace);

  return (PyObject*)self;
}

static PyObject* TextBoxControl_set_text(TextBoxControl *self, PyObject *args, PyObject *kwds)
{
  char *text = NULL;
  char *font = NULL;
  char *rgb = NULL;

  static char *kwlist[] = {"text", "font", "rgb", NULL};
    
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "|sss", kwlist, &text, &font, &rgb))	
    return NULL;

  if(text)
    ((GUITextBoxControl*)self->guicontrol)->set_text(text);
  if(font)
    ((GUITextBoxControl*)self->guicontrol)->set_textfont(font);
  if(rgb)
    ((GUITextBoxControl*)self->guicontrol)->set_textrgb(rgb);

  Py_INCREF(Py_None);
  return Py_None;
}

static void TextBoxControl_Dealloc(TextBoxControl* self)
{
  delete self->guicontrol;
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* TextBoxControl_clear(TextBoxControl *self, PyObject *args)
{
  ((GUITextBoxControl*)self->guicontrol)->clear();

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* TextBoxControl_set_textheight(TextBoxControl *self, PyObject *args)
{
  int textheight = 0;

  if (!PyArg_ParseTuple(args, "l", &textheight))
    return NULL;

  ((GUITextBoxControl*)self->guicontrol)->set_textheight(textheight);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* TextBoxControl_set_textspace(TextBoxControl *self, PyObject *args)
{
  int textspace = 0;

  if (!PyArg_ParseTuple(args, "l", &textspace))
    return NULL;

  ((GUITextBoxControl*)self->guicontrol)->set_textspace(textspace);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyMethodDef TextBoxControl_methods[] = {
  {"set_text", (PyCFunction)TextBoxControl_set_text, METH_VARARGS | METH_KEYWORDS, 0},
  {"clear", (PyCFunction)TextBoxControl_clear, METH_VARARGS, 0},
  {"set_textheight", (PyCFunction)TextBoxControl_set_textheight, METH_VARARGS, 0},
  {"set_textspace", (PyCFunction)TextBoxControl_set_textspace, METH_VARARGS, 0},
  {NULL, NULL, 0, NULL}
};

PyTypeObject TextBoxControl_Type = {
	PyObject_HEAD_INIT(NULL)
	0,                         /*ob_size*/
	"mmsv2gui.TextBoxControl",    /*tp_name*/
	sizeof(TextBoxControl),      /*tp_basicsize*/
	0,                         /*tp_itemsize*/
	(destructor)TextBoxControl_Dealloc,/*tp_dealloc*/
	0,                         /*tp_print*/
	0,                         /*tp_getattr*/
	0,                         /*tp_setattr*/
	0,                         /*tp_compare*/
	0,                         /*tp_repr*/
	0,                         /*tp_as_number*/
	0,                         /*tp_as_sequence*/
	0,                         /*tp_as_mapping*/
	0,                         /*tp_hash */
	0,                         /*tp_call*/
	0,                         /*tp_str*/
	0,                         /*tp_getattro*/
	0,                         /*tp_setattro*/
	0,                         /*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
	0,       /* tp_doc */
	0,		                     /* tp_traverse */
	0,		                     /* tp_clear */
	0,		                     /* tp_richcompare */
	0,		                     /* tp_weaklistoffset */
	0,		                     /* tp_iter */
	0,		                     /* tp_iternext */
	TextBoxControl_methods,      /* tp_methods */
	0,                         /* tp_members */
	0,                         /* tp_getset */
	&Control_Type,             /* tp_base */
	0,                         /* tp_dict */
	0,                         /* tp_descr_get */
	0,                         /* tp_descr_set */
	0,                         /* tp_dictoffset */
	0,                         /* tp_init */
	0,                         /* tp_alloc */
	TextBoxControl_New,          /* tp_new */
};

}

}

#ifdef __cplusplus
}
#endif
