#include "pywindowdialog.hpp"
#include "gui/guiwindowmanager.hpp"

using namespace pymms::gui;

#ifdef __cplusplus
extern "C" {
#endif

namespace pymms {

namespace python {

PyObject* Dialog_New(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
  Dialog *self;

  self = (Dialog*)type->tp_alloc(type, 0);
  if (!self) 
    return NULL;

  return (PyObject*)self;
}

void Dialog_Dealloc(Dialog* self)
{
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* Dialog_yesno(Dialog *self, PyObject *args, PyObject *kwds)
{
  char *h = NULL;
  char *l1 = NULL;
  char *l2 = "";
  char *l3 = "";
  bool retval = false;

  static char *kwlist[] = {"header", "line1", "line2", "line3", NULL};

  if (!PyArg_ParseTupleAndKeywords(args, kwds, "ss|ss", kwlist, &h, &l1, &l2, &l3))
    return NULL;
  
  GUIWindowManager *windowmanager = S_WindowManager::get_instance();

  GUIWindowDialogYesNo *yesnowin = new GUIWindowDialogYesNo(h, l1, l2, l3);

  windowmanager->register_window(yesnowin);
  windowmanager->set_active_window(yesnowin);

  retval = yesnowin->yesno();

  windowmanager->unregister_window(yesnowin);
  delete yesnowin;

  return Py_BuildValue("b", retval);
}

PyObject* Dialog_ok(Dialog *self, PyObject *args, PyObject *kwds)
{
  char *h = NULL;
  char *l1 = NULL;
  char *l2 = "";
  char *l3 = "";

  static char *kwlist[] = {"header", "line1", "line2", "line3", NULL};

  if (!PyArg_ParseTupleAndKeywords(args, kwds, "ss|ss", kwlist, &h, &l1, &l2, &l3))
    return NULL;

  GUIWindowManager *windowmanager = S_WindowManager::get_instance();

  GUIWindowDialogOk *okwin = new GUIWindowDialogOk(h, l1, l2, l3);

  windowmanager->register_window(okwin);
  windowmanager->set_active_window(okwin);

  okwin->ok();

  windowmanager->unregister_window(okwin);
  delete okwin;

  return Py_BuildValue("b", true);
}

PyObject* DialogProgress_New(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
  DialogProgress *self;

  self = (DialogProgress*)type->tp_alloc(type, 0);
  if (!self)
    return NULL;

  self->dialogProgress = NULL;

  return (PyObject*)self;
}

PyObject* DialogProgress_close(DialogProgress *self, PyObject *args)
{
  if(self->dialogProgress)
  {
    S_WindowManager::get_instance()->unregister_window(self->dialogProgress);
    delete self->dialogProgress;
    self->dialogProgress = NULL;
  }

  Py_INCREF(Py_None);
  return Py_None;
}

void DialogProgress_Dealloc(DialogProgress* self)
{
  DialogProgress_close(self, NULL);
  self->ob_type->tp_free((PyObject*)self);
}

PyObject* DialogProgress_create(DialogProgress *self, PyObject *args, PyObject *kwds)
{
  char *h = NULL;
  char *l1 = "";
  char *l2 = "";
  char *l3 = "";

  if(!self->dialogProgress)
  {
    static char *kwlist[] = {"header", "line1", "line2", "line3", NULL};

    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s|sss", kwlist, &h, &l1, &l2, &l3))
      return NULL;
    
    GUIWindowManager *windowmanager = S_WindowManager::get_instance();

    self->dialogProgress = new GUIWindowDialogProgress(h, l1, l2, l3);

    windowmanager->register_window(self->dialogProgress);
    windowmanager->set_active_window(self->dialogProgress);
  }

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject* DialogProgress_update(DialogProgress *self, PyObject *args)
{
  int percent = 0;

  if(self->dialogProgress)
  {
    if (!PyArg_ParseTuple(args, "l", &percent))
      return NULL;

    self->dialogProgress->update(percent);
  }

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject* DialogProgress_is_canceled(DialogProgress *self, PyObject *args)
{
  bool retval = false;

  if(self->dialogProgress)
    retval = self->dialogProgress->is_canceled();

  return Py_BuildValue("b", retval);
}

PyObject* DialogProgress_set_text(DialogProgress *self, PyObject *args, PyObject *kwds)
{
  char *h = NULL;
  char *l1 = NULL;
  char *l2 = NULL;
  char *l3 = NULL;

  if(self->dialogProgress)
  {
    static char *kwlist[] = {"header", "line1", "line2", "line3", NULL};

    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|ssss", kwlist, &h, &l1, &l2, &l3))
      return NULL;

    if(h)
      self->dialogProgress->set_header(h);
    if(l1)
      self->dialogProgress->set_line1(l1);
    if(l2)
      self->dialogProgress->set_line2(l2);
    if(l3)
      self->dialogProgress->set_line3(l3);
  }

  Py_INCREF(Py_None);
  return Py_None;
}

PyMethodDef Dialog_methods[] = {
  {"yesno", (PyCFunction)Dialog_yesno, METH_VARARGS | METH_KEYWORDS, 0},
  {"ok", (PyCFunction)Dialog_ok, METH_VARARGS | METH_KEYWORDS, 0},
  {NULL, NULL, 0, NULL}
};

PyMethodDef DialogProgress_methods[] = {
  {"create", (PyCFunction)DialogProgress_create, METH_VARARGS | METH_KEYWORDS, 0},
  {"close", (PyCFunction)DialogProgress_close, METH_VARARGS, 0},
  {"update", (PyCFunction)DialogProgress_update, METH_VARARGS, 0},
  {"is_canceled", (PyCFunction)DialogProgress_is_canceled, METH_VARARGS, 0},
  {"set_text", (PyCFunction)DialogProgress_set_text, METH_VARARGS | METH_KEYWORDS, 0},
  {NULL, NULL, 0, NULL}
};

PyTypeObject Dialog_Type = {
	PyObject_HEAD_INIT(NULL)
	0,                         /*ob_size*/
	"mmsv2gui.Dialog",         /*tp_name*/
	sizeof(Dialog),            /*tp_basicsize*/
	0,                         /*tp_itemsize*/
	(destructor)Dialog_Dealloc,/*tp_dealloc*/
	0,                         /*tp_print*/
	0,                         /*tp_getattr*/
	0,                         /*tp_setattr*/
	0,                         /*tp_compare*/
	0,                         /*tp_repr*/
	0,                         /*tp_as_number*/
	0,                         /*tp_as_sequence*/
	0,                         /*tp_as_mapping*/
	0,                         /*tp_hash */
	0,                         /*tp_call*/
	0,                         /*tp_str*/
	0,                         /*tp_getattro*/
	0,                         /*tp_setattro*/
	0,                         /*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
	0,      /* tp_doc */
	0,		                     /* tp_traverse */
	0,		                     /* tp_clear */
	0,		                     /* tp_richcompare */
	0,		                     /* tp_weaklistoffset */
	0,		                     /* tp_iter */
	0,		                     /* tp_iternext */
	Dialog_methods,            /* tp_methods */
	0,                         /* tp_members */
	0,                         /* tp_getset */
	0,                         /* tp_base */
	0,                         /* tp_dict */
	0,                         /* tp_descr_get */
	0,                         /* tp_descr_set */
	0,                         /* tp_dictoffset */
	0,                         /* tp_init */
	0,                         /* tp_alloc */
	Dialog_New,                /* tp_new */
};

PyTypeObject DialogProgress_Type = {
        PyObject_HEAD_INIT(NULL)
        0,                         /*ob_size*/
        "mmsv2gui.DialogProgress",         /*tp_name*/
        sizeof(DialogProgress),            /*tp_basicsize*/
        0,                         /*tp_itemsize*/
        (destructor)DialogProgress_Dealloc,/*tp_dealloc*/
        0,                         /*tp_print*/
        0,                         /*tp_getattr*/
        0,                         /*tp_setattr*/
        0,                         /*tp_compare*/
        0,                         /*tp_repr*/
        0,                         /*tp_as_number*/
        0,                         /*tp_as_sequence*/
        0,                         /*tp_as_mapping*/
        0,                         /*tp_hash */
        0,                         /*tp_call*/
        0,                         /*tp_str*/
        0,                         /*tp_getattro*/
        0,                         /*tp_setattro*/
        0,                         /*tp_as_buffer*/
        Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
        0,      /* tp_doc */
        0,                                   /* tp_traverse */
        0,                                   /* tp_clear */
        0,                                   /* tp_richcompare */
        0,                                   /* tp_weaklistoffset */
        0,                                   /* tp_iter */
        0,                                   /* tp_iternext */
        DialogProgress_methods,            /* tp_methods */
        0,                         /* tp_members */
        0,                         /* tp_getset */
        0,                         /* tp_base */
        0,                         /* tp_dict */
        0,                         /* tp_descr_get */
        0,                         /* tp_descr_set */
        0,                         /* tp_dictoffset */
        0,                         /* tp_init */
        0,                         /* tp_alloc */
        DialogProgress_New,                /* tp_new */
};

}

}

#ifdef __cplusplus
}
#endif


