/* 
 *  Copyright (C) 2006 by:
 *  Da He <da.he@uni-dortmund.de>
 *  Julian Flake <julian.flake@uni-dortmund.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef _CRENDERER_H
#define _CRENDERER_H

#include "players/audio_player.hpp"
#include "singleton.hpp"
#include "module.hpp"

#include "cService.hpp"
#include "cConnectionManager.hpp"
#include "cAVTransport.hpp"
#include "cRenderingControl.hpp"

#include <uuid/uuid.h>

#include <upnp/upnp.h>
#include <upnp/upnptools.h>
#include <upnp/ixml.h>
#include <upnp/ithread.h>

#define DEBUG

class cRenderer : public Module, public ost::Thread
{
private:

  enum {
    ConnectionManager,
    AVTransport,
    RenderingControl
  };
  cService *services[3] ;

  // mutex
  ithread_mutex_t rendererMutex;
	
  // methods
  static void wakeup(int in);
  IXML_Document* getDescriptionDoc();

  void trim(std::string& str);
  void str_replace(std::string needle, std::string replacement, std::string& haystack);
  void getVarName(std::string& varName, std::string& haystack);
  void getValue(std::string& value, std::string& haystack);

  char* get_uuid();
  char* generate_uuid();

  void handleActionRequest(Upnp_Action_Request* Event);
  void handleGetVarRequest(Upnp_State_Var_Request* Event);
  void handleSubscriptionRequest(Upnp_Subscription_Request* Event);
	
  AudioPlayer *audioplayer;

  std::string web_root;
  char *friendlyName;
  char xmlIcon[100];
  std::string ip_addr;
	
public:
	
  // cService needs this stuff
  char* udn;
  UpnpDevice_Handle renderer_hdl;

  // module stuff
  Options *get_opts() { return 0; }
  void startup_updater() { start(); }
  std::string mainloop() {}
  void run();
  bool init() { return false; };
	
  cRenderer();
  ~cRenderer();

  static int callbackEventHandler (Upnp_EventType EventType, void *Event, void *Cookie);

  char* getXMLValue(IXML_Document *doc, char tagName[]);
  //void setVar(Upnp_Action_Request *Event,int service, int var, char* val);

  void seconds2tstamp(char* out, const char* in);
  int time2seconds(char* in);

  int positionToSeek;

  void play();
  void stop();
  void pause();

  static void* PositionPercentThreadMethod(void *ptr);
  static void* PositionTimeThreadMethod(void *ptr);
};

typedef Singleton<cRenderer> S_Renderer;

#endif /* _CRENDERER_H */
