#ifndef KEYBOARD_HPP
#define KEYBOARD_HPP

#include "config.h"

#include "input.hpp"
#include "config.hpp"
#include "common-output.hpp"

#ifdef use_sdl
#include <SDL/SDL.h>
#endif

// keyboard
#include <termios.h>
#include "termio/getch2.h"
#include "termio/keycodes.h"

#include <list>
#include <string>

#include <iostream>

class Keyboard : public InputDevice 
{
private:
  // delay for input
  int halfdelay_time;

  int tty_mode;
  struct termios orig_tty;
  struct termios new_tty;

#ifdef use_sdl
  std::string getSDLkey();
#endif

  void normal();
  void cbreak();

  std::list<std::string> not_accepted_keys();

  std::list<std::string> not_accepted_chars;

  void generate_search_top()
  {
    not_accepted_chars = not_accepted_keys();
  }

  void generate_search_input(std::string& str, std::string &lowercase_search_str,
			     Input input)
  {
    if (input.command != "erase") {
      bool found_match = false;

      if (input.key == "SPACE")
	input.key = " ";
      else
	for (std::list<std::string>::iterator i = not_accepted_chars.begin(), end = not_accepted_chars.end();
	     i != end; ++i)
	  if (input.key == *i)
	    found_match = true;

      if (!found_match) {
	str += input.key;
	lowercase_search_str = string_format::lowercase(str);
      }
    } else // erase
      if (input.mode == "search" && str.size() != 0) {
	str = str.substr(0, str.size()-1);
	lowercase_search_str = string_format::lowercase(str);
      }
  }

  void search_top()
  {
    not_accepted_chars = not_accepted_keys();
  }

  void search_input(std::string &search_str, std::string &lowercase_search_str,
		    Input input, int& offset, GenSearchBaseFunc *func)
  {
    if (input.command == "complete")
      complete_string(search_str, lowercase_search_str, func);
	
    generate_search_input(search_str, lowercase_search_str, input);

    if (input.command == "next")
      ++offset;
    else if (input.command == "prev") {
      --offset;
      if (offset == -1)
	offset = func->size(search_str)-1;
    } else 
      offset = 0;
  }
  
  void search_graphical_input(std::string &search_str, std::string &lowercase_search_str,
			      Input input, int& offset, int& search_help_offset, GenSearchBaseFunc *func)
  {
    // FIXME: refactor this function

    generate_search_input(search_str, lowercase_search_str, input);

    // FIXME: right = change search level

    if (input.command == "next")
      ++offset;
    else if (input.command == "prev") {
      --offset;
      if (offset == -1)
	offset = func->size(search_str)-1;
    } else 
      offset = 0;
  }

public:

  void run();
  bool init();

  Keyboard();
  ~Keyboard();
  
  void suspend() {}
  void wake_up() { Input i; input = i; }
};

#endif
