#include "search.hpp"

#include "global.hpp"
#include "graphics.hpp"
#include "extra_menu.hpp"
#include "busy_indicator.hpp"

SearchModule::SearchModule(const std::string& n, SearchBaseGenList* sgl, EnterBaseFunctor* e_f)
  : list_gen(sgl), enter_func(e_f), name(n)
{}

Search::Search()
  : search_word(""), search_module("all"), pos(-1), em("", true)
{
  em.set_drop_down_pos(conf->p_v_res()-250+85, 85);
  em.add_item(ExtraMenuItem("all", "", new TimeFunctorvstring<Search>(this, &Search::set_module, "all")));
}

void Search::set_module(const std::string& mod)
{
  search_module = mod;
}

void Search::print_element(const result& r, const result& position, int& y)
{
  if (r.module != cur_module) {

    if (y != 135)
      y += 10;
    
    render->current.add(new TObj(r.module, "Vera/18", 85, y, 
				 themes->search_module_font1, themes->search_module_font2, 
				 themes->search_module_font3, 3));
    y += 35;
    cur_module = r.module;
  }

  std::string name = r.name;

  string_format::format_to_size(name, "Vera/16", conf->p_v_res()-2*75, true);
  
  if (r == position) {
    // FIXME: movie marked
    render->current.add(new PObj("movie/marked.png", 70, y-1, conf->p_v_res()-(2*70), 2));
    render->current.add(new TObj(name, "Vera/16", 100, y, 
				 themes->search_marked_font1, themes->search_marked_font2, 
				 themes->search_marked_font3, 3));
  } else {
    render->current.add(new TObj(name, "Vera/16", 100, y, 
				 themes->search_font1, themes->search_font2, 
				 themes->search_font3, 3));
  }
}

void Search::print(const std::string& search_w)
{
  render->prepare_new_image();

  render->current.add(new PObj(themes->search_background, 0, 0, 0, SCALE_FULL));
  
  render->current.add(new PObj(themes->general_search, 25, 10, 2, NOSCALING));

  if (themes->show_header) {
    render->current.add(new TObj(gettext("Search"), "Vera/28", 100, 10, 
				 themes->search_header_font1, themes->search_header_font2, 
				 themes->search_header_font3, 2));
  }

  int x = 85, y = 85, max_x = conf->p_v_res()-250;

  if (pos == -1)
    render->current.add(new RObj(x-5, y-3, max_x+10, 25+6, 0, 0, 0, 215, 2));

  std::string sw = search_w;

  std::string s = gettext("Search: ");
  int x_size = string_format::calculate_string_width(s, "Vera/17");
  string_format::format_to_size(sw, "Vera/17", max_x-x_size, true);

  render->current.add(new TObj(s + sw, "Vera/17", x, y, themes->search_font1, 
			       themes->search_font2, themes->search_font3, 3));

  render->current.add(new TObj(search_module, "Vera/17", x+max_x+25, y, themes->search_font1, 
			       themes->search_font2, themes->search_font3, 3));

  y += 50;

  if (results.size() > 0) {

    int buttom = 90;
  
    if (conf->p_h_res() == 405)
      buttom = 65;

    bool direction_down = true;
    int space = 63;
    int max_space = conf->p_h_res()-(y+buttom);
    int divert_pos_down = 0, divert_pos_up = 0;
    int cur_pos = 0, nr_elements = 0;
    bool direction_up_ok = true, direction_down_ok = true;
    bool down_failed = false, up_failed = false;

    // the value (bool) is dummy, we just need a fast lookup mechanism
    std::map<std::string, bool> available_modules;
    if (pos != -1)
      available_modules.insert(std::make_pair(vector_lookup(results, pos).module, true));
    else
      available_modules.insert(std::make_pair(vector_lookup(results, 0).module, true));

    std::string module;

    int last_add;
    
    while (true) {

      if (!direction_down) {
	cur_pos = pos + divert_pos_up + 1;
	if (cur_pos <= results.size() - 1) {
	  module = vector_lookup(results, cur_pos).module;
	  if (available_modules.find(module) == available_modules.end()) {
	    space += 63;
	    available_modules.insert(std::make_pair(module, true));
	  } else {
	    space += 28;
	  }

	  if (space > max_space) {
	    if (down_failed)
	      break;
	    else {
	      up_failed = true;
	      direction_down = true;
	      continue;
	    }
	  } else {
	    nr_elements += 1;
	    divert_pos_up += 1;
	  }
	} else {
	  up_failed = true;
	  direction_up_ok = false;
	}
	
	direction_down = true;
      } else {
	cur_pos = pos - divert_pos_down - 1;
	if (cur_pos >= 0) {
	  module = vector_lookup(results, cur_pos).module;
	  if (available_modules.find(module) == available_modules.end()) {
	    space += 63;
	    last_add = 63;
	    available_modules.insert(std::make_pair(module, true));
	  } else {
	    space += 28;
	    last_add = 28;
	  }

	  if (space > max_space) {
	    if (up_failed)
	      break;
	    else {
	      space -= last_add;
	      down_failed = true;
	      direction_down = false;
	      continue;
	    }
	  } else {
	    nr_elements += 1;
	    divert_pos_down += 1;
	  }
	} else {
	  down_failed = true;
	  direction_down_ok = false;
	}
	
	direction_down = false;
      }

      if (!direction_up_ok && !direction_down_ok)
	break;
    }

    cur_module = "";

    int startpos = pos - divert_pos_down;

    // pos == -1
    if (startpos < 0)
      startpos = 0;

    render->current.add(new RObj(55, y-10, conf->p_v_res()-2*55, max_space+20, 
				 themes->general_rectangle_color1, themes->general_rectangle_color2,
				 themes->general_rectangle_color3, themes->general_rectangle_color4, 1));

    result cur_element;

    if (pos != -1)
      cur_element = vector_lookup(results, pos);
    
    for (int i = startpos, end = results.size(); i < end; ++i) {

      if (i - startpos > nr_elements)
	break;

      print_element(vector_lookup(results, i), cur_element, y);

      y += 28;
    }
  }
  
  render->draw_and_release("search");
}

void Search::register_module(const SearchModule& s)
{
  if (s.name != "all") {
    modules.push_back(s);
    em.add_item(ExtraMenuItem(s.name, "", new TimeFunctorvstring<Search>(this, &Search::set_module, s.name)));
  } else
    // FIXME: complain loudly
    ;

  std::cout << "Search: registered module: " << s.name << std::endl;
}

class SearchDialogFunctor
{
 private:
  void (*fpt)(const std::string&, int, Overlay&);
  Overlay o;
  
 public:

  SearchDialogFunctor(void(*_fpt)(const std::string&, int, Overlay&))
    : o(Overlay("fixed size dialog"))
  { fpt=_fpt; };

  ~SearchDialogFunctor()
  {
    Render *render = S_Render::get_instance();
    render->image_mut.enterMutex();
    o.cleanup();
    render->image_mut.leaveMutex();
  }

  void call(const std::string& r)
  { return (*fpt)(gettext("Search: ") + r, 450, o); };
};

std::string Search::mainloop()
{
  Input input;

  print(search_word);

  std::list<Input> exit_inputs;
  Input exit_input;
  exit_input.command = "back"; exit_input.mode = "general";
  exit_inputs.push_back(exit_input);
  exit_input.command = "right"; exit_input.mode = "general";
  exit_inputs.push_back(exit_input);
  exit_input.command = "next"; exit_input.mode = "general";
  exit_inputs.push_back(exit_input);

  bool in_select_module = false;

  BusyIndicator::get_instance().idle();

  bool update_needed = true;

  while (!(input.command == "back" && input.mode == "general"))
    {
      if (pos == -1 && !in_select_module) {

	std::string old_search_word = search_word;

	input_master->set_map("search");

	ConstFunctor<Search, std::string, void> print_func(this, &Search::print);
	std::pair<bool, Input> res = input_master->device->generate_string(search_word, 
									   &print_func, exit_inputs);

	if (res.first) {
	  results.clear();
	  for (std::vector<SearchModule>::iterator i = modules.begin(), iend = modules.end(); i != iend; ++i)
	    if (search_module == i->name || search_module == "all") {
	      std::vector<std::pair<std::string, int> > temp = i->list_gen->call(search_word);
	      for (std::vector<std::pair<std::string, int> >::iterator ti = temp.begin(), tiend = temp.end(); 
		   ti != tiend; ++ti) {
		result r;
		r.module = i->name;
		r.name = ti->first;
		r.id = ti->second;
		results.push_back(r);
	      }
	    }

	  if (results.size() > 0) {
	    input_master->set_map("default");
	    pos = 0;
	  } else {
	    DialogWaitPrint pdialog(gettext("No search results found"), 2000);
	    continue;
	  }

	} else {

	  if (res.second.command == "exit") {
	    search_word = old_search_word;
	    input.command = "back";
	    input.mode = "general";
	  } else if (res.second.command == "right") {
	    in_select_module = true;
	    continue;
	  } else if (res.second.command == "next" && results.size() > 0) {
	    input_master->set_map("default");
	    pos = 0;
	    continue;
	  } else 
	    input = res.second;
	}

	continue;

      } else if (pos == -1 && in_select_module) {
	
	em.mainloop();

	BusyIndicator::get_instance().idle();

	print(search_word);
	in_select_module = false;
	continue;
	  
      } else {
	if (update_needed)
	  print(search_word);

	update_needed = true;
      }

      input = input_master->device->get_input();

      if (input.command == "prev")
	{
	  if (pos != -1)
	    --pos;

	  // if we go up again we need to draw again since otherwise the first
	  // element will be marked
	  if (pos == -1)
	    print(search_word);
 	}
      else if (input.command == "next")
 	{
	  if (pos != results.size()-1)
	    ++pos;
 	}
      else if (input.command == "action")
	{
	  result cur = vector_lookup(results, pos);
	  for (std::vector<SearchModule>::iterator i = modules.begin(), iend = modules.end(); i != iend; ++i)
	    if (cur.module == i->name) {
	      i->enter_func->call(cur.id);
	      break;
	    }
	}
      else
	update_needed = !global->check_commands(input, true);
    }

  return "";
}
