// used for #ifdef
#include "config.h"

#include "startmenu.hpp"

//Input
#include "input.hpp"

#include "gettext.hpp"
#include "renderer.hpp"
#include "config.hpp"
#include "graphics.hpp"
#include "updater.hpp"
#include "plugins.hpp"
#include "input.hpp"
#include "busy_indicator.hpp"
#include "shutdown.hpp"
#include "extra_menu.hpp"

#include "theme.hpp"
#include "search.hpp"

#ifdef use_inotify
#include "notify.hpp"
#endif

// ostringstream
#include <sstream>

#include <iostream>

using std::list;
using std::string;
using std::vector;

#define mms_debug 1

startmenu_item::startmenu_item(const string& n, const string& a, const string& p, int i, const VoidFunctor& f)
  : name(n), action(a), pic(p), id(i), func(f)
{
}

void Startmenu::StartmenuOpts::set_options()
{
  // header
  header = gettext("Options");

  // values
  vector<string> empty_values;

  // global options
  g_o = new Option(true, gettext("Global Options"), "Global Options", 0, 
		   empty_values, empty_values);
  val.push_back(g_o);

  val.push_back(go->dir_order);

  if (conf->p_cdpath().size() > 1) 
    val.push_back(go->cd_device);

  if (go->audio_device->values.size() > 1) 
    val.push_back(go->audio_device);

  Plugins *plugins = S_Plugins::get_instance();
  for(vector<FeaturePlugin*>::const_iterator iter=plugins->fp_data.begin(); iter!=plugins->fp_data.end(); ++iter)
    if ((*iter)->module && (*iter)->module->get_opts()) {

      (*iter)->module->get_opts()->load();

      std::vector<Option*> plugin_values = (*iter)->module->get_opts()->values();

      // FIXME: little weak check since not everyone has both sorting and reload
      if (plugin_values.size() < 3)
	continue;
      
      val.push_back(new Option (true, (*iter)->module->get_opts()->get_header(), 
				(*iter)->module->get_opts()->get_english_header(), 
				0, empty_values, empty_values));

      for (unsigned int i = 0; i < plugin_values.size(); ++i) {
	string name = plugin_values[i]->name;
        if (name != gettext("reload") && name != gettext("sorting") && 
	     name != gettext("cd device") && name != gettext("audio device"))
	    val.push_back(plugin_values[i]);
      }
    }

  load();
}

Startmenu::StartmenuOpts::~StartmenuOpts() 
{
  // global
  delete g_o;
}

void Startmenu::StartmenuOpts::save() 
{
  // intentionally left blank
}

void Startmenu::StartmenuOpts::load()
{
  // intentionally left blank
}

void Startmenu::print_lcd_menu(const startmenu_row& row, const startmenu_item& current_item)
{
  if (!global->lcd_output_possible())
    return;
  
  global->lcd_add_output("     Startmenu");
  global->lcd_add_output("");
  global->lcd_add_output(string_format::pretty_print_string_copy(row.name + ":", 20));
  global->lcd_add_output(string_format::pretty_print_string_copy(current_item.name, 20));
  global->lcd_print();
}

string Startmenu::mainloop()
{
  InputMaster *im = S_InputMaster::get_instance();

  Input input;

  im->set_map("default");

  list<startmenu_row>::iterator startmenu_iter = startmenu.begin();
  list<startmenu_item>::iterator startmenu_iter_row = startmenu_iter->items.begin();

  BusyIndicator &busy_indicator = BusyIndicator::get_instance();

  Plugins *plugins = S_Plugins::get_instance();
  for(vector<FeaturePlugin*>::const_iterator iter=plugins->fp_data.begin(); 
      iter!=plugins->fp_data.end(); ++iter)
    (*iter)->module->startup_updater();

  opts.set_options();

  // updater
  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

  screen_updater->start();
  screen_updater->activate();

  BackgroundUpdater *background_updater = S_BackgroundUpdater::get_instance();

  background_updater->start();
  background_updater->activate();

  Shutdown *sd = S_Shutdown::get_instance();

#ifdef use_inotify
  S_Notify::get_instance()->start();
#endif

  bool update_needed = true;

  while (!(input.command == "action" && startmenu_iter_row->action == "quit"))
    {
      if (update_needed) {
	print(*startmenu_iter, *startmenu_iter_row);
	print_lcd_menu(*startmenu_iter, *startmenu_iter_row);
      }

      busy_indicator.idle();

      input = im->device->get_input();

      busy_indicator.busy();
      
      if (sd->is_enabled()) {
      	sd->cancel();
      	continue;
      }

      update_needed = true;

      if (input.command == "prev")
 	{
 	  if (startmenu_iter != startmenu.begin()) {
	    --startmenu_iter;
	    startmenu_iter_row = startmenu_iter->items.begin();
	  } 
 	}
      else if (input.command == "next")
 	{
 	  list<startmenu_row>::iterator j = startmenu.end(); --j;
 	  if (startmenu_iter != j) {
 	    ++startmenu_iter;
	    startmenu_iter_row = startmenu_iter->items.begin();
	  } 
 	}
      else if (input.command == "left")
 	{
 	  if (startmenu_iter_row != startmenu_iter->items.begin())
 	    --startmenu_iter_row;
 	}
      else if (input.command == "right")
 	{
 	  list<startmenu_item>::iterator j = startmenu_iter->items.end(); --j;
 	  if (startmenu_iter_row != j)
 	    ++startmenu_iter_row;
 	}
      else if (input.command == "second_action")
	{
	  // FIXME
#if 0
	  // make back work for playlist commands
	  input_master->cur_map = &input_master->standard;

	  ExtraMenu em;

	  em.add_item(ExtraMenuItem(gettext("Playlist commands"), 
				    input_master->find_shortcut("playlist_commands"), 
				    new TimeFunctorv<Audio>(Audio::get_instance(), &Audio::playlist_commands)));

	  em.mainloop();

	  // reset
	  input_master->cur_map = &input_master->startmenu;
#endif
	}
      else if (input.command == "action" && startmenu_iter_row->action != "quit")
 	{
#ifdef mms_debug
	  cerr << " --- CALLING FEATURE PLUGIN MAINLOOP --- " << endl;
#endif
	  for (list<startmenu_row>::const_iterator row_iter = startmenu.begin(); row_iter != startmenu.end(); ++row_iter)
	    for (list<startmenu_item>::const_iterator iter2 = row_iter->items.begin(); iter2 != row_iter->items.end(); ++iter2) {
	      if (iter2->action == startmenu_iter_row->action) {
#ifdef mms_debug
		cerr << "Startmenu: Enter plugin action " << startmenu_iter_row->name << endl;
#endif
		iter2->func.call();

		// set the input map back to startmenu
		im->set_map("default");
		break;
	      }
	    }
	}
      else
	// FIXME: ?
	// make back work for playlist commands
	//im->cur_map = &im->standard;
	update_needed = !global->check_commands(input);
    }
  return "quit";
}

void Startmenu::print(const startmenu_row& row, const startmenu_item& current_item)
{
  Render *render = S_Render::get_instance();
  Themes *themes = S_Themes::get_instance();

  render->prepare_new_image();

  render->current.add(new PObj(themes->startmenu_background, 0, 0, 0, SCALE_FULL));

  int y = 120;
  int x = 0;

  if (conf->p_h_res() == 405)
    y = 40;
  
  for (list<startmenu_row>::const_iterator iter = startmenu.begin(); iter != startmenu.end(); ++iter)
    {
      if (iter->name == row.name) {
	for (list<startmenu_item>::const_iterator iter2 = iter->items.begin(); iter2 != iter->items.end(); ++iter2) {
	  if (current_item.name == iter2->name) {
	    
	    render->current.add(new PObj("startmenu/marked-left.png", 65, (y-2), 1, NOSCALING));

	    if (x == 70)
	      render->current.add(new PObj("startmenu/marked2.png", 90, (y-2), conf->p_v_res()-220+x-50, 1));
	    else if (x == 140)
	      render->current.add(new PObj("startmenu/marked3.png", 90, (y-2), conf->p_v_res()-220+x-50, 1));
	    else
	      render->current.add(new PObj("startmenu/marked.png", 90, (y-2), conf->p_v_res()-220+x-50, 1));
	    
	    render->current.add(new PObj("startmenu/marked-right.png", 90+conf->p_v_res()-220+x-50, (y-2), 1, NOSCALING));

	    render->current.add(new TObj(iter2->name, "Vera/22", 75, y+5, 
					 themes->startmenu_font1, themes->startmenu_font2, 
					 themes->startmenu_font3, 2));
	  }
	  render->current.add(new PObj(iter2->pic, conf->p_v_res()-220+x, y, 2, NOSCALING));
	  x += 70;
	}
      } else {
	render->current.add(new TObj(iter->name, "Vera/22", 75, y+5, 
				     themes->startmenu_font1, themes->startmenu_font2, 
				     themes->startmenu_font3, 2));
	for (list<startmenu_item>::const_iterator iter2 = iter->items.begin(); iter2 != iter->items.end(); ++iter2) {
	  render->current.add(new PObj(iter2->pic, conf->p_v_res()-220+x, y, 2, NOSCALING));
	  x += 70;
	}
      }
      x = 0;
      y += 51;
    }

  // render the logo in the startmenu
  render->current.add(new PObj("logo.png", conf->p_v_res()-85, conf->p_h_res()-41, 2, NOSCALING));

  render->draw_and_release("startmenu");
}

Startmenu::Startmenu()
{
  generate_startmenu();
  conf = S_Config::get_instance();
  global = S_Global::get_instance();
}

void search_func()
{
  S_Search::get_instance()->mainloop();
}

void options_func()
{
  S_Startmenu::get_instance()->opts.mainloop();
}

void open_close_func()
{
  S_Cd::get_instance()->openclose_cddrive();
}

void themes_func()
{
  S_Theme::get_instance()->mainloop();
}

void quit_func()
{}

void Startmenu::generate_startmenu()
{
  Themes *themes = S_Themes::get_instance();
  Config *conf = S_Config::get_instance();

  // generate menues.
  list<startmenu_row> start_menu;
  startmenu_row row;

/*
  Category stored in plugin_name() is what menu displays when unselected
  ie Audio, Video, Midi, Picture, EPG, TV, NET...
  priority was for column dynamic allocation (0..2) (unused)
*/

  Plugins *plugins = S_Plugins::get_instance();
  for(vector<FeaturePlugin*>::const_iterator iter=plugins->fp_data.begin(); iter!=plugins->fp_data.end(); iter++) {
    if ((*iter)->module->init()) {
#ifdef mms_debug
      cerr << " --- REGISTER PLUGIN FEATURES for " << (*iter)->plugin_name() << endl;
#endif
      row.name = (*iter)->plugin_name();
      row.items=(*iter)->items();
      start_menu.push_back(row);
    } else
      cerr << gettext("Plugin can't init: ") << (*iter)->filename << endl;
  }

  row.items.clear();

  // search button
  row.items.push_back(startmenu_item(gettext("Search"), "search", 
				     themes->general_search, 0, VoidFunctor(&search_func)));

  // options button
  row.items.push_back(startmenu_item(gettext("Options"), "options", 
				     themes->startmenu_options, 0, VoidFunctor(&options_func)));

  // themes button
  row.items.push_back(startmenu_item(gettext("Themes"), "themes", 
                                     themes->startmenu_themes, 1, VoidFunctor(&themes_func)));

  
  // Open/Close Tray button
  if (conf->p_openclose())
    row.items.push_back(startmenu_item(gettext("Open/Close Tray"), "openclose", "openclose.png", 2, VoidFunctor(&open_close_func)));

  if (row.items.size() != 0) {
    row.name = gettext("Search");
    start_menu.push_back(row);
    row.items.clear();
  }

  // quit button
  row.name = gettext("Quit");
  row.items.push_back(startmenu_item(gettext("Quit"), "quit", 
				     themes->startmenu_quit, 0, VoidFunctor(&quit_func)));

  start_menu.push_back(row);
  row.items.clear();

  startmenu = start_menu;
}
