#include "updater.hpp"

// time 
#include <time.h>

#include <string>

#include <iostream>

using std::string;
using std::list;

Updater::Updater() 
  : progressbar(Overlay("progressbar")), status_progressbar(0), 
    total_progressbar(0), header(""), running(false)
{}

void Updater::activate()
{
  running = true;
}

void Updater::deactivate()
{
  running = false;
}

void Updater::run()
{
  set_nicelevel();

  while (true) {
    while (!running)
      sleep(500); // ms

    timer.elements_mut.enterMutex();

    gettimeofday(&timer.current_time, NULL);

    long int next_timeout = 1000;
    long int min_timeout = 250; // at least sleep 100 ms

    for (list<TimeElement>::iterator iter = timer.elements.begin(), 
	   end = timer.elements.end(); iter != end; ++iter)
      if (iter->active) {

	struct timeval diff = diff_timeval(iter->next_time, timer.current_time);

	if (diff.tv_sec < 0 || diff.tv_usec < 0) {
	  iter->call_back_f->call();
	  iter->next_time = timer.set_timeout(timer.current_time, iter->time_f->call());
	} 
	  
	next_timeout = std::min(next_timeout, 
				(iter->next_time.tv_sec-timer.current_time.tv_sec)*1000+
				(iter->next_time.tv_usec-timer.current_time.tv_usec)/1000);
      }

    if (next_timeout < min_timeout)
      next_timeout = min_timeout;

    timer.elements_mut.leaveMutex();

    sleep(next_timeout); // ms
  }
}

struct timeval Updater::diff_timeval(const struct timeval& t1, const struct timeval& t2) 
{
  struct timeval diff;

  if (t1.tv_usec > t2.tv_usec){
    diff.tv_usec = t1.tv_usec - t2.tv_usec;
    diff.tv_sec = t1.tv_sec - t2.tv_sec;
  } else {
    diff.tv_usec = t1.tv_usec + 1000000 - t2.tv_usec;
    diff.tv_sec = t1.tv_sec - t2.tv_sec - 1;
  }

  return diff;
}

struct timeval Updater::Timer::set_timeout(const struct timeval& t, int wait)
{
  struct timeval temp;
  temp.tv_sec = t.tv_sec + wait/1000;
  temp.tv_usec = t.tv_usec + (wait - ((wait/1000)*1000)) * 1000;
  if (temp.tv_usec > 1000*1000) {
    temp.tv_sec += 1;
    temp.tv_usec -= 1000*1000;
  }
  return temp;
}

void Updater::Timer::add(TimeElement te)
{
  elements_mut.enterMutex();

  gettimeofday(&current_time, NULL);
  te.next_time = set_timeout(current_time, te.time_f->call());
  elements.push_back(te);

  elements_mut.leaveMutex();
}

void Updater::Timer::del(string name)
{
  elements_mut.enterMutex();

  for (list<TimeElement>::iterator iter = elements.begin(), end = elements.end();
       iter != end; ++iter)
    if (iter->name == name) {
      elements.erase(iter);
      break;
    }

  elements_mut.leaveMutex();
}

void Updater::Timer::activate(const string& name)
{
  elements_mut.enterMutex();

  for (list<TimeElement>::iterator iter = elements.begin(), end = elements.end();
       iter != end; ++iter)
    if (iter->name == name) {
      gettimeofday(&current_time, NULL);
      iter->next_time.tv_sec = current_time.tv_sec + iter->time_f->call()/1000;
      iter->next_time.tv_usec = current_time.tv_usec + (iter->time_f->call() - (iter->next_time.tv_sec*1000)) * 1000;
      iter->active = true;
      break;
    }

  elements_mut.leaveMutex();
}

void Updater::Timer::deactivate(const string& name)
{
  elements_mut.enterMutex();

  for (list<TimeElement>::iterator iter = elements.begin(), end = elements.end();
       iter != end; ++iter)
    if (iter->name == name) {
      iter->active = false;
      break;
    }

  elements_mut.leaveMutex();
}

void Updater::Timer::run_and_deactivate(const string& name)
{
  elements_mut.enterMutex();

  for (list<TimeElement>::iterator iter = elements.begin(), end = elements.end();
       iter != end; ++iter)
    if (iter->name == name) {
      iter->call_back_f->call();
      iter->active = false;
      break;
    }

  elements_mut.leaveMutex();
}

bool Updater::Timer::status(const string& name)
{
  elements_mut.enterMutex();

  bool active_element = false;
  for (list<TimeElement>::iterator iter = elements.begin(), end = elements.end();
       iter != end; ++iter)
    if (iter->name == name) {
      active_element = iter->active;
      break;
    }

  elements_mut.leaveMutex();

  return active_element;
}

// trigger

void Updater::Trigger::add(const TriggerElement& te)
{
  elements_mut.enterMutex();

  elements.push_back(te);

  elements_mut.leaveMutex();
}

void Updater::Trigger::del(const string& name)
{
  elements_mut.enterMutex();

  for (list<TriggerElement>::iterator iter = elements.begin(), end = elements.end();
       iter != end; ++iter)
    if (iter->name == name) {
      if (iter->clean_up != 0)
	iter->clean_up->call();
      elements.erase(iter);
      break;
    }

  elements_mut.leaveMutex();
}

void Updater::Trigger::trigger_input(const string& trigger)
{
  elements_mut.enterMutex();

  for (list<TriggerElement>::iterator iter = elements.begin(), end = elements.end();
       iter != end; ++iter) {
    if (iter->trigger.size() == 1 && iter->trigger.front().empty()) { // trigger
								      // on all input
      iter->trigger_func->call();
    } else {
      for (list<string>::iterator i = iter->trigger.begin(), iend = iter->trigger.end(); 
	   i != iend; ++i) {
	if (*i == trigger) {
	  iter->trigger_func->call();
	  break;
	}
      }
    }
  }

  elements_mut.leaveMutex();
}

void BackgroundUpdater::set_nicelevel()
{
  Config *conf = S_Config::get_instance();

  if (conf->p_priority_change()) 
    nice(10); //Lower priority to keep audio from skipping
}

void ScreenUpdater::set_nicelevel()
{
  Config *conf = S_Config::get_instance();

  if (conf->p_priority_change()) 
    nice(3); //Lower priority to keep audio from skipping
}
